﻿using System;
using System.Collections.Generic;
using System.Linq;
using Michigan.Ods.Sdk.Helpers;
using Michigan.Ods.Sdk.Models;
using RestSharp;
  using System.Net;
  using System.Text;
  
namespace Michigan.Ods.Sdk.Api 
{
    public class EemEducationalEntityMasterDetailsApi 
    {
        private readonly IRestClient client;

        public EemEducationalEntityMasterDetailsApi(IRestClient client)
        {
            this.client = client;
        }
      
        /// <summary>
        /// Retrieves resources based with paging capabilities (using the &quot;Get All&quot; pattern). This GET operation provides access to resources using the &quot;Get All&quot; pattern. In this version of the API there is support for paging.
        /// </summary>
        /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
        /// <param name="minChangeVersion">Used in synchronization to set sequence minimum ChangeVersion</param>
        /// <param name="maxChangeVersion">Used in synchronization to set sequence maximum ChangeVersion</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<List<EemEducationalEntityMasterDetail>> GetEEMEducationalEntityMasterDetailsAll(int? offset= null, int? limit= null, int? minChangeVersion= null, int? maxChangeVersion= null) 
        {
            var request = new RestRequest("/eemEducationalEntityMasterDetails", Method.GET);
            request.RequestFormat = DataFormat.Json;

            if (offset != null)
                request.AddParameter("offset", offset);
            if (limit != null)
                request.AddParameter("limit", limit);
            if (minChangeVersion != null)
                request.AddParameter("minChangeVersion", minChangeVersion);
            if (maxChangeVersion != null)
                request.AddParameter("maxChangeVersion", maxChangeVersion);
            request.AddHeader("Accept", "application/json");
            var response = client.Execute<List<EemEducationalEntityMasterDetail>>(request);

            return response;
        }
        /// <summary>
        /// Retrieves resources matching values of an example resource (using the &quot;Get By Example&quot; pattern). This GET operation provides access to resources using the &quot;Get by Example&quot; search pattern.  The values of any properties of the resource that are specified will be used to return all matching results (if it exists).
        /// </summary>
        /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
        /// <param name="minChangeVersion">Used in synchronization to set sequence minimum ChangeVersion</param>
        /// <param name="maxChangeVersion">Used in synchronization to set sequence maximum ChangeVersion</param>
        /// <param name="eemEducationalEntityMasterId">Educational Entity Master identifier from EEM system.</param>
        /// <param name="eemAncillaryFunctionDescriptor">Indicates the non-instructional functions designated by the EEM authorized user for this entity.</param>
        /// <param name="eemEarlyMiddleCollegeTypeDescriptor">Indicates the Early Middle College type for which this Educational Entity has to allow a pupil to earn a high school diploma and an associate degree or up to two years of transferable college credits.</param>
        /// <param name="eemLocaleDescriptor">Indicates the measure of an EEM School location relative to populous areas.</param>
        /// <param name="eemMagnetDescriptor">Denotes whether or not an Educational Entity is a magnet school.</param>
        /// <param name="eemOwnershipDescriptor">Describes the type of Ownership of an Educational Entity.</param>
        /// <param name="eemReligiousOrientationDescriptor">Reflects the list of organized groups of religious congregations.</param>
        /// <param name="eemseeSubTypeDescriptor">Indicates the sub type of the EEM Shared Educational Entity (SEE) participation type.</param>
        /// <param name="eemseeTypeDescriptor">Indicates the EEM Shared Educational Entity (SEE) participation type.</param>
        /// <param name="effectiveDate">NEED DESCRIPTION.</param>
        /// <param name="expirationDate">NEED DESCRIPTION.</param>
        /// <param name="isHeadCount">1 = Is headcount; 0 or NULL = is not headcount;</param>
        /// <param name="isSchoolOfChoiceOffered">1 = School of Choice is Offered; 0 or NULL = School of Choice is not Offered;</param>
        /// <param name="isSharedEntity">1 = is shared entity; 0 or NULL = is not shared entity;</param>
        /// <param name="eemAgreementNumber">The 9-digit agreement number assigned by the EEM system. Formerly referred to as the recipient code. Used by the Budget and Grants Office of the Michigan Department of Education.</param>
        /// <param name="eemremcId">EEM Regional Educational Media Center identifier.</param>
        /// <param name="eemCharteringAgencyCode">Provided by MDE. The ten position code number of the agency that charters a Public School Academy.</param>
        /// <param name="eemCharteringAgencyEntityId">The EEM identifier of the agency that charters a Public School Academy.</param>
        /// <param name="eemactTestCenter">Denotes the ACT Test Center of the Educational Entity.</param>
        /// <param name="eemsatTestCenter">Denotes the SAT Test Center for the Educational Entity.</param>
        /// <param name="eemReceivesTransportationServicesFrom">Denotes from where Educational Entity receives transportation services.</param>
        /// <param name="id"></param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<List<EemEducationalEntityMasterDetail>> GetEEMEducationalEntityMasterDetailsByExample(int? offset= null, int? limit= null, int? minChangeVersion= null, int? maxChangeVersion= null, int? eemEducationalEntityMasterId= null, string eemAncillaryFunctionDescriptor= null, string eemEarlyMiddleCollegeTypeDescriptor= null, string eemLocaleDescriptor= null, string eemMagnetDescriptor= null, string eemOwnershipDescriptor= null, string eemReligiousOrientationDescriptor= null, string eemseeSubTypeDescriptor= null, string eemseeTypeDescriptor= null, DateTime? effectiveDate= null, DateTime? expirationDate= null, bool? isHeadCount= null, bool? isSchoolOfChoiceOffered= null, bool? isSharedEntity= null, string eemAgreementNumber= null, string eemremcId= null, string eemCharteringAgencyCode= null, int? eemCharteringAgencyEntityId= null, string eemactTestCenter= null, string eemsatTestCenter= null, int? eemReceivesTransportationServicesFrom= null, string id= null) 
        {
            var request = new RestRequest("/eemEducationalEntityMasterDetails", Method.GET);
            request.RequestFormat = DataFormat.Json;

            if (offset != null)
                request.AddParameter("offset", offset);
            if (limit != null)
                request.AddParameter("limit", limit);
            if (minChangeVersion != null)
                request.AddParameter("minChangeVersion", minChangeVersion);
            if (maxChangeVersion != null)
                request.AddParameter("maxChangeVersion", maxChangeVersion);
            if (eemEducationalEntityMasterId != null)
                request.AddParameter("eemEducationalEntityMasterId", eemEducationalEntityMasterId);
            if (eemAncillaryFunctionDescriptor != null)
                request.AddParameter("eemAncillaryFunctionDescriptor", eemAncillaryFunctionDescriptor);
            if (eemEarlyMiddleCollegeTypeDescriptor != null)
                request.AddParameter("eemEarlyMiddleCollegeTypeDescriptor", eemEarlyMiddleCollegeTypeDescriptor);
            if (eemLocaleDescriptor != null)
                request.AddParameter("eemLocaleDescriptor", eemLocaleDescriptor);
            if (eemMagnetDescriptor != null)
                request.AddParameter("eemMagnetDescriptor", eemMagnetDescriptor);
            if (eemOwnershipDescriptor != null)
                request.AddParameter("eemOwnershipDescriptor", eemOwnershipDescriptor);
            if (eemReligiousOrientationDescriptor != null)
                request.AddParameter("eemReligiousOrientationDescriptor", eemReligiousOrientationDescriptor);
            if (eemseeSubTypeDescriptor != null)
                request.AddParameter("eemseeSubTypeDescriptor", eemseeSubTypeDescriptor);
            if (eemseeTypeDescriptor != null)
                request.AddParameter("eemseeTypeDescriptor", eemseeTypeDescriptor);
            if (effectiveDate != null)
                request.AddParameter("effectiveDate", effectiveDate);
            if (expirationDate != null)
                request.AddParameter("expirationDate", expirationDate);
            if (isHeadCount != null)
                request.AddParameter("isHeadCount", isHeadCount);
            if (isSchoolOfChoiceOffered != null)
                request.AddParameter("isSchoolOfChoiceOffered", isSchoolOfChoiceOffered);
            if (isSharedEntity != null)
                request.AddParameter("isSharedEntity", isSharedEntity);
            if (eemAgreementNumber != null)
                request.AddParameter("eemAgreementNumber", eemAgreementNumber);
            if (eemremcId != null)
                request.AddParameter("eemremcId", eemremcId);
            if (eemCharteringAgencyCode != null)
                request.AddParameter("eemCharteringAgencyCode", eemCharteringAgencyCode);
            if (eemCharteringAgencyEntityId != null)
                request.AddParameter("eemCharteringAgencyEntityId", eemCharteringAgencyEntityId);
            if (eemactTestCenter != null)
                request.AddParameter("eemactTestCenter", eemactTestCenter);
            if (eemsatTestCenter != null)
                request.AddParameter("eemsatTestCenter", eemsatTestCenter);
            if (eemReceivesTransportationServicesFrom != null)
                request.AddParameter("eemReceivesTransportationServicesFrom", eemReceivesTransportationServicesFrom);
            if (id != null)
                request.AddParameter("id", id);
            request.AddHeader("Accept", "application/json");
            var response = client.Execute<List<EemEducationalEntityMasterDetail>>(request);

            return response;
        }
        /// <summary>
        /// Retrieves a specific resource using the values of the resource's natural key (using the &quot;Get By Key&quot; pattern). This GET operation provides access to resources using the &quot;Get by Key&quot; search pattern. The values of the natural key of the resource must be fully specified, and the service will return the matching result (if it exists).
        /// </summary>
        /// <param name="eemEducationalEntityMasterId">Educational Entity Master identifier from EEM system.</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<EemEducationalEntityMasterDetail> GetEEMEducationalEntityMasterDetailByKey(int? eemEducationalEntityMasterId, string IfNoneMatch= null) 
        {
            var request = new RestRequest("/eemEducationalEntityMasterDetails", Method.GET);
            request.RequestFormat = DataFormat.Json;

            // verify required params are set
            if (eemEducationalEntityMasterId == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            if (eemEducationalEntityMasterId != null)
                request.AddParameter("eemEducationalEntityMasterId", eemEducationalEntityMasterId);
            request.AddHeader("If-None-Match", IfNoneMatch);
            request.AddHeader("Accept", "application/json");
            var response = client.Execute<EemEducationalEntityMasterDetail>(request);

            return response;
        }
        /// <summary>
        /// Creates or updates resources based on the natural key values of the supplied resource. The POST operation can be used to create or update resources. In database terms, this is often referred to as an &quot;upsert&quot; operation (insert + update).  Clients should NOT include the resource &quot;id&quot; in the JSON body because it will result in an error (you must use a PUT operation to update a resource by &quot;id&quot;). The web service will identify whether the resource already exists based on the natural key values provided, and update or create the resource appropriately.
        /// </summary>
        /// <param name="body">The JSON representation of the &quot;eemEducationalEntityMasterDetail&quot; resource to be created or updated.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse PostEEMEducationalEntityMasterDetails(EemEducationalEntityMasterDetail body) 
        {
            var request = new RestRequest("/eemEducationalEntityMasterDetails", Method.POST);
            request.RequestFormat = DataFormat.Json;

            // verify required params are set
            if (body == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            request.AddBody(body);
            request.Parameters.First(param => param.Type == ParameterType.RequestBody).Name = "application/json";
            var response = client.Execute(request);

            var location = response.Headers.FirstOrDefault(x => x.Name == "Location");

            if (location != null && !string.IsNullOrWhiteSpace(location.Value.ToString()))
                body.id = location.Value.ToString().Split('/').Last();
            return response;
        }
        /// <summary>
        /// Retrieves a specific resource using the resource's identifier (using the &quot;Get By Id&quot; pattern). This GET operation retrieves a resource by the specified resource identifier.
        /// </summary>
        /// <param name="id">A resource identifier specifying the resource to be retrieved.</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<EemEducationalEntityMasterDetail> GetEEMEducationalEntityMasterDetailsById(string id, string IfNoneMatch= null) 
        {
            var request = new RestRequest("/eemEducationalEntityMasterDetails/{id}", Method.GET);
            request.RequestFormat = DataFormat.Json;

            request.AddUrlSegment("id", id);
            // verify required params are set
            if (id == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            request.AddHeader("If-None-Match", IfNoneMatch);
            request.AddHeader("Accept", "application/json");
            var response = client.Execute<EemEducationalEntityMasterDetail>(request);

            return response;
        }
        /// <summary>
        /// Updates or creates a resource based on the resource identifier. The PUT operation is used to update or create a resource by identifier.  If the resource doesn't exist, the resource will be created using that identifier.  Additionally, natural key values cannot be changed using this operation, and will not be modified in the database.  If the resource &quot;id&quot; is provided in the JSON body, it will be ignored as well.
        /// </summary>
        /// <param name="id">A resource identifier specifying the resource to be updated.</param>
        /// <param name="IfMatch">The ETag header value used to prevent the PUT from updating a resource modified by another consumer.</param>
        /// <param name="body">The JSON representation of the &quot;eemEducationalEntityMasterDetail&quot; resource to be updated.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse PutEEMEducationalEntityMasterDetail(string id, string IfMatch, EemEducationalEntityMasterDetail body) 
        {
            var request = new RestRequest("/eemEducationalEntityMasterDetails/{id}", Method.PUT);
            request.RequestFormat = DataFormat.Json;

            request.AddUrlSegment("id", id);
            // verify required params are set
            if (id == null || body == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            request.AddHeader("If-Match", IfMatch);
            request.AddBody(body);
            request.Parameters.First(param => param.Type == ParameterType.RequestBody).Name = "application/json";
            var response = client.Execute(request);

            var location = response.Headers.FirstOrDefault(x => x.Name == "Location");

            if (location != null && !string.IsNullOrWhiteSpace(location.Value.ToString()))
                body.id = location.Value.ToString().Split('/').Last();
            return response;
        }
        /// <summary>
        /// Deletes an existing resource using the resource identifier. The DELETE operation is used to delete an existing resource by identifier.  If the resource doesn't exist, an error will result (the resource will not be found).
        /// </summary>
        /// <param name="id">A resource identifier specifying the resource to be deleted.</param>
        /// <param name="IfMatch">The ETag header value used to prevent the DELETE from removing a resource modified by another consumer.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse DeleteEEMEducationalEntityMasterDetailById(string id, string IfMatch= null) 
        {
            var request = new RestRequest("/eemEducationalEntityMasterDetails/{id}", Method.DELETE);
            request.RequestFormat = DataFormat.Json;

            request.AddUrlSegment("id", id);
            // verify required params are set
            if (id == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            request.AddHeader("If-Match", IfMatch);
            var response = client.Execute(request);

            return response;
        }
        /// <summary>
        ///Retrieves a specific resource using the values of the resource's natural key (using the &quot;Get By Key&quot; pattern). This GET operation provides access to resources using the &quot;Get by Key&quot; search pattern. The values of the natural key of the resource must be fully specified, and the service will return the matching result (if it exists).
        ///</summary>
        /// <param name="eemEducationalEntityMasterId">Educational Entity Master identifier from EEM system.</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        ///<returns></returns>
        public IRestResponse PatchGetEEMEducationalEntityMasterDetailByKey(int? eemEducationalEntityMasterId, string IfNoneMatch, EemEducationalEntityMasterDetail patchEemEducationalEntityMasterDetail)
        {
            // verify required params are set
            if (eemEducationalEntityMasterId == null ) {
                throw new ArgumentException("API method call is missing required parameters");
            }
            var sdkUsageRecord = new SdkUsageRecord();
            var jsonSerializer = new RestSharp.Serializers.JsonSerializer();
            var statisticsTracker = new StatisticsTracker();

            var getResponse = this.GetEEMEducationalEntityMasterDetailByKey( eemEducationalEntityMasterId, IfNoneMatch);

            if (!(getResponse.StatusCode == HttpStatusCode.OK || getResponse.StatusCode == HttpStatusCode.Found))
                return getResponse;

            if (statisticsTracker.ShouldTrackStatistics)
            {
                sdkUsageRecord.DateTimeStamp = DateTime.Now;
                sdkUsageRecord.HttpMethod = "PatchGetEEMEducationalEntityMasterDetailByKey";
                sdkUsageRecord.Url = "/eemEducationalEntityMasterDetails";
                var serializedGetObject = jsonSerializer.Serialize(getResponse.Data);
                sdkUsageRecord.GetSizeInBytes = Encoding.UTF8.GetByteCount(serializedGetObject);
                var serializedPatchObject = jsonSerializer.Serialize(patchEemEducationalEntityMasterDetail);
                sdkUsageRecord.PatchSizeInBytes = Encoding.UTF8.GetByteCount(serializedPatchObject);
            }

            new PatchApplier().Execute(getResponse.Data, patchEemEducationalEntityMasterDetail);

            var putResponse = PutEEMEducationalEntityMasterDetail( getResponse.Data.id, null, getResponse.Data ); 
            if (statisticsTracker.ShouldTrackStatistics)
            {
                var serializedPutObject = jsonSerializer.Serialize(getResponse.Data);
                sdkUsageRecord.PutSizeInBytes = Encoding.UTF8.GetByteCount(serializedPutObject);
                statisticsTracker.TrackIfEnabled(sdkUsageRecord);
            }
            return putResponse;
        }
        /// <summary>
        ///Retrieves a specific resource using the resource's identifier (using the &quot;Get By Id&quot; pattern). This GET operation retrieves a resource by the specified resource identifier.
        ///</summary>
        /// <param name="id">A resource identifier specifying the resource to be retrieved.</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        ///<returns></returns>
        public IRestResponse PatchGetEEMEducationalEntityMasterDetailsById(string id, string IfNoneMatch, EemEducationalEntityMasterDetail patchEemEducationalEntityMasterDetail)
        {
            // verify required params are set
            if (id == null ) {
                throw new ArgumentException("API method call is missing required parameters");
            }
            var sdkUsageRecord = new SdkUsageRecord();
            var jsonSerializer = new RestSharp.Serializers.JsonSerializer();
            var statisticsTracker = new StatisticsTracker();

            var getResponse = this.GetEEMEducationalEntityMasterDetailsById( id, IfNoneMatch);

            if (!(getResponse.StatusCode == HttpStatusCode.OK || getResponse.StatusCode == HttpStatusCode.Found))
                return getResponse;

            if (statisticsTracker.ShouldTrackStatistics)
            {
                sdkUsageRecord.DateTimeStamp = DateTime.Now;
                sdkUsageRecord.HttpMethod = "PatchGetEEMEducationalEntityMasterDetailsById";
                sdkUsageRecord.Url = "/eemEducationalEntityMasterDetails/{id}";
                var serializedGetObject = jsonSerializer.Serialize(getResponse.Data);
                sdkUsageRecord.GetSizeInBytes = Encoding.UTF8.GetByteCount(serializedGetObject);
                var serializedPatchObject = jsonSerializer.Serialize(patchEemEducationalEntityMasterDetail);
                sdkUsageRecord.PatchSizeInBytes = Encoding.UTF8.GetByteCount(serializedPatchObject);
            }

            new PatchApplier().Execute(getResponse.Data, patchEemEducationalEntityMasterDetail);

            var putResponse = PutEEMEducationalEntityMasterDetail( getResponse.Data.id, null, getResponse.Data ); 
            if (statisticsTracker.ShouldTrackStatistics)
            {
                var serializedPutObject = jsonSerializer.Serialize(getResponse.Data);
                sdkUsageRecord.PutSizeInBytes = Encoding.UTF8.GetByteCount(serializedPutObject);
                statisticsTracker.TrackIfEnabled(sdkUsageRecord);
            }
            return putResponse;
        }
        }
}

