﻿using System;
using System.Collections.Generic;
using System.Linq;
using Michigan.Ods.Sdk.Helpers;
using Michigan.Ods.Sdk.Models;
using RestSharp;
  using System.Net;
  using System.Text;
  
namespace Michigan.Ods.Sdk.Api 
{
    public class EemEducationalEntityMastersApi 
    {
        private readonly IRestClient client;

        public EemEducationalEntityMastersApi(IRestClient client)
        {
            this.client = client;
        }
      
        /// <summary>
        /// Retrieves resources based with paging capabilities (using the &quot;Get All&quot; pattern). This GET operation provides access to resources using the &quot;Get All&quot; pattern. In this version of the API there is support for paging.
        /// </summary>
        /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
        /// <param name="minChangeVersion">Used in synchronization to set sequence minimum ChangeVersion</param>
        /// <param name="maxChangeVersion">Used in synchronization to set sequence maximum ChangeVersion</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<List<EemEducationalEntityMaster>> GetEEMEducationalEntityMastersAll(int? offset= null, int? limit= null, int? minChangeVersion= null, int? maxChangeVersion= null) 
        {
            var request = new RestRequest("/eemEducationalEntityMasters", Method.GET);
            request.RequestFormat = DataFormat.Json;

            if (offset != null)
                request.AddParameter("offset", offset);
            if (limit != null)
                request.AddParameter("limit", limit);
            if (minChangeVersion != null)
                request.AddParameter("minChangeVersion", minChangeVersion);
            if (maxChangeVersion != null)
                request.AddParameter("maxChangeVersion", maxChangeVersion);
            request.AddHeader("Accept", "application/json");
            var response = client.Execute<List<EemEducationalEntityMaster>>(request);

            return response;
        }
        /// <summary>
        /// Retrieves resources matching values of an example resource (using the &quot;Get By Example&quot; pattern). This GET operation provides access to resources using the &quot;Get by Example&quot; search pattern.  The values of any properties of the resource that are specified will be used to return all matching results (if it exists).
        /// </summary>
        /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
        /// <param name="minChangeVersion">Used in synchronization to set sequence minimum ChangeVersion</param>
        /// <param name="maxChangeVersion">Used in synchronization to set sequence maximum ChangeVersion</param>
        /// <param name="eemEducationalEntityMasterId">Educational Entity Master identifier from EEM system.</param>
        /// <param name="eemCountyId">EEM County identifier.</param>
        /// <param name="geographicEEMCountyId">EEM County identifier.</param>
        /// <param name="previousDistrictEEMEducationalEntityMasterId">Educational Entity Master identifier from EEM system.</param>
        /// <param name="eemEntityTypeClassificationId">The EEM EntityType identifier.</param>
        /// <param name="eemCloseReasonDescriptor">Indicates the reason why an Educational Entity was closed.</param>
        /// <param name="eemFiscalEntityTypeDescriptor">FiscalEntityCode as indicated in EEM.</param>
        /// <param name="eemStatusDescriptor">Denotes the EEM open / close status of the entity.</param>
        /// <param name="eemEntityOfficialName">The EEM official name of the Educational Entity</param>
        /// <param name="openDate">This characteristic denotes the actual opening date of the entity or when it begins to do business. *** needs to be datetime ***</param>
        /// <param name="closeDate">This characteristic is the actual closing date of the entity or the date the entity ceased to do business. *** needs to be datetime ***</param>
        /// <param name="earlyMiddleCollegeFlag">Indicates the education organization provides both a high school education as well as up to 60 college credits.</param>
        /// <param name="id"></param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<List<EemEducationalEntityMaster>> GetEEMEducationalEntityMastersByExample(int? offset= null, int? limit= null, int? minChangeVersion= null, int? maxChangeVersion= null, int? eemEducationalEntityMasterId= null, int? eemCountyId= null, int? geographicEEMCountyId= null, int? previousDistrictEEMEducationalEntityMasterId= null, int? eemEntityTypeClassificationId= null, string eemCloseReasonDescriptor= null, string eemFiscalEntityTypeDescriptor= null, string eemStatusDescriptor= null, string eemEntityOfficialName= null, DateTime? openDate= null, DateTime? closeDate= null, bool? earlyMiddleCollegeFlag= null, string id= null) 
        {
            var request = new RestRequest("/eemEducationalEntityMasters", Method.GET);
            request.RequestFormat = DataFormat.Json;

            if (offset != null)
                request.AddParameter("offset", offset);
            if (limit != null)
                request.AddParameter("limit", limit);
            if (minChangeVersion != null)
                request.AddParameter("minChangeVersion", minChangeVersion);
            if (maxChangeVersion != null)
                request.AddParameter("maxChangeVersion", maxChangeVersion);
            if (eemEducationalEntityMasterId != null)
                request.AddParameter("eemEducationalEntityMasterId", eemEducationalEntityMasterId);
            if (eemCountyId != null)
                request.AddParameter("eemCountyId", eemCountyId);
            if (geographicEEMCountyId != null)
                request.AddParameter("geographicEEMCountyId", geographicEEMCountyId);
            if (previousDistrictEEMEducationalEntityMasterId != null)
                request.AddParameter("previousDistrictEEMEducationalEntityMasterId", previousDistrictEEMEducationalEntityMasterId);
            if (eemEntityTypeClassificationId != null)
                request.AddParameter("eemEntityTypeClassificationId", eemEntityTypeClassificationId);
            if (eemCloseReasonDescriptor != null)
                request.AddParameter("eemCloseReasonDescriptor", eemCloseReasonDescriptor);
            if (eemFiscalEntityTypeDescriptor != null)
                request.AddParameter("eemFiscalEntityTypeDescriptor", eemFiscalEntityTypeDescriptor);
            if (eemStatusDescriptor != null)
                request.AddParameter("eemStatusDescriptor", eemStatusDescriptor);
            if (eemEntityOfficialName != null)
                request.AddParameter("eemEntityOfficialName", eemEntityOfficialName);
            if (openDate != null)
                request.AddParameter("openDate", openDate);
            if (closeDate != null)
                request.AddParameter("closeDate", closeDate);
            if (earlyMiddleCollegeFlag != null)
                request.AddParameter("earlyMiddleCollegeFlag", earlyMiddleCollegeFlag);
            if (id != null)
                request.AddParameter("id", id);
            request.AddHeader("Accept", "application/json");
            var response = client.Execute<List<EemEducationalEntityMaster>>(request);

            return response;
        }
        /// <summary>
        /// Retrieves a specific resource using the values of the resource's natural key (using the &quot;Get By Key&quot; pattern). This GET operation provides access to resources using the &quot;Get by Key&quot; search pattern. The values of the natural key of the resource must be fully specified, and the service will return the matching result (if it exists).
        /// </summary>
        /// <param name="eemEducationalEntityMasterId">Educational Entity Master identifier from EEM system.</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<EemEducationalEntityMaster> GetEEMEducationalEntityMasterByKey(int? eemEducationalEntityMasterId, string IfNoneMatch= null) 
        {
            var request = new RestRequest("/eemEducationalEntityMasters", Method.GET);
            request.RequestFormat = DataFormat.Json;

            // verify required params are set
            if (eemEducationalEntityMasterId == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            if (eemEducationalEntityMasterId != null)
                request.AddParameter("eemEducationalEntityMasterId", eemEducationalEntityMasterId);
            request.AddHeader("If-None-Match", IfNoneMatch);
            request.AddHeader("Accept", "application/json");
            var response = client.Execute<EemEducationalEntityMaster>(request);

            return response;
        }
        /// <summary>
        /// Creates or updates resources based on the natural key values of the supplied resource. The POST operation can be used to create or update resources. In database terms, this is often referred to as an &quot;upsert&quot; operation (insert + update).  Clients should NOT include the resource &quot;id&quot; in the JSON body because it will result in an error (you must use a PUT operation to update a resource by &quot;id&quot;). The web service will identify whether the resource already exists based on the natural key values provided, and update or create the resource appropriately.
        /// </summary>
        /// <param name="body">The JSON representation of the &quot;eemEducationalEntityMaster&quot; resource to be created or updated.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse PostEEMEducationalEntityMasters(EemEducationalEntityMaster body) 
        {
            var request = new RestRequest("/eemEducationalEntityMasters", Method.POST);
            request.RequestFormat = DataFormat.Json;

            // verify required params are set
            if (body == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            request.AddBody(body);
            request.Parameters.First(param => param.Type == ParameterType.RequestBody).Name = "application/json";
            var response = client.Execute(request);

            var location = response.Headers.FirstOrDefault(x => x.Name == "Location");

            if (location != null && !string.IsNullOrWhiteSpace(location.Value.ToString()))
                body.id = location.Value.ToString().Split('/').Last();
            return response;
        }
        /// <summary>
        /// Retrieves a specific resource using the resource's identifier (using the &quot;Get By Id&quot; pattern). This GET operation retrieves a resource by the specified resource identifier.
        /// </summary>
        /// <param name="id">A resource identifier specifying the resource to be retrieved.</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<EemEducationalEntityMaster> GetEEMEducationalEntityMastersById(string id, string IfNoneMatch= null) 
        {
            var request = new RestRequest("/eemEducationalEntityMasters/{id}", Method.GET);
            request.RequestFormat = DataFormat.Json;

            request.AddUrlSegment("id", id);
            // verify required params are set
            if (id == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            request.AddHeader("If-None-Match", IfNoneMatch);
            request.AddHeader("Accept", "application/json");
            var response = client.Execute<EemEducationalEntityMaster>(request);

            return response;
        }
        /// <summary>
        /// Updates or creates a resource based on the resource identifier. The PUT operation is used to update or create a resource by identifier.  If the resource doesn't exist, the resource will be created using that identifier.  Additionally, natural key values cannot be changed using this operation, and will not be modified in the database.  If the resource &quot;id&quot; is provided in the JSON body, it will be ignored as well.
        /// </summary>
        /// <param name="id">A resource identifier specifying the resource to be updated.</param>
        /// <param name="IfMatch">The ETag header value used to prevent the PUT from updating a resource modified by another consumer.</param>
        /// <param name="body">The JSON representation of the &quot;eemEducationalEntityMaster&quot; resource to be updated.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse PutEEMEducationalEntityMaster(string id, string IfMatch, EemEducationalEntityMaster body) 
        {
            var request = new RestRequest("/eemEducationalEntityMasters/{id}", Method.PUT);
            request.RequestFormat = DataFormat.Json;

            request.AddUrlSegment("id", id);
            // verify required params are set
            if (id == null || body == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            request.AddHeader("If-Match", IfMatch);
            request.AddBody(body);
            request.Parameters.First(param => param.Type == ParameterType.RequestBody).Name = "application/json";
            var response = client.Execute(request);

            var location = response.Headers.FirstOrDefault(x => x.Name == "Location");

            if (location != null && !string.IsNullOrWhiteSpace(location.Value.ToString()))
                body.id = location.Value.ToString().Split('/').Last();
            return response;
        }
        /// <summary>
        /// Deletes an existing resource using the resource identifier. The DELETE operation is used to delete an existing resource by identifier.  If the resource doesn't exist, an error will result (the resource will not be found).
        /// </summary>
        /// <param name="id">A resource identifier specifying the resource to be deleted.</param>
        /// <param name="IfMatch">The ETag header value used to prevent the DELETE from removing a resource modified by another consumer.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse DeleteEEMEducationalEntityMasterById(string id, string IfMatch= null) 
        {
            var request = new RestRequest("/eemEducationalEntityMasters/{id}", Method.DELETE);
            request.RequestFormat = DataFormat.Json;

            request.AddUrlSegment("id", id);
            // verify required params are set
            if (id == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            request.AddHeader("If-Match", IfMatch);
            var response = client.Execute(request);

            return response;
        }
        /// <summary>
        ///Retrieves a specific resource using the values of the resource's natural key (using the &quot;Get By Key&quot; pattern). This GET operation provides access to resources using the &quot;Get by Key&quot; search pattern. The values of the natural key of the resource must be fully specified, and the service will return the matching result (if it exists).
        ///</summary>
        /// <param name="eemEducationalEntityMasterId">Educational Entity Master identifier from EEM system.</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        ///<returns></returns>
        public IRestResponse PatchGetEEMEducationalEntityMasterByKey(int? eemEducationalEntityMasterId, string IfNoneMatch, EemEducationalEntityMaster patchEemEducationalEntityMaster)
        {
            // verify required params are set
            if (eemEducationalEntityMasterId == null ) {
                throw new ArgumentException("API method call is missing required parameters");
            }
            var sdkUsageRecord = new SdkUsageRecord();
            var jsonSerializer = new RestSharp.Serializers.JsonSerializer();
            var statisticsTracker = new StatisticsTracker();

            var getResponse = this.GetEEMEducationalEntityMasterByKey( eemEducationalEntityMasterId, IfNoneMatch);

            if (!(getResponse.StatusCode == HttpStatusCode.OK || getResponse.StatusCode == HttpStatusCode.Found))
                return getResponse;

            if (statisticsTracker.ShouldTrackStatistics)
            {
                sdkUsageRecord.DateTimeStamp = DateTime.Now;
                sdkUsageRecord.HttpMethod = "PatchGetEEMEducationalEntityMasterByKey";
                sdkUsageRecord.Url = "/eemEducationalEntityMasters";
                var serializedGetObject = jsonSerializer.Serialize(getResponse.Data);
                sdkUsageRecord.GetSizeInBytes = Encoding.UTF8.GetByteCount(serializedGetObject);
                var serializedPatchObject = jsonSerializer.Serialize(patchEemEducationalEntityMaster);
                sdkUsageRecord.PatchSizeInBytes = Encoding.UTF8.GetByteCount(serializedPatchObject);
            }

            new PatchApplier().Execute(getResponse.Data, patchEemEducationalEntityMaster);

            var putResponse = PutEEMEducationalEntityMaster( getResponse.Data.id, null, getResponse.Data ); 
            if (statisticsTracker.ShouldTrackStatistics)
            {
                var serializedPutObject = jsonSerializer.Serialize(getResponse.Data);
                sdkUsageRecord.PutSizeInBytes = Encoding.UTF8.GetByteCount(serializedPutObject);
                statisticsTracker.TrackIfEnabled(sdkUsageRecord);
            }
            return putResponse;
        }
        /// <summary>
        ///Retrieves a specific resource using the resource's identifier (using the &quot;Get By Id&quot; pattern). This GET operation retrieves a resource by the specified resource identifier.
        ///</summary>
        /// <param name="id">A resource identifier specifying the resource to be retrieved.</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        ///<returns></returns>
        public IRestResponse PatchGetEEMEducationalEntityMastersById(string id, string IfNoneMatch, EemEducationalEntityMaster patchEemEducationalEntityMaster)
        {
            // verify required params are set
            if (id == null ) {
                throw new ArgumentException("API method call is missing required parameters");
            }
            var sdkUsageRecord = new SdkUsageRecord();
            var jsonSerializer = new RestSharp.Serializers.JsonSerializer();
            var statisticsTracker = new StatisticsTracker();

            var getResponse = this.GetEEMEducationalEntityMastersById( id, IfNoneMatch);

            if (!(getResponse.StatusCode == HttpStatusCode.OK || getResponse.StatusCode == HttpStatusCode.Found))
                return getResponse;

            if (statisticsTracker.ShouldTrackStatistics)
            {
                sdkUsageRecord.DateTimeStamp = DateTime.Now;
                sdkUsageRecord.HttpMethod = "PatchGetEEMEducationalEntityMastersById";
                sdkUsageRecord.Url = "/eemEducationalEntityMasters/{id}";
                var serializedGetObject = jsonSerializer.Serialize(getResponse.Data);
                sdkUsageRecord.GetSizeInBytes = Encoding.UTF8.GetByteCount(serializedGetObject);
                var serializedPatchObject = jsonSerializer.Serialize(patchEemEducationalEntityMaster);
                sdkUsageRecord.PatchSizeInBytes = Encoding.UTF8.GetByteCount(serializedPatchObject);
            }

            new PatchApplier().Execute(getResponse.Data, patchEemEducationalEntityMaster);

            var putResponse = PutEEMEducationalEntityMaster( getResponse.Data.id, null, getResponse.Data ); 
            if (statisticsTracker.ShouldTrackStatistics)
            {
                var serializedPutObject = jsonSerializer.Serialize(getResponse.Data);
                sdkUsageRecord.PutSizeInBytes = Encoding.UTF8.GetByteCount(serializedPutObject);
                statisticsTracker.TrackIfEnabled(sdkUsageRecord);
            }
            return putResponse;
        }
        }
}

