﻿using System;
using System.Collections.Generic;
using System.Linq;
using Michigan.Ods.Sdk.Helpers;
using Michigan.Ods.Sdk.Models;
using RestSharp;
  using System.Net;
  using System.Text;
  
namespace Michigan.Ods.Sdk.Api 
{
    public class StudentAssessmentsApi 
    {
        private readonly IRestClient client;

        public StudentAssessmentsApi(IRestClient client)
        {
            this.client = client;
        }
      
        /// <summary>
        /// Retrieves resources based with paging capabilities (using the &quot;Get All&quot; pattern). This GET operation provides access to resources using the &quot;Get All&quot; pattern. In this version of the API there is support for paging.
        /// </summary>
        /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
        /// <param name="minChangeVersion">Used in synchronization to set sequence minimum ChangeVersion</param>
        /// <param name="maxChangeVersion">Used in synchronization to set sequence maximum ChangeVersion</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<List<StudentAssessment>> GetStudentAssessmentsAll(int? offset= null, int? limit= null, int? minChangeVersion= null, int? maxChangeVersion= null) 
        {
            var request = new RestRequest("/studentAssessments", Method.GET);
            request.RequestFormat = DataFormat.Json;

            if (offset != null)
                request.AddParameter("offset", offset);
            if (limit != null)
                request.AddParameter("limit", limit);
            if (minChangeVersion != null)
                request.AddParameter("minChangeVersion", minChangeVersion);
            if (maxChangeVersion != null)
                request.AddParameter("maxChangeVersion", maxChangeVersion);
            request.AddHeader("Accept", "application/json");
            var response = client.Execute<List<StudentAssessment>>(request);

            return response;
        }
        /// <summary>
        /// Retrieves resources matching values of an example resource (using the &quot;Get By Example&quot; pattern). This GET operation provides access to resources using the &quot;Get by Example&quot; search pattern.  The values of any properties of the resource that are specified will be used to return all matching results (if it exists).
        /// </summary>
        /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
        /// <param name="minChangeVersion">Used in synchronization to set sequence minimum ChangeVersion</param>
        /// <param name="maxChangeVersion">Used in synchronization to set sequence maximum ChangeVersion</param>
        /// <param name="administrationDate">The month(s), day(s), and year on which an assessment is administered or first day of administration if over multiple days.</param>
        /// <param name="assessmentTitle">The title or name of the Assessment.</param>
        /// <param name="assessedGradeLevelDescriptor">The typical grade level for which an assessment is designed. If the assessment spans a range of grades, then this attribute holds the highest grade assessed. If only one grade level is assessed then only this attribute is used. For example:          Adult          Prekindergarten          First grade          Second grade          ...</param>
        /// <param name="academicSubjectDescriptor">The description of the content or subject area (e.g., arts, mathematics, reading, stenography, or a foreign language) of an assessment.</param>
        /// <param name="version">The version identifier for the assessment.</param>
        /// <param name="studentUniqueId">A unique alphanumeric code assigned to a student.</param>
        /// <param name="administrationEnvironmentType">The environment in which the test was administered. For example:          Electronic          Classroom          Testing Center          ...</param>
        /// <param name="eventCircumstanceType">An unusual event occurred during the administration of the assessment. This could include fire alarm, student became ill, etc.</param>
        /// <param name="whenAssessedGradeLevelDescriptor">The grade level of a student when assessed.</param>
        /// <param name="administrationLanguageDescriptor">The language in which an assessment is written and/or administered.</param>
        /// <param name="reasonNotTestedType">The primary reason student is not tested. For example:          Absent          Refusal by parent          Refusal by student          Medical waiver          Illness          Disruptive behavior          LEP Exempt          ...</param>
        /// <param name="retestIndicatorType">Indicator if the test was retaken. For example:          Primary administration          First retest          Second retest          ...</param>
        /// <param name="administrationEndDate">Assessment Administration End Date, if administered over multiple days.</param>
        /// <param name="serialNumber">The unique number for the assessment form or answer document.</param>
        /// <param name="eventDescription">Describes special events that occur before during or after the assessment session that may impact use of results.</param>
        /// <param name="id"></param>
        /// <param name="assessmentReasonDescriptor">(MI-EXT) A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table.</param>
        /// <param name="attemptedness">(MI EXT) Identifies whether the student attempted the assessment and answered enough questions to calculate an overall scale score.</param>
        /// <param name="barcode">(MI EXT) Student's barcode number providing detailed information regarding the student and the assessment.</param>
        /// <param name="createdDate">(MI-EXT) Date record was created in MDE system.</param>
        /// <param name="designatedSupportUsed">1 = Student used a Designated Support  0 = Student did not use a Designated Support</param>
        /// <param name="exemptedSpecialEd504">(MI EXT) 1 = Y - Student exempted because of 504   0 = N - Student not exempted because of 504</param>
        /// <param name="forceSubmitPart1Flag">1 = Part 1 of the Online test was not submitted by the student but was forced submit by the system.  0 = Part 1 of the Online tet was submitted by the student.</param>
        /// <param name="forceSubmitPart2Flag">1 = Part 2 of the Online test was not submitted by the student but was forced submit by the system.  0 = Part 2 of the Online tet was submitted by the student.</param>
        /// <param name="forceSubmitPTFlag">1 = Perrformance task of the Online test was not submitted by the student but was forced submit by the system.  0 = Performance task of the Online tet was submitted by the student.</param>
        /// <param name="gradeClusterDescriptor">(MI-EXT) A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table.</param>
        /// <param name="invalidReasonCodeDescriptor">A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table.</param>
        /// <param name="misadministration">(MI EXT) Identifies whether there was a misadministration of the assessment to the student. Misadministration of an assessment includes</param>
        /// <param name="modifiedDate">(MI-EXT) Date record was modified in the MDE system.</param>
        /// <param name="nonstandardAccommodation">(MI EXT) Identifies whether a student received accommodations that do not qualify as standard accommodations.</param>
        /// <param name="online">(MI EXT) Identifies whether the assessment was administered in an online testing environment.</param>
        /// <param name="outOfLevel">(MI EXT) Identifies whether student tested in an incorrect grade.</param>
        /// <param name="parentConferenceDate">(MI-EXT) Date the parent conference occured.</param>
        /// <param name="parentConferenceMethodDescriptor">(MI-EXT) A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table.</param>
        /// <param name="previousAdministrationPerformanceLevelDescriptor">A specification of which performance level value describes the student proficiency.</param>
        /// <param name="prohibitedBehavior">(MI EXT) Identifies whether the student was involved in prohibited behavior while the assessment was being administered.</param>
        /// <param name="reportedTier">(MI-EXT) A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table.</param>
        /// <param name="scribedStudentResponse">(MI EXT) 1 = Y - Writing was scribed   0 = N - Writing was not scribed</param>
        /// <param name="standardAccommodation">(MI EXT)  Identifies whether a student received a standard accommodation while the assessment was administered. Standard accommodations are listed and includes: Braille, Text-to-speech passage, Read-aloud, Enlarged Print ...</param>
        /// <param name="testMarkedInvalid">(MI EXT) 1 = Y - Student test is marked as invalid on answer document   0 = N - Student test is not marked as invalid on answer document</param>
        /// <param name="testMarkedInvalidReasonDescriptor">(MI-EXT) A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table.</param>
        /// <param name="testModeDescriptor">(MI-EXT) A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table.</param>
        /// <param name="unableToTestReasonDescriptor">(MI-EXT) A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table.</param>
        /// <param name="valid">(MI EXT) Identifies whether the student received a valid test score for the assessment.</param>
        /// <param name="validContent">(MI-EXT) 1 = Y - Content is a valid test   0 = N - Content is not a valid test</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<List<StudentAssessment>> GetStudentAssessmentsByExample(int? offset= null, int? limit= null, int? minChangeVersion= null, int? maxChangeVersion= null, DateTime? administrationDate= null, string assessmentTitle= null, string assessedGradeLevelDescriptor= null, string academicSubjectDescriptor= null, int? version= null, string studentUniqueId= null, string administrationEnvironmentType= null, string eventCircumstanceType= null, string whenAssessedGradeLevelDescriptor= null, string administrationLanguageDescriptor= null, string reasonNotTestedType= null, string retestIndicatorType= null, DateTime? administrationEndDate= null, string serialNumber= null, string eventDescription= null, string id= null, string assessmentReasonDescriptor= null, bool? attemptedness= null, string barcode= null, DateTime? createdDate= null, bool? designatedSupportUsed= null, bool? exemptedSpecialEd504= null, bool? forceSubmitPart1Flag= null, bool? forceSubmitPart2Flag= null, bool? forceSubmitPTFlag= null, string gradeClusterDescriptor= null, string invalidReasonCodeDescriptor= null, bool? misadministration= null, DateTime? modifiedDate= null, bool? nonstandardAccommodation= null, bool? online= null, bool? outOfLevel= null, DateTime? parentConferenceDate= null, string parentConferenceMethodDescriptor= null, string previousAdministrationPerformanceLevelDescriptor= null, bool? prohibitedBehavior= null, string reportedTier= null, bool? scribedStudentResponse= null, bool? standardAccommodation= null, bool? testMarkedInvalid= null, string testMarkedInvalidReasonDescriptor= null, string testModeDescriptor= null, string unableToTestReasonDescriptor= null, bool? valid= null, bool? validContent= null) 
        {
            var request = new RestRequest("/studentAssessments", Method.GET);
            request.RequestFormat = DataFormat.Json;

            if (offset != null)
                request.AddParameter("offset", offset);
            if (limit != null)
                request.AddParameter("limit", limit);
            if (minChangeVersion != null)
                request.AddParameter("minChangeVersion", minChangeVersion);
            if (maxChangeVersion != null)
                request.AddParameter("maxChangeVersion", maxChangeVersion);
            if (administrationDate != null)
                request.AddParameter("administrationDate", administrationDate);
            if (assessmentTitle != null)
                request.AddParameter("assessmentTitle", assessmentTitle);
            if (assessedGradeLevelDescriptor != null)
                request.AddParameter("assessedGradeLevelDescriptor", assessedGradeLevelDescriptor);
            if (academicSubjectDescriptor != null)
                request.AddParameter("academicSubjectDescriptor", academicSubjectDescriptor);
            if (version != null)
                request.AddParameter("version", version);
            if (studentUniqueId != null)
                request.AddParameter("studentUniqueId", studentUniqueId);
            if (administrationEnvironmentType != null)
                request.AddParameter("administrationEnvironmentType", administrationEnvironmentType);
            if (eventCircumstanceType != null)
                request.AddParameter("eventCircumstanceType", eventCircumstanceType);
            if (whenAssessedGradeLevelDescriptor != null)
                request.AddParameter("whenAssessedGradeLevelDescriptor", whenAssessedGradeLevelDescriptor);
            if (administrationLanguageDescriptor != null)
                request.AddParameter("administrationLanguageDescriptor", administrationLanguageDescriptor);
            if (reasonNotTestedType != null)
                request.AddParameter("reasonNotTestedType", reasonNotTestedType);
            if (retestIndicatorType != null)
                request.AddParameter("retestIndicatorType", retestIndicatorType);
            if (administrationEndDate != null)
                request.AddParameter("administrationEndDate", administrationEndDate);
            if (serialNumber != null)
                request.AddParameter("serialNumber", serialNumber);
            if (eventDescription != null)
                request.AddParameter("eventDescription", eventDescription);
            if (id != null)
                request.AddParameter("id", id);
            if (assessmentReasonDescriptor != null)
                request.AddParameter("assessmentReasonDescriptor", assessmentReasonDescriptor);
            if (attemptedness != null)
                request.AddParameter("attemptedness", attemptedness);
            if (barcode != null)
                request.AddParameter("barcode", barcode);
            if (createdDate != null)
                request.AddParameter("createdDate", createdDate);
            if (designatedSupportUsed != null)
                request.AddParameter("designatedSupportUsed", designatedSupportUsed);
            if (exemptedSpecialEd504 != null)
                request.AddParameter("exemptedSpecialEd504", exemptedSpecialEd504);
            if (forceSubmitPart1Flag != null)
                request.AddParameter("forceSubmitPart1Flag", forceSubmitPart1Flag);
            if (forceSubmitPart2Flag != null)
                request.AddParameter("forceSubmitPart2Flag", forceSubmitPart2Flag);
            if (forceSubmitPTFlag != null)
                request.AddParameter("forceSubmitPTFlag", forceSubmitPTFlag);
            if (gradeClusterDescriptor != null)
                request.AddParameter("gradeClusterDescriptor", gradeClusterDescriptor);
            if (invalidReasonCodeDescriptor != null)
                request.AddParameter("invalidReasonCodeDescriptor", invalidReasonCodeDescriptor);
            if (misadministration != null)
                request.AddParameter("misadministration", misadministration);
            if (modifiedDate != null)
                request.AddParameter("modifiedDate", modifiedDate);
            if (nonstandardAccommodation != null)
                request.AddParameter("nonstandardAccommodation", nonstandardAccommodation);
            if (online != null)
                request.AddParameter("online", online);
            if (outOfLevel != null)
                request.AddParameter("outOfLevel", outOfLevel);
            if (parentConferenceDate != null)
                request.AddParameter("parentConferenceDate", parentConferenceDate);
            if (parentConferenceMethodDescriptor != null)
                request.AddParameter("parentConferenceMethodDescriptor", parentConferenceMethodDescriptor);
            if (previousAdministrationPerformanceLevelDescriptor != null)
                request.AddParameter("previousAdministrationPerformanceLevelDescriptor", previousAdministrationPerformanceLevelDescriptor);
            if (prohibitedBehavior != null)
                request.AddParameter("prohibitedBehavior", prohibitedBehavior);
            if (reportedTier != null)
                request.AddParameter("reportedTier", reportedTier);
            if (scribedStudentResponse != null)
                request.AddParameter("scribedStudentResponse", scribedStudentResponse);
            if (standardAccommodation != null)
                request.AddParameter("standardAccommodation", standardAccommodation);
            if (testMarkedInvalid != null)
                request.AddParameter("testMarkedInvalid", testMarkedInvalid);
            if (testMarkedInvalidReasonDescriptor != null)
                request.AddParameter("testMarkedInvalidReasonDescriptor", testMarkedInvalidReasonDescriptor);
            if (testModeDescriptor != null)
                request.AddParameter("testModeDescriptor", testModeDescriptor);
            if (unableToTestReasonDescriptor != null)
                request.AddParameter("unableToTestReasonDescriptor", unableToTestReasonDescriptor);
            if (valid != null)
                request.AddParameter("valid", valid);
            if (validContent != null)
                request.AddParameter("validContent", validContent);
            request.AddHeader("Accept", "application/json");
            var response = client.Execute<List<StudentAssessment>>(request);

            return response;
        }
        /// <summary>
        /// Retrieves a specific resource using the values of the resource's natural key (using the &quot;Get By Key&quot; pattern). This GET operation provides access to resources using the &quot;Get by Key&quot; search pattern. The values of the natural key of the resource must be fully specified, and the service will return the matching result (if it exists).
        /// </summary>
        /// <param name="academicSubjectDescriptor">The description of the content or subject area (e.g., arts, mathematics, reading, stenography, or a foreign language) of an assessment.</param>
        /// <param name="administrationDate">The month(s), day(s), and year on which an assessment is administered or first day of administration if over multiple days.</param>
        /// <param name="assessedGradeLevelDescriptor">The typical grade level for which an assessment is designed. If the assessment spans a range of grades, then this attribute holds the highest grade assessed. If only one grade level is assessed then only this attribute is used. For example:          Adult          Prekindergarten          First grade          Second grade          ...</param>
        /// <param name="assessmentTitle">The title or name of the Assessment.</param>
        /// <param name="studentUniqueId">A unique alphanumeric code assigned to a student.</param>
        /// <param name="version">The version identifier for the assessment.</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<StudentAssessment> GetStudentAssessmentByKey(string academicSubjectDescriptor, DateTime? administrationDate, string assessedGradeLevelDescriptor, string assessmentTitle, string studentUniqueId, int? version, string IfNoneMatch= null) 
        {
            var request = new RestRequest("/studentAssessments", Method.GET);
            request.RequestFormat = DataFormat.Json;

            // verify required params are set
            if (academicSubjectDescriptor == null || administrationDate == null || assessedGradeLevelDescriptor == null || assessmentTitle == null || studentUniqueId == null || version == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            if (academicSubjectDescriptor != null)
                request.AddParameter("academicSubjectDescriptor", academicSubjectDescriptor);
            if (administrationDate != null)
                request.AddParameter("administrationDate", administrationDate);
            if (assessedGradeLevelDescriptor != null)
                request.AddParameter("assessedGradeLevelDescriptor", assessedGradeLevelDescriptor);
            if (assessmentTitle != null)
                request.AddParameter("assessmentTitle", assessmentTitle);
            if (studentUniqueId != null)
                request.AddParameter("studentUniqueId", studentUniqueId);
            if (version != null)
                request.AddParameter("version", version);
            request.AddHeader("If-None-Match", IfNoneMatch);
            request.AddHeader("Accept", "application/json");
            var response = client.Execute<StudentAssessment>(request);

            return response;
        }
        /// <summary>
        /// Creates or updates resources based on the natural key values of the supplied resource. The POST operation can be used to create or update resources. In database terms, this is often referred to as an &quot;upsert&quot; operation (insert + update).  Clients should NOT include the resource &quot;id&quot; in the JSON body because it will result in an error (you must use a PUT operation to update a resource by &quot;id&quot;). The web service will identify whether the resource already exists based on the natural key values provided, and update or create the resource appropriately.
        /// </summary>
        /// <param name="body">The JSON representation of the &quot;studentAssessment&quot; resource to be created or updated.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse PostStudentAssessments(StudentAssessment body) 
        {
            var request = new RestRequest("/studentAssessments", Method.POST);
            request.RequestFormat = DataFormat.Json;

            // verify required params are set
            if (body == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            request.AddBody(body);
            request.Parameters.First(param => param.Type == ParameterType.RequestBody).Name = "application/json";
            var response = client.Execute(request);

            var location = response.Headers.FirstOrDefault(x => x.Name == "Location");

            if (location != null && !string.IsNullOrWhiteSpace(location.Value.ToString()))
                body.id = location.Value.ToString().Split('/').Last();
            return response;
        }
        /// <summary>
        /// Retrieves a specific resource using the resource's identifier (using the &quot;Get By Id&quot; pattern). This GET operation retrieves a resource by the specified resource identifier.
        /// </summary>
        /// <param name="id">A resource identifier specifying the resource to be retrieved.</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<StudentAssessment> GetStudentAssessmentsById(string id, string IfNoneMatch= null) 
        {
            var request = new RestRequest("/studentAssessments/{id}", Method.GET);
            request.RequestFormat = DataFormat.Json;

            request.AddUrlSegment("id", id);
            // verify required params are set
            if (id == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            request.AddHeader("If-None-Match", IfNoneMatch);
            request.AddHeader("Accept", "application/json");
            var response = client.Execute<StudentAssessment>(request);

            return response;
        }
        /// <summary>
        /// Updates or creates a resource based on the resource identifier. The PUT operation is used to update or create a resource by identifier.  If the resource doesn't exist, the resource will be created using that identifier.  Additionally, natural key values cannot be changed using this operation, and will not be modified in the database.  If the resource &quot;id&quot; is provided in the JSON body, it will be ignored as well.
        /// </summary>
        /// <param name="id">A resource identifier specifying the resource to be updated.</param>
        /// <param name="IfMatch">The ETag header value used to prevent the PUT from updating a resource modified by another consumer.</param>
        /// <param name="body">The JSON representation of the &quot;studentAssessment&quot; resource to be updated.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse PutStudentAssessment(string id, string IfMatch, StudentAssessment body) 
        {
            var request = new RestRequest("/studentAssessments/{id}", Method.PUT);
            request.RequestFormat = DataFormat.Json;

            request.AddUrlSegment("id", id);
            // verify required params are set
            if (id == null || body == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            request.AddHeader("If-Match", IfMatch);
            request.AddBody(body);
            request.Parameters.First(param => param.Type == ParameterType.RequestBody).Name = "application/json";
            var response = client.Execute(request);

            var location = response.Headers.FirstOrDefault(x => x.Name == "Location");

            if (location != null && !string.IsNullOrWhiteSpace(location.Value.ToString()))
                body.id = location.Value.ToString().Split('/').Last();
            return response;
        }
        /// <summary>
        /// Deletes an existing resource using the resource identifier. The DELETE operation is used to delete an existing resource by identifier.  If the resource doesn't exist, an error will result (the resource will not be found).
        /// </summary>
        /// <param name="id">A resource identifier specifying the resource to be deleted.</param>
        /// <param name="IfMatch">The ETag header value used to prevent the DELETE from removing a resource modified by another consumer.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse DeleteStudentAssessmentById(string id, string IfMatch= null) 
        {
            var request = new RestRequest("/studentAssessments/{id}", Method.DELETE);
            request.RequestFormat = DataFormat.Json;

            request.AddUrlSegment("id", id);
            // verify required params are set
            if (id == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            request.AddHeader("If-Match", IfMatch);
            var response = client.Execute(request);

            return response;
        }
        /// <summary>
        ///Retrieves a specific resource using the values of the resource's natural key (using the &quot;Get By Key&quot; pattern). This GET operation provides access to resources using the &quot;Get by Key&quot; search pattern. The values of the natural key of the resource must be fully specified, and the service will return the matching result (if it exists).
        ///</summary>
        /// <param name="academicSubjectDescriptor">The description of the content or subject area (e.g., arts, mathematics, reading, stenography, or a foreign language) of an assessment.</param>
        /// <param name="administrationDate">The month(s), day(s), and year on which an assessment is administered or first day of administration if over multiple days.</param>
        /// <param name="assessedGradeLevelDescriptor">The typical grade level for which an assessment is designed. If the assessment spans a range of grades, then this attribute holds the highest grade assessed. If only one grade level is assessed then only this attribute is used. For example:          Adult          Prekindergarten          First grade          Second grade          ...</param>
        /// <param name="assessmentTitle">The title or name of the Assessment.</param>
        /// <param name="studentUniqueId">A unique alphanumeric code assigned to a student.</param>
        /// <param name="version">The version identifier for the assessment.</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        ///<returns></returns>
        public IRestResponse PatchGetStudentAssessmentByKey(string academicSubjectDescriptor, DateTime? administrationDate, string assessedGradeLevelDescriptor, string assessmentTitle, string studentUniqueId, int? version, string IfNoneMatch, StudentAssessment patchStudentAssessment)
        {
            // verify required params are set
            if (academicSubjectDescriptor == null || administrationDate == null || assessedGradeLevelDescriptor == null || assessmentTitle == null || studentUniqueId == null || version == null ) {
                throw new ArgumentException("API method call is missing required parameters");
            }
            var sdkUsageRecord = new SdkUsageRecord();
            var jsonSerializer = new RestSharp.Serializers.JsonSerializer();
            var statisticsTracker = new StatisticsTracker();

            var getResponse = this.GetStudentAssessmentByKey( academicSubjectDescriptor, administrationDate, assessedGradeLevelDescriptor, assessmentTitle, studentUniqueId, version, IfNoneMatch);

            if (!(getResponse.StatusCode == HttpStatusCode.OK || getResponse.StatusCode == HttpStatusCode.Found))
                return getResponse;

            if (statisticsTracker.ShouldTrackStatistics)
            {
                sdkUsageRecord.DateTimeStamp = DateTime.Now;
                sdkUsageRecord.HttpMethod = "PatchGetStudentAssessmentByKey";
                sdkUsageRecord.Url = "/studentAssessments";
                var serializedGetObject = jsonSerializer.Serialize(getResponse.Data);
                sdkUsageRecord.GetSizeInBytes = Encoding.UTF8.GetByteCount(serializedGetObject);
                var serializedPatchObject = jsonSerializer.Serialize(patchStudentAssessment);
                sdkUsageRecord.PatchSizeInBytes = Encoding.UTF8.GetByteCount(serializedPatchObject);
            }

            new PatchApplier().Execute(getResponse.Data, patchStudentAssessment);

            var putResponse = PutStudentAssessment( getResponse.Data.id, null, getResponse.Data ); 
            if (statisticsTracker.ShouldTrackStatistics)
            {
                var serializedPutObject = jsonSerializer.Serialize(getResponse.Data);
                sdkUsageRecord.PutSizeInBytes = Encoding.UTF8.GetByteCount(serializedPutObject);
                statisticsTracker.TrackIfEnabled(sdkUsageRecord);
            }
            return putResponse;
        }
        /// <summary>
        ///Retrieves a specific resource using the resource's identifier (using the &quot;Get By Id&quot; pattern). This GET operation retrieves a resource by the specified resource identifier.
        ///</summary>
        /// <param name="id">A resource identifier specifying the resource to be retrieved.</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        ///<returns></returns>
        public IRestResponse PatchGetStudentAssessmentsById(string id, string IfNoneMatch, StudentAssessment patchStudentAssessment)
        {
            // verify required params are set
            if (id == null ) {
                throw new ArgumentException("API method call is missing required parameters");
            }
            var sdkUsageRecord = new SdkUsageRecord();
            var jsonSerializer = new RestSharp.Serializers.JsonSerializer();
            var statisticsTracker = new StatisticsTracker();

            var getResponse = this.GetStudentAssessmentsById( id, IfNoneMatch);

            if (!(getResponse.StatusCode == HttpStatusCode.OK || getResponse.StatusCode == HttpStatusCode.Found))
                return getResponse;

            if (statisticsTracker.ShouldTrackStatistics)
            {
                sdkUsageRecord.DateTimeStamp = DateTime.Now;
                sdkUsageRecord.HttpMethod = "PatchGetStudentAssessmentsById";
                sdkUsageRecord.Url = "/studentAssessments/{id}";
                var serializedGetObject = jsonSerializer.Serialize(getResponse.Data);
                sdkUsageRecord.GetSizeInBytes = Encoding.UTF8.GetByteCount(serializedGetObject);
                var serializedPatchObject = jsonSerializer.Serialize(patchStudentAssessment);
                sdkUsageRecord.PatchSizeInBytes = Encoding.UTF8.GetByteCount(serializedPatchObject);
            }

            new PatchApplier().Execute(getResponse.Data, patchStudentAssessment);

            var putResponse = PutStudentAssessment( getResponse.Data.id, null, getResponse.Data ); 
            if (statisticsTracker.ShouldTrackStatistics)
            {
                var serializedPutObject = jsonSerializer.Serialize(getResponse.Data);
                sdkUsageRecord.PutSizeInBytes = Encoding.UTF8.GetByteCount(serializedPutObject);
                statisticsTracker.TrackIfEnabled(sdkUsageRecord);
            }
            return putResponse;
        }
        }
}

