﻿using System;
using System.Collections.Generic;
using System.Linq;
using Michigan.Ods.Sdk.Helpers;
using Michigan.Ods.Sdk.Models;
using RestSharp;
  using System.Net;
  using System.Text;
  
namespace Michigan.Ods.Sdk.Api 
{
    public class StudentCommunityProviderLocationAttendanceEventsApi 
    {
        private readonly IRestClient client;

        public StudentCommunityProviderLocationAttendanceEventsApi(IRestClient client)
        {
            this.client = client;
        }
      
        /// <summary>
        /// Retrieves resources based with paging capabilities (using the &quot;Get All&quot; pattern). This GET operation provides access to resources using the &quot;Get All&quot; pattern. In this version of the API there is support for paging.
        /// </summary>
        /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
        /// <param name="minChangeVersion">Used in synchronization to set sequence minimum ChangeVersion</param>
        /// <param name="maxChangeVersion">Used in synchronization to set sequence maximum ChangeVersion</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<List<StudentCommunityProviderLocationAttendanceEvent>> GetStudentCommunityProviderLocationAttendanceEventsAll(int? offset= null, int? limit= null, int? minChangeVersion= null, int? maxChangeVersion= null) 
        {
            var request = new RestRequest("/studentCommunityProviderLocationAttendanceEvents", Method.GET);
            request.RequestFormat = DataFormat.Json;

            if (offset != null)
                request.AddParameter("offset", offset);
            if (limit != null)
                request.AddParameter("limit", limit);
            if (minChangeVersion != null)
                request.AddParameter("minChangeVersion", minChangeVersion);
            if (maxChangeVersion != null)
                request.AddParameter("maxChangeVersion", maxChangeVersion);
            request.AddHeader("Accept", "application/json");
            var response = client.Execute<List<StudentCommunityProviderLocationAttendanceEvent>>(request);

            return response;
        }
        /// <summary>
        /// Retrieves resources matching values of an example resource (using the &quot;Get By Example&quot; pattern). This GET operation provides access to resources using the &quot;Get by Example&quot; search pattern.  The values of any properties of the resource that are specified will be used to return all matching results (if it exists).
        /// </summary>
        /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
        /// <param name="minChangeVersion">Used in synchronization to set sequence minimum ChangeVersion</param>
        /// <param name="maxChangeVersion">Used in synchronization to set sequence maximum ChangeVersion</param>
        /// <param name="attendanceEventCategoryDescriptor">A code describing the attendance event, for example:          Present          Unexcused absence          Excused absence          Tardy.</param>
        /// <param name="eventDate">Date for this attendance event.</param>
        /// <param name="communityProviderId">The identifier assigned to a Community Provider.</param>
        /// <param name="communityProviderLocationId">A unique number or alphanumeric code assigned to an education organization by a school, school system, a state, or other agency or entity.</param>
        /// <param name="studentUniqueId">A unique alphanumeric code assigned to a student.</param>
        /// <param name="educationalEnvironmentType">The setting in which a child receives education and related services. This attribute is only used if it differs from the EducationalEnvironment of the Section. This is only used in the AttendanceEvent if different from the associated Section.</param>
        /// <param name="attendanceEventReason">The reported reason for a student's absence.</param>
        /// <param name="id"></param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<List<StudentCommunityProviderLocationAttendanceEvent>> GetStudentCommunityProviderLocationAttendanceEventsByExample(int? offset= null, int? limit= null, int? minChangeVersion= null, int? maxChangeVersion= null, string attendanceEventCategoryDescriptor= null, DateTime? eventDate= null, int? communityProviderId= null, int? communityProviderLocationId= null, string studentUniqueId= null, string educationalEnvironmentType= null, string attendanceEventReason= null, string id= null) 
        {
            var request = new RestRequest("/studentCommunityProviderLocationAttendanceEvents", Method.GET);
            request.RequestFormat = DataFormat.Json;

            if (offset != null)
                request.AddParameter("offset", offset);
            if (limit != null)
                request.AddParameter("limit", limit);
            if (minChangeVersion != null)
                request.AddParameter("minChangeVersion", minChangeVersion);
            if (maxChangeVersion != null)
                request.AddParameter("maxChangeVersion", maxChangeVersion);
            if (attendanceEventCategoryDescriptor != null)
                request.AddParameter("attendanceEventCategoryDescriptor", attendanceEventCategoryDescriptor);
            if (eventDate != null)
                request.AddParameter("eventDate", eventDate);
            if (communityProviderId != null)
                request.AddParameter("communityProviderId", communityProviderId);
            if (communityProviderLocationId != null)
                request.AddParameter("communityProviderLocationId", communityProviderLocationId);
            if (studentUniqueId != null)
                request.AddParameter("studentUniqueId", studentUniqueId);
            if (educationalEnvironmentType != null)
                request.AddParameter("educationalEnvironmentType", educationalEnvironmentType);
            if (attendanceEventReason != null)
                request.AddParameter("attendanceEventReason", attendanceEventReason);
            if (id != null)
                request.AddParameter("id", id);
            request.AddHeader("Accept", "application/json");
            var response = client.Execute<List<StudentCommunityProviderLocationAttendanceEvent>>(request);

            return response;
        }
        /// <summary>
        /// Retrieves a specific resource using the values of the resource's natural key (using the &quot;Get By Key&quot; pattern). This GET operation provides access to resources using the &quot;Get by Key&quot; search pattern. The values of the natural key of the resource must be fully specified, and the service will return the matching result (if it exists).
        /// </summary>
        /// <param name="attendanceEventCategoryDescriptor">A code describing the attendance event, for example:          Present          Unexcused absence          Excused absence          Tardy.</param>
        /// <param name="communityProviderId">The identifier assigned to a Community Provider.</param>
        /// <param name="communityProviderLocationId">A unique number or alphanumeric code assigned to an education organization by a school, school system, a state, or other agency or entity.</param>
        /// <param name="eventDate">Date for this attendance event.</param>
        /// <param name="studentUniqueId">A unique alphanumeric code assigned to a student.</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<StudentCommunityProviderLocationAttendanceEvent> GetStudentCommunityProviderLocationAttendanceEventByKey(string attendanceEventCategoryDescriptor, int? communityProviderId, int? communityProviderLocationId, DateTime? eventDate, string studentUniqueId, string IfNoneMatch= null) 
        {
            var request = new RestRequest("/studentCommunityProviderLocationAttendanceEvents", Method.GET);
            request.RequestFormat = DataFormat.Json;

            // verify required params are set
            if (attendanceEventCategoryDescriptor == null || communityProviderId == null || communityProviderLocationId == null || eventDate == null || studentUniqueId == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            if (attendanceEventCategoryDescriptor != null)
                request.AddParameter("attendanceEventCategoryDescriptor", attendanceEventCategoryDescriptor);
            if (communityProviderId != null)
                request.AddParameter("communityProviderId", communityProviderId);
            if (communityProviderLocationId != null)
                request.AddParameter("communityProviderLocationId", communityProviderLocationId);
            if (eventDate != null)
                request.AddParameter("eventDate", eventDate);
            if (studentUniqueId != null)
                request.AddParameter("studentUniqueId", studentUniqueId);
            request.AddHeader("If-None-Match", IfNoneMatch);
            request.AddHeader("Accept", "application/json");
            var response = client.Execute<StudentCommunityProviderLocationAttendanceEvent>(request);

            return response;
        }
        /// <summary>
        /// Creates or updates resources based on the natural key values of the supplied resource. The POST operation can be used to create or update resources. In database terms, this is often referred to as an &quot;upsert&quot; operation (insert + update).  Clients should NOT include the resource &quot;id&quot; in the JSON body because it will result in an error (you must use a PUT operation to update a resource by &quot;id&quot;). The web service will identify whether the resource already exists based on the natural key values provided, and update or create the resource appropriately.
        /// </summary>
        /// <param name="body">The JSON representation of the &quot;studentCommunityProviderLocationAttendanceEvent&quot; resource to be created or updated.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse PostStudentCommunityProviderLocationAttendanceEvents(StudentCommunityProviderLocationAttendanceEvent body) 
        {
            var request = new RestRequest("/studentCommunityProviderLocationAttendanceEvents", Method.POST);
            request.RequestFormat = DataFormat.Json;

            // verify required params are set
            if (body == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            request.AddBody(body);
            request.Parameters.First(param => param.Type == ParameterType.RequestBody).Name = "application/json";
            var response = client.Execute(request);

            var location = response.Headers.FirstOrDefault(x => x.Name == "Location");

            if (location != null && !string.IsNullOrWhiteSpace(location.Value.ToString()))
                body.id = location.Value.ToString().Split('/').Last();
            return response;
        }
        /// <summary>
        /// Retrieves a specific resource using the resource's identifier (using the &quot;Get By Id&quot; pattern). This GET operation retrieves a resource by the specified resource identifier.
        /// </summary>
        /// <param name="id">A resource identifier specifying the resource to be retrieved.</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<StudentCommunityProviderLocationAttendanceEvent> GetStudentCommunityProviderLocationAttendanceEventsById(string id, string IfNoneMatch= null) 
        {
            var request = new RestRequest("/studentCommunityProviderLocationAttendanceEvents/{id}", Method.GET);
            request.RequestFormat = DataFormat.Json;

            request.AddUrlSegment("id", id);
            // verify required params are set
            if (id == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            request.AddHeader("If-None-Match", IfNoneMatch);
            request.AddHeader("Accept", "application/json");
            var response = client.Execute<StudentCommunityProviderLocationAttendanceEvent>(request);

            return response;
        }
        /// <summary>
        /// Updates or creates a resource based on the resource identifier. The PUT operation is used to update or create a resource by identifier.  If the resource doesn't exist, the resource will be created using that identifier.  Additionally, natural key values cannot be changed using this operation, and will not be modified in the database.  If the resource &quot;id&quot; is provided in the JSON body, it will be ignored as well.
        /// </summary>
        /// <param name="id">A resource identifier specifying the resource to be updated.</param>
        /// <param name="IfMatch">The ETag header value used to prevent the PUT from updating a resource modified by another consumer.</param>
        /// <param name="body">The JSON representation of the &quot;studentCommunityProviderLocationAttendanceEvent&quot; resource to be updated.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse PutStudentCommunityProviderLocationAttendanceEvent(string id, string IfMatch, StudentCommunityProviderLocationAttendanceEvent body) 
        {
            var request = new RestRequest("/studentCommunityProviderLocationAttendanceEvents/{id}", Method.PUT);
            request.RequestFormat = DataFormat.Json;

            request.AddUrlSegment("id", id);
            // verify required params are set
            if (id == null || body == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            request.AddHeader("If-Match", IfMatch);
            request.AddBody(body);
            request.Parameters.First(param => param.Type == ParameterType.RequestBody).Name = "application/json";
            var response = client.Execute(request);

            var location = response.Headers.FirstOrDefault(x => x.Name == "Location");

            if (location != null && !string.IsNullOrWhiteSpace(location.Value.ToString()))
                body.id = location.Value.ToString().Split('/').Last();
            return response;
        }
        /// <summary>
        /// Deletes an existing resource using the resource identifier. The DELETE operation is used to delete an existing resource by identifier.  If the resource doesn't exist, an error will result (the resource will not be found).
        /// </summary>
        /// <param name="id">A resource identifier specifying the resource to be deleted.</param>
        /// <param name="IfMatch">The ETag header value used to prevent the DELETE from removing a resource modified by another consumer.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse DeleteStudentCommunityProviderLocationAttendanceEventById(string id, string IfMatch= null) 
        {
            var request = new RestRequest("/studentCommunityProviderLocationAttendanceEvents/{id}", Method.DELETE);
            request.RequestFormat = DataFormat.Json;

            request.AddUrlSegment("id", id);
            // verify required params are set
            if (id == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            request.AddHeader("If-Match", IfMatch);
            var response = client.Execute(request);

            return response;
        }
        /// <summary>
        ///Retrieves a specific resource using the values of the resource's natural key (using the &quot;Get By Key&quot; pattern). This GET operation provides access to resources using the &quot;Get by Key&quot; search pattern. The values of the natural key of the resource must be fully specified, and the service will return the matching result (if it exists).
        ///</summary>
        /// <param name="attendanceEventCategoryDescriptor">A code describing the attendance event, for example:          Present          Unexcused absence          Excused absence          Tardy.</param>
        /// <param name="communityProviderId">The identifier assigned to a Community Provider.</param>
        /// <param name="communityProviderLocationId">A unique number or alphanumeric code assigned to an education organization by a school, school system, a state, or other agency or entity.</param>
        /// <param name="eventDate">Date for this attendance event.</param>
        /// <param name="studentUniqueId">A unique alphanumeric code assigned to a student.</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        ///<returns></returns>
        public IRestResponse PatchGetStudentCommunityProviderLocationAttendanceEventByKey(string attendanceEventCategoryDescriptor, int? communityProviderId, int? communityProviderLocationId, DateTime? eventDate, string studentUniqueId, string IfNoneMatch, StudentCommunityProviderLocationAttendanceEvent patchStudentCommunityProviderLocationAttendanceEvent)
        {
            // verify required params are set
            if (attendanceEventCategoryDescriptor == null || communityProviderId == null || communityProviderLocationId == null || eventDate == null || studentUniqueId == null ) {
                throw new ArgumentException("API method call is missing required parameters");
            }
            var sdkUsageRecord = new SdkUsageRecord();
            var jsonSerializer = new RestSharp.Serializers.JsonSerializer();
            var statisticsTracker = new StatisticsTracker();

            var getResponse = this.GetStudentCommunityProviderLocationAttendanceEventByKey( attendanceEventCategoryDescriptor, communityProviderId, communityProviderLocationId, eventDate, studentUniqueId, IfNoneMatch);

            if (!(getResponse.StatusCode == HttpStatusCode.OK || getResponse.StatusCode == HttpStatusCode.Found))
                return getResponse;

            if (statisticsTracker.ShouldTrackStatistics)
            {
                sdkUsageRecord.DateTimeStamp = DateTime.Now;
                sdkUsageRecord.HttpMethod = "PatchGetStudentCommunityProviderLocationAttendanceEventByKey";
                sdkUsageRecord.Url = "/studentCommunityProviderLocationAttendanceEvents";
                var serializedGetObject = jsonSerializer.Serialize(getResponse.Data);
                sdkUsageRecord.GetSizeInBytes = Encoding.UTF8.GetByteCount(serializedGetObject);
                var serializedPatchObject = jsonSerializer.Serialize(patchStudentCommunityProviderLocationAttendanceEvent);
                sdkUsageRecord.PatchSizeInBytes = Encoding.UTF8.GetByteCount(serializedPatchObject);
            }

            new PatchApplier().Execute(getResponse.Data, patchStudentCommunityProviderLocationAttendanceEvent);

            var putResponse = PutStudentCommunityProviderLocationAttendanceEvent( getResponse.Data.id, null, getResponse.Data ); 
            if (statisticsTracker.ShouldTrackStatistics)
            {
                var serializedPutObject = jsonSerializer.Serialize(getResponse.Data);
                sdkUsageRecord.PutSizeInBytes = Encoding.UTF8.GetByteCount(serializedPutObject);
                statisticsTracker.TrackIfEnabled(sdkUsageRecord);
            }
            return putResponse;
        }
        /// <summary>
        ///Retrieves a specific resource using the resource's identifier (using the &quot;Get By Id&quot; pattern). This GET operation retrieves a resource by the specified resource identifier.
        ///</summary>
        /// <param name="id">A resource identifier specifying the resource to be retrieved.</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        ///<returns></returns>
        public IRestResponse PatchGetStudentCommunityProviderLocationAttendanceEventsById(string id, string IfNoneMatch, StudentCommunityProviderLocationAttendanceEvent patchStudentCommunityProviderLocationAttendanceEvent)
        {
            // verify required params are set
            if (id == null ) {
                throw new ArgumentException("API method call is missing required parameters");
            }
            var sdkUsageRecord = new SdkUsageRecord();
            var jsonSerializer = new RestSharp.Serializers.JsonSerializer();
            var statisticsTracker = new StatisticsTracker();

            var getResponse = this.GetStudentCommunityProviderLocationAttendanceEventsById( id, IfNoneMatch);

            if (!(getResponse.StatusCode == HttpStatusCode.OK || getResponse.StatusCode == HttpStatusCode.Found))
                return getResponse;

            if (statisticsTracker.ShouldTrackStatistics)
            {
                sdkUsageRecord.DateTimeStamp = DateTime.Now;
                sdkUsageRecord.HttpMethod = "PatchGetStudentCommunityProviderLocationAttendanceEventsById";
                sdkUsageRecord.Url = "/studentCommunityProviderLocationAttendanceEvents/{id}";
                var serializedGetObject = jsonSerializer.Serialize(getResponse.Data);
                sdkUsageRecord.GetSizeInBytes = Encoding.UTF8.GetByteCount(serializedGetObject);
                var serializedPatchObject = jsonSerializer.Serialize(patchStudentCommunityProviderLocationAttendanceEvent);
                sdkUsageRecord.PatchSizeInBytes = Encoding.UTF8.GetByteCount(serializedPatchObject);
            }

            new PatchApplier().Execute(getResponse.Data, patchStudentCommunityProviderLocationAttendanceEvent);

            var putResponse = PutStudentCommunityProviderLocationAttendanceEvent( getResponse.Data.id, null, getResponse.Data ); 
            if (statisticsTracker.ShouldTrackStatistics)
            {
                var serializedPutObject = jsonSerializer.Serialize(getResponse.Data);
                sdkUsageRecord.PutSizeInBytes = Encoding.UTF8.GetByteCount(serializedPutObject);
                statisticsTracker.TrackIfEnabled(sdkUsageRecord);
            }
            return putResponse;
        }
        }
}

