﻿using System;
using System.Collections.Generic;
using System.Linq;
using Michigan.Ods.Sdk.Helpers;
using Michigan.Ods.Sdk.Composites.FoodService;
using RestSharp;
  using System.Net;
  using System.Text;
  
namespace Michigan.Ods.Sdk.Composites.FoodService 
{
    public class FoodServiceApi 
    {
        private readonly IRestClient client;

        public FoodServiceApi(IRestClient client)
        {
            this.client = client;
        }
      
        /// <summary>
        /// Retrieves resources based with paging capabilities (using the &quot;Get All&quot; pattern). This GET operation provides access to resources using the &quot;Get All&quot; pattern. In this version of the API there is support for paging.
        /// </summary>
        /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
        /// <param name="minChangeVersion">Used in synchronization to set sequence minimum ChangeVersion</param>
        /// <param name="maxChangeVersion">Used in synchronization to set sequence maximum ChangeVersion</param>
        /// <param name="fields">Specifies a subset of properties that should be returned for each entity (e.g. &quot;property1,collection1(collProp1,collProp2)&quot;).</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<List<FoodServiceStudent>> GetFoodServiceStudentsAll(int? offset= null, int? limit= null, int? minChangeVersion= null, int? maxChangeVersion= null, string fields= null) 
        {
            var request = new RestRequest("/foodService/foodServiceStudents", Method.GET);
            request.RequestFormat = DataFormat.Json;

            if (offset != null)
                request.AddParameter("offset", offset);
            if (limit != null)
                request.AddParameter("limit", limit);
            if (minChangeVersion != null)
                request.AddParameter("minChangeVersion", minChangeVersion);
            if (maxChangeVersion != null)
                request.AddParameter("maxChangeVersion", maxChangeVersion);
            if (fields != null)
                request.AddParameter("fields", fields);
            var response = client.Execute<List<FoodServiceStudent>>(request);

            return response;
        }
        /// <summary>
        /// Retrieves a specific resource using the resource's identifier (using the &quot;Get By Example&quot; pattern). This GET operation provides access to resources using the &quot;Get By Example&quot; pattern. The values of any properties of the resource that are specified will be used to return all matching results (if it exists).
        /// </summary>
        /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
        /// <param name="minChangeVersion">Used in synchronization to set sequence minimum ChangeVersion</param>
        /// <param name="maxChangeVersion">Used in synchronization to set sequence maximum ChangeVersion</param>
        /// <param name="fields">Specifies a subset of properties that should be returned for each entity (e.g. &quot;property1,collection1(collProp1,collProp2)&quot;).</param>
        /// <param name="q">Specifies a query filter expression for the request.  Currently only supports range-based queries on dates and numbers (e.g. &quot;[2016-03-07..2016-03-10]&quot;).</param>
        /// <param name="studentUniqueId">A unique alphanumeric code assigned to a student.</param>
        /// <param name="citizenshipStatusType">An indicator of whether or not the person is a U.S. citizen.</param>
        /// <param name="birthCountryDescriptor">The country in which an individual is born.</param>
        /// <param name="limitedEnglishProficiencyDescriptor">An indication that the student has been identified as limited English proficient by the Language Proficiency Assessment Committee (LPAC), or English proficient.</param>
        /// <param name="oldEthnicityType">Previous definition of Ethnicity combining Hispanic/Latino and race:         1 - American Indian or Alaskan Native         2 - Asian or Pacific Islander         3 - Black, not of Hispanic origin         4 - Hispanic         5 - White, not of Hispanic origin.</param>
        /// <param name="schoolFoodServicesEligibilityDescriptor">An indication of a student's level of eligibility for breakfast, lunch, snack, supper, and milk programs.</param>
        /// <param name="sexType">A person's gender.</param>
        /// <param name="birthStateAbbreviationType">The abbreviation for the name of the state (within the United States) or extra-state jurisdiction in which an individual was born.</param>
        /// <param name="personalTitlePrefix">A prefix used to denote the title, degree, position, or seniority of the person.</param>
        /// <param name="firstName">A name given to an individual at birth, baptism, or during another naming ceremony, or through legal change.</param>
        /// <param name="middleName">A secondary name given to an individual at birth, baptism, or during another naming ceremony.</param>
        /// <param name="lastSurname">The name borne in common by members of a family.</param>
        /// <param name="generationCodeSuffix">An appendage, if any, used to denote an individual's generation in his family (e.g., Jr., Sr., III).</param>
        /// <param name="maidenName">The person's maiden name.</param>
        /// <param name="birthDate">The month, day, and year on which an individual was born.</param>
        /// <param name="birthCity">The city the student was born in.</param>
        /// <param name="birthInternationalProvince">For students born outside of the U.S., the Province or jurisdiction in which an individual is born.</param>
        /// <param name="dateEnteredUS">For students born outside of the U.S., the date the student entered the U.S.</param>
        /// <param name="multipleBirthStatus">Indicator of whether the student was born with other siblings (i.e., twins, triplets, etc.)</param>
        /// <param name="profileThumbnail">Locator for the student photo.</param>
        /// <param name="hispanicLatinoEthnicity">An indication that the individual traces his or her origin or descent to Mexico, Puerto Rico, Cuba, Central, and South America, and other Spanish cultures, regardless of race. The term, &quot;Spanish origin,&quot; can be used in addition to &quot;Hispanic or Latino.&quot;</param>
        /// <param name="economicDisadvantaged">An indication of inadequate financial condition of an individual's family, as determined by family income, number of family members/dependents, participation in public assistance programs, and/or other characteristics considered relevant by federal, state, and local policy.</param>
        /// <param name="displacementStatus">Indicates a state health or weather related event that displaces a group of students, and may require additional funding, educational, or social services.</param>
        /// <param name="loginId">The login ID for the user; used for security access control interface.</param>
        /// <param name="id"></param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<List<FoodServiceStudent>> GetFoodServiceStudentsByExample(int? offset= null, int? limit= null, int? minChangeVersion= null, int? maxChangeVersion= null, string fields= null, string q= null, string studentUniqueId= null, string citizenshipStatusType= null, string birthCountryDescriptor= null, string limitedEnglishProficiencyDescriptor= null, string oldEthnicityType= null, string schoolFoodServicesEligibilityDescriptor= null, string sexType= null, string birthStateAbbreviationType= null, string personalTitlePrefix= null, string firstName= null, string middleName= null, string lastSurname= null, string generationCodeSuffix= null, string maidenName= null, DateTime? birthDate= null, string birthCity= null, string birthInternationalProvince= null, DateTime? dateEnteredUS= null, bool? multipleBirthStatus= null, string profileThumbnail= null, bool? hispanicLatinoEthnicity= null, bool? economicDisadvantaged= null, string displacementStatus= null, string loginId= null, string id= null) 
        {
            var request = new RestRequest("/foodService/foodServiceStudents", Method.GET);
            request.RequestFormat = DataFormat.Json;

            if (offset != null)
                request.AddParameter("offset", offset);
            if (limit != null)
                request.AddParameter("limit", limit);
            if (minChangeVersion != null)
                request.AddParameter("minChangeVersion", minChangeVersion);
            if (maxChangeVersion != null)
                request.AddParameter("maxChangeVersion", maxChangeVersion);
            if (fields != null)
                request.AddParameter("fields", fields);
            if (q != null)
                request.AddParameter("q", q);
            if (studentUniqueId != null)
                request.AddParameter("studentUniqueId", studentUniqueId);
            if (citizenshipStatusType != null)
                request.AddParameter("citizenshipStatusType", citizenshipStatusType);
            if (birthCountryDescriptor != null)
                request.AddParameter("birthCountryDescriptor", birthCountryDescriptor);
            if (limitedEnglishProficiencyDescriptor != null)
                request.AddParameter("limitedEnglishProficiencyDescriptor", limitedEnglishProficiencyDescriptor);
            if (oldEthnicityType != null)
                request.AddParameter("oldEthnicityType", oldEthnicityType);
            if (schoolFoodServicesEligibilityDescriptor != null)
                request.AddParameter("schoolFoodServicesEligibilityDescriptor", schoolFoodServicesEligibilityDescriptor);
            if (sexType != null)
                request.AddParameter("sexType", sexType);
            if (birthStateAbbreviationType != null)
                request.AddParameter("birthStateAbbreviationType", birthStateAbbreviationType);
            if (personalTitlePrefix != null)
                request.AddParameter("personalTitlePrefix", personalTitlePrefix);
            if (firstName != null)
                request.AddParameter("firstName", firstName);
            if (middleName != null)
                request.AddParameter("middleName", middleName);
            if (lastSurname != null)
                request.AddParameter("lastSurname", lastSurname);
            if (generationCodeSuffix != null)
                request.AddParameter("generationCodeSuffix", generationCodeSuffix);
            if (maidenName != null)
                request.AddParameter("maidenName", maidenName);
            if (birthDate != null)
                request.AddParameter("birthDate", birthDate);
            if (birthCity != null)
                request.AddParameter("birthCity", birthCity);
            if (birthInternationalProvince != null)
                request.AddParameter("birthInternationalProvince", birthInternationalProvince);
            if (dateEnteredUS != null)
                request.AddParameter("dateEnteredUS", dateEnteredUS);
            if (multipleBirthStatus != null)
                request.AddParameter("multipleBirthStatus", multipleBirthStatus);
            if (profileThumbnail != null)
                request.AddParameter("profileThumbnail", profileThumbnail);
            if (hispanicLatinoEthnicity != null)
                request.AddParameter("hispanicLatinoEthnicity", hispanicLatinoEthnicity);
            if (economicDisadvantaged != null)
                request.AddParameter("economicDisadvantaged", economicDisadvantaged);
            if (displacementStatus != null)
                request.AddParameter("displacementStatus", displacementStatus);
            if (loginId != null)
                request.AddParameter("loginId", loginId);
            if (id != null)
                request.AddParameter("id", id);
            var response = client.Execute<List<FoodServiceStudent>>(request);

            return response;
        }
        /// <summary>
        /// Retrieves a specific resource using the resource's identifier (using the &quot;Get By Key&quot; pattern). This GET operation provides access to resources using the &quot;Get By Key&quot; pattern. The values of the natural key of the resource must be fully specified, and the service will return the matching result (if it exists).
        /// </summary>
        /// <param name="studentUniqueId">A unique alphanumeric code assigned to a student.</param>
        /// <param name="fields">Specifies a subset of properties that should be returned for each entity (e.g. &quot;property1,collection1(collProp1,collProp2)&quot;).</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<FoodServiceStudent> GetFoodServiceStudentsByKey(string studentUniqueId= null, string fields= null, string IfNoneMatch= null) 
        {
            var request = new RestRequest("/foodService/foodServiceStudents", Method.GET);
            request.RequestFormat = DataFormat.Json;

            if (studentUniqueId != null)
                request.AddParameter("studentUniqueId", studentUniqueId);
            if (fields != null)
                request.AddParameter("fields", fields);
            request.AddHeader("If-None-Match", IfNoneMatch);
            var response = client.Execute<FoodServiceStudent>(request);

            return response;
        }
        /// <summary>
        /// Retrieves a specific resource using the resource's identifier (using the &quot;Get By Id&quot; pattern). This GET operation provides access to resources using the &quot;Get By Id&quot; pattern.
        /// </summary>
        /// <param name="id"></param>
        /// <param name="fields">Specifies a subset of properties that should be returned for each entity (e.g. &quot;property1,collection1(collProp1,collProp2)&quot;).</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<FoodServiceStudent> GetFoodServiceStudentsById(string id, string fields= null, string IfNoneMatch= null) 
        {
            var request = new RestRequest("/foodService/foodServiceStudents/{id}", Method.GET);
            request.RequestFormat = DataFormat.Json;

            request.AddUrlSegment("id", id);
            // verify required params are set
            if (id == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            if (fields != null)
                request.AddParameter("fields", fields);
            request.AddHeader("If-None-Match", IfNoneMatch);
            var response = client.Execute<FoodServiceStudent>(request);

            return response;
        }
        }
}

