﻿using System;
using System.Linq;
using RestSharp;
using System.Collections.Generic;
using System.Reflection;

namespace Michigan.Ods.Sdk.Helpers
{
    public class StatisticsTracker
    {
        public StatisticsTracker()
        {
            
        }

        public bool ShouldTrackStatistics
        {
            get { return !string.IsNullOrEmpty(TrackStatisticsUrl); }
        }

        private string _trackStatisticsUrl;
        public string TrackStatisticsUrl {
            get
            {
                if (_trackStatisticsUrl == null)
                    _trackStatisticsUrl = System.Configuration.ConfigurationManager.AppSettings["TrackStatisticsUrl"];
                return _trackStatisticsUrl;
            }
        }

        private IRestClient _restClient;
        public IRestClient RestClient {
            get
            {
                if (!ShouldTrackStatistics)
                    return null;
                if( _restClient == null)
                    _restClient = new RestClient(TrackStatisticsUrl);
                return _restClient;
            }
        }

        public void TrackIfEnabled(SdkUsageRecord sdkUsageRecord)
        {
            if (! ShouldTrackStatistics)
                return;

            var request = new RestRequest("/api/usage-records", Method.POST);
            request.RequestFormat = DataFormat.Json;

            if (sdkUsageRecord == null)
            {
                throw new ArgumentException("API method call is missing required parameters");
            }
            request.AddBody(sdkUsageRecord);
            this.RestClient.ExecuteAsync(request, restResponse => { });
        }
    }
    
    public class SdkUsageRecord
    {
        public int Id { get; set; }

        public string Url { get; set; }

        public string HttpMethod { get; set; }

        public long GetSizeInBytes { get; set; }

        public long PutSizeInBytes { get; set; }

        public long PatchSizeInBytes { get; set; }

        public DateTime DateTimeStamp { get; set; }

    }
}

