﻿using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

namespace Michigan.Ods.Sdk.Models 
{
    public class AccountabilityRating 
    {
        /// <summary>
        /// The unique identifier of the resource.
        /// </summary>
        public string id { get; set; }

        /// <summary>
        /// A reference to the related EducationOrganization resource.
        /// </summary>
        public EducationOrganizationReference educationOrganizationReference { get; set; }

        /// <summary>
        /// A reference to the related SchoolYearType resource.
        /// </summary>
        public SchoolYearTypeReference schoolYearTypeReference { get; set; }

        /// <summary>
        /// An accountability rating level, designation, or assessment.
        /// </summary>
        public string rating { get; set; }

        /// <summary>
        /// The date the rating was awarded.
        /// </summary>
        public DateTime? ratingDate { get; set; }

        /// <summary>
        /// The organization that assessed the rating.
        /// </summary>
        public string ratingOrganization { get; set; }

        /// <summary>
        /// The program associated with the AccountabilityRating (e.g., NCLB, AEIS).
        /// </summary>
        public string ratingProgram { get; set; }

        /// <summary>
        /// The title of the rating (e.g., School Rating, Safety Score).
        /// </summary>
        public string ratingTitle { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        public string _etag { get; set; }

        }
}

