﻿using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

namespace Michigan.Ods.Sdk.Models 
{
    public class Assessment 
    {
        /// <summary>
        /// The unique identifier of the resource.
        /// </summary>
        public string id { get; set; }

        /// <summary>
        /// A reference to the related AssessmentFamily resource.
        /// </summary>
        public AssessmentFamilyReference assessmentFamilyReference { get; set; }

        /// <summary>
        /// The description of the content or subject area (e.g., arts, mathematics, reading, stenography, or a foreign language) of an assessment.
        /// </summary>
        public string academicSubjectDescriptor { get; set; }

        /// <summary>
        /// The typical grade level for which an assessment is designed. If the assessment spans a range of grades, then this attribute holds the highest grade assessed. If only one grade level is assessed then only this attribute is used. For example:          Adult          Prekindergarten          First grade          Second grade          ...
        /// </summary>
        public string assessedGradeLevelDescriptor { get; set; }

        /// <summary>
        /// The category of an assessment based on format and content. For example:          Achievement test          Advanced placement test          Alternate assessment/grade-level standards          Attitudinal test          Cognitive and perceptual skills test          ...
        /// </summary>
        public string categoryDescriptor { get; set; }

        /// <summary>
        /// Identifies the form of the assessment, for example a regular versus makeup form, multiple choice versus constructed response, etc.
        /// </summary>
        public string form { get; set; }

        /// <summary>
        /// The period or window in which an assessment is supposed to be administered.
        /// </summary>
        public string periodDescriptor { get; set; }

        /// <summary>
        /// The title or name of the Assessment.
        /// </summary>
        public string title { get; set; }

        /// <summary>
        /// If the assessment spans a range of grades, then this attribute holds the lowest grade assessed. If only one grade level is assessed, then this attribute is omitted. For example:          Adult          Prekindergarten          First grade          Second grade          ...
        /// </summary>
        public string lowestAssessedGradeLevelDescriptor { get; set; }

        /// <summary>
        /// The maximum raw score achievable across all assessment items that are correct and scored at the maximum.
        /// </summary>
        public int? maxRawScore { get; set; }

        /// <summary>
        /// Namespace for the Assessment.
        /// </summary>
        public string @namespace { get; set; }

        /// <summary>
        /// Reflects the specific nomenclature used for Assessment.
        /// </summary>
        public string nomenclature { get; set; }

        /// <summary>
        /// Form number of Performance Task. NA if Performance Task invalid or not available for grade
        /// </summary>
        public string performanceTaskForm { get; set; }

        /// <summary>
        /// The month, day, and year that the conceptual design for the assessment was most recently revised substantially.
        /// </summary>
        public DateTime? revisionDate { get; set; }

        /// <summary>
        /// The version identifier for the assessment.
        /// </summary>
        public int? version { get; set; }

        /// <summary>
        /// An indication as to whether an assessment conforms to a standard (e.g., local standard, statewide standard, regional standard, association standard).
        /// </summary>
        public AssessmentContentStandard contentStandard { get; set; }

        /// <summary>
        /// An unordered collection of assessmentEssays.  (MI EXT) This identifies the essay associated with an assessment.
        /// </summary>
        public List<AssessmentEssay> essays { get; set; }

        /// <summary>
        /// An unordered collection of assessmentIdentificationCodes.  A unique number or alphanumeric code assigned to an assessment by a school, school system, a state, or other agency or entity.
        /// </summary>
        public List<AssessmentIdentificationCode> identificationCodes { get; set; }

        /// <summary>
        /// An unordered collection of assessmentLanguages.  An indication of the languages in which the assessment is designed.
        /// </summary>
        public List<AssessmentLanguage> languages { get; set; }

        /// <summary>
        /// An unordered collection of assessmentPerformanceLevels.  Definition of the performance levels and the associated cut scores. Three styles are supported:          1. Specification of performance level by minimum and maximum score          2. Specification of performance level by cut score, using only minimum score          3. Specification of performance level without any mapping to scores.
        /// </summary>
        public List<AssessmentPerformanceLevel> performanceLevels { get; set; }

        /// <summary>
        /// An unordered collection of assessmentPrograms.  The programs associated with the Assessment.
        /// </summary>
        public List<AssessmentProgram> programs { get; set; }

        /// <summary>
        /// An unordered collection of assessmentScores.  Definition of the scores to be expected from this assessment.
        /// </summary>
        public List<AssessmentScore> scores { get; set; }

        /// <summary>
        /// An unordered collection of assessmentSections.  The Section(s) to which the Assessment is associated.
        /// </summary>
        public List<AssessmentSection> sections { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        public string _etag { get; set; }

        }
}

