﻿using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

namespace Michigan.Ods.Sdk.Models 
{
    public class InterventionPrescription 
    {
        /// <summary>
        /// The unique identifier of the resource.
        /// </summary>
        public string id { get; set; }

        /// <summary>
        /// A reference to the related EducationOrganization resource.
        /// </summary>
        public EducationOrganizationReference educationOrganizationReference { get; set; }

        /// <summary>
        /// (MI EXT) A statement that provides information regarding the cost of this InterventionPrescription.
        /// </summary>
        public string costStatement { get; set; }

        /// <summary>
        /// (MI EXT) Indicates this InterventionPrescription has been deleted.  0=No  1=Yes  NULL=Unknown
        /// </summary>
        public bool? deleted { get; set; }

        /// <summary>
        /// (MI EXT) Duration of length.
        /// </summary>
        public int? durationLength { get; set; }

        /// <summary>
        /// (MI EXT) Duration of period.
        /// </summary>
        public string durationPeriod { get; set; }

        /// <summary>
        /// (MI EXT) A statement that provides information regarding the eligibility of this Intervention.
        /// </summary>
        public string eligibilityStatement { get; set; }

        /// <summary>
        /// 0=No  1=Yes  NULL=Unknown
        /// </summary>
        public bool? hasCost { get; set; }

        /// <summary>
        /// (MI EXT) 0=No  1=Yes  NULL=Unknown
        /// </summary>
        public bool? hasDuration { get; set; }

        /// <summary>
        /// (MI EXT) 0=No  1=Yes  NULL=Unknown
        /// </summary>
        public bool? hasEligibility { get; set; }

        /// <summary>
        /// The way in which an intervention is used: curriculum, supplement, or practice.
        /// </summary>
        public string interventionClassType { get; set; }

        /// <summary>
        /// (MI EXT) Electronic mail (e-mail) of the Contact for this InterventionPrescription.
        /// </summary>
        public string contactElectronicMailAddress { get; set; }

        /// <summary>
        /// (MI EXT) Name of the Contact for this InterventionPrescription.
        /// </summary>
        public string contactName { get; set; }

        /// <summary>
        /// (MI EXT) Telephone number of the Contact for this InterventionPrescription.
        /// </summary>
        public string contactTelephoneNumber { get; set; }

        /// <summary>
        /// (MI EXT) A description of this InterventionPrescription.
        /// </summary>
        public string description { get; set; }

        /// <summary>
        /// A unique number or alphanumeric code assigned to an intervention prescription.
        /// </summary>
        public string identificationCode { get; set; }

        /// <summary>
        /// (MI EXT) The name of this InterventionPrescription.
        /// </summary>
        public string name { get; set; }

        /// <summary>
        /// (MI EXT) 0 = No  1 = Yes  NULL = Unknown
        /// </summary>
        public bool? locationReferral { get; set; }

        /// <summary>
        /// (MI EXT) Indicates the organizational support.
        /// </summary>
        public string organizationalSupportDescriptor { get; set; }

        /// <summary>
        /// (MI EXT) Indicates the source where Intervention or InterventionPrescription originated.
        /// </summary>
        public string originatingSourceDescriptor { get; set; }

        /// <summary>
        /// (MI EXT) 0 = No  1 = Yes  NULL = Unknown
        /// </summary>
        public bool? parentPermissionRequired { get; set; }

        /// <summary>
        /// (MI EXT) Type of the period.
        /// </summary>
        public string periodType { get; set; }

        /// <summary>
        /// (MI EXT) Indicates the tier.
        /// </summary>
        public string tierDescriptor { get; set; }

        /// <summary>
        /// (MI EXT) 0 = No  1 = Yes
        /// </summary>
        public bool? vettedByPET { get; set; }

        /// <summary>
        /// An unordered collection of interventionPrescriptionAcademicSubsets.  (MI EXT) Indicates the academic subset.
        /// </summary>
        public List<InterventionPrescriptionAcademicSubset> academicSubsets { get; set; }

        /// <summary>
        /// An unordered collection of interventionPrescriptionAppropriateGradeLevels.  Grade levels for the prescribed intervention. If omitted, considered generally applicable.
        /// </summary>
        public List<InterventionPrescriptionAppropriateGradeLevel> appropriateGradeLevels { get; set; }

        /// <summary>
        /// An unordered collection of interventionPrescriptionAppropriateSexes.  Sexes for the InterventionPrescription. If omitted, considered generally applicable.
        /// </summary>
        public List<InterventionPrescriptionAppropriateSex> appropriateSexes { get; set; }

        /// <summary>
        /// An unordered collection of interventionPrescriptionAvailabilityEducationOrganizations.  (MI EXT) Relates the EducationOrganization to the Intervention availability.
        /// </summary>
        public List<InterventionPrescriptionAvailabilityEducationOrganization> availabilityEducationOrganizations { get; set; }

        /// <summary>
        /// An unordered collection of interventionPrescriptionDeliveryMethods.  The way in which an intervention was implemented: individual, small group, whole class, or whole school.
        /// </summary>
        public List<InterventionPrescriptionDeliveryMethod> deliveryMethods { get; set; }

        /// <summary>
        /// An unordered collection of interventionPrescriptionDiagnoses.  Targeted purpose of the InterventionPrescription (e.g., attendance issue, dropout risk).
        /// </summary>
        public List<InterventionPrescriptionDiagnosis> diagnoses { get; set; }

        /// <summary>
        /// An unordered collection of interventionPrescriptionEducationContents.  Relates the education content source to the education content.
        /// </summary>
        public List<InterventionPrescriptionEducationContent> educationContents { get; set; }

        /// <summary>
        /// An unordered collection of interventionPrescriptionInterventionAliases.  (MI EXT) Common alias or nickname of this InterventionPrescription.
        /// </summary>
        public List<InterventionPrescriptionInterventionAlias> interventionAliases { get; set; }

        /// <summary>
        /// An unordered collection of interventionPrescriptionInterventionKeywords.  (MI EXT) keyword for lookup.
        /// </summary>
        public List<InterventionPrescriptionInterventionKeyword> interventionKeywords { get; set; }

        /// <summary>
        /// An unordered collection of interventionPrescriptionLearningResourceMetadataURIs.  The URI (typical a URL) pointing to the metadata entry in a LRMI metadata repository, which describes this content item.
        /// </summary>
        public List<InterventionPrescriptionLearningResourceMetadataURI> learningResourceMetadataURIs { get; set; }

        /// <summary>
        /// An unordered collection of interventionPrescriptionLevelOfEvidences.  (MI EXT) Indicates the level of evidence.
        /// </summary>
        public List<InterventionPrescriptionLevelOfEvidence> levelOfEvidences { get; set; }

        /// <summary>
        /// An unordered collection of interventionPrescriptionPopulationServeds.  A subset of students that are the focus of the InterventionPrescription.
        /// </summary>
        public List<InterventionPrescriptionPopulationServed> populationServeds { get; set; }

        /// <summary>
        /// An unordered collection of interventionPrescriptionSchools.  (MI EXT) The location associated with the InterventionPrescription.
        /// </summary>
        public List<InterventionPrescriptionSchool> schools { get; set; }

        /// <summary>
        /// An unordered collection of interventionPrescriptionURIs.  The URI (typical a URL) pointing to an education content item.
        /// </summary>
        public List<InterventionPrescriptionURI> uris { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        public string _etag { get; set; }

        }
}

