﻿using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

namespace Michigan.Ods.Sdk.Models 
{
    public class OpenStaffPosition 
    {
        /// <summary>
        /// The unique identifier of the resource.
        /// </summary>
        public string id { get; set; }

        /// <summary>
        /// A reference to the related EducationOrganization resource.
        /// </summary>
        public EducationOrganizationReference educationOrganizationReference { get; set; }

        /// <summary>
        /// Date the OpenStaffPosition was posted.
        /// </summary>
        public DateTime? datePosted { get; set; }

        /// <summary>
        /// The date the posting was removed or filled.
        /// </summary>
        public DateTime? datePostingRemoved { get; set; }

        /// <summary>
        /// Reflects the type of employment or contract desired for the position; for example:          Probationary          Contractual          Substitute/temporary          Tenured or permanent          Volunteer/no contract          ...
        /// </summary>
        public string employmentStatusDescriptor { get; set; }

        /// <summary>
        /// The descriptive name of an individual's position.
        /// </summary>
        public string positionTitle { get; set; }

        /// <summary>
        /// Indication of whether the OpenStaffPosition was filled or retired without filling.
        /// </summary>
        public string postingResultType { get; set; }

        /// <summary>
        /// The name of the program for which the OpenStaffPosition will be assigned; for example:          Regular education          Title I-Academic          Title I-Non-Academic          Special Education'          Bilingual/English as a Second Language.
        /// </summary>
        public string programAssignmentDescriptor { get; set; }

        /// <summary>
        /// The number or identifier assigned to an open staff position, typically a requisition number assigned by Human Resources.
        /// </summary>
        public string requisitionNumber { get; set; }

        /// <summary>
        /// The titles of employment, official status, or rank of education staff.
        /// </summary>
        public string staffClassificationDescriptor { get; set; }

        /// <summary>
        /// An unordered collection of openStaffPositionAcademicSubjects.  The teaching field required for the OpenStaffPosition, for example English/Language Arts, Reading, Mathematics, Science, Social Sciences, etc.
        /// </summary>
        public List<OpenStaffPositionAcademicSubject> academicSubjects { get; set; }

        /// <summary>
        /// An unordered collection of openStaffPositionInstructionalGradeLevels.  The set of grade levels for which the position's assignment is responsible.
        /// </summary>
        public List<OpenStaffPositionInstructionalGradeLevel> instructionalGradeLevels { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        public string _etag { get; set; }

        }
}

