﻿using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

namespace Michigan.Ods.Sdk.Models 
{
    public class Program 
    {
        /// <summary>
        /// The unique identifier of the resource.
        /// </summary>
        public string id { get; set; }

        /// <summary>
        /// A reference to the related EducationOrganization resource.
        /// </summary>
        public EducationOrganizationReference educationOrganizationReference { get; set; }

        /// <summary>
        /// A reference to the related EducationOrganization resource.
        /// </summary>
        public EducationOrganizationReference fiscalEntity2EducationOrganizationReference { get; set; }

        /// <summary>
        /// (ELI) Date program begins.
        /// </summary>
        public DateTime? beginDate { get; set; }

        /// <summary>
        /// (ELI) Date program ends.
        /// </summary>
        public DateTime? endDate { get; set; }

        /// <summary>
        /// A unique number or alphanumeric code assigned to a program by a school, school system, a state, or other agency or entity.
        /// </summary>
        public string programId { get; set; }

        /// <summary>
        /// The formal name of the Program of instruction, training, services, or benefits available through federal, state, or local agencies.
        /// </summary>
        public string name { get; set; }

        /// <summary>
        /// Ultimate and intermediate providers of funds for a particular educational or service program or activity, or for an individual's participation in the program or activity (e.g., Federal, State, ESC, District, School, Private Organization).
        /// </summary>
        public string sponsorType { get; set; }

        /// <summary>
        /// The type of program.
        /// </summary>
        public string type { get; set; }

        /// <summary>
        /// An unordered collection of programCharacteristics.  Reflects important characteristics of the Program, such as categories or particular indications.
        /// </summary>
        public List<ProgramCharacteristic> characteristics { get; set; }

        /// <summary>
        /// An unordered collection of programLearningObjectives.  References the LearningObjective(s) with which the Program is associated.
        /// </summary>
        public List<ProgramLearningObjective> learningObjectives { get; set; }

        /// <summary>
        /// An unordered collection of programLearningStandards.  LearningStandard followed by this program.
        /// </summary>
        public List<ProgramLearningStandard> learningStandards { get; set; }

        /// <summary>
        /// An unordered collection of programServices.  Defines the services this program provides to students.
        /// </summary>
        public List<ProgramService> services { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        public string _etag { get; set; }

        }
}

