﻿using System;
using System.Collections.Generic;
using System.Linq;
using Michigan.Ods.Sdk.Helpers;
using Michigan.Ods.Sdk.Profiles.Alert.Models;
using RestSharp;
  using System.Net;
  using System.Text;
  
namespace Michigan.Ods.Sdk.Profiles.Alert.Api 
{
    public class SchoolsApi 
    {
        private readonly IRestClient client;

        public SchoolsApi(IRestClient client)
        {
            this.client = client;
        }
      
        /// <summary>
        /// Retrieves resources based with paging capabilities (using the &quot;Get All&quot; pattern). This GET operation provides access to resources using the &quot;Get All&quot; pattern. In this version of the API there is support for paging.
        /// </summary>
        /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
        /// <param name="minChangeVersion">Used in synchronization to set sequence minimum ChangeVersion</param>
        /// <param name="maxChangeVersion">Used in synchronization to set sequence maximum ChangeVersion</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<List<School_Readable>> GetSchoolsAll(int? offset= null, int? limit= null, int? minChangeVersion= null, int? maxChangeVersion= null) 
        {
            var request = new RestRequest("/schools", Method.GET);
            request.RequestFormat = DataFormat.Json;

            if (offset != null)
                request.AddParameter("offset", offset);
            if (limit != null)
                request.AddParameter("limit", limit);
            if (minChangeVersion != null)
                request.AddParameter("minChangeVersion", minChangeVersion);
            if (maxChangeVersion != null)
                request.AddParameter("maxChangeVersion", maxChangeVersion);
            request.AddHeader("Accept", "application/vnd.ed-fi.school.alert.readable+json");
            var response = client.Execute<List<School_Readable>>(request);

            return response;
        }
        /// <summary>
        /// Retrieves resources matching values of an example resource (using the &quot;Get By Example&quot; pattern). This GET operation provides access to resources using the &quot;Get by Example&quot; search pattern.  The values of any properties of the resource that are specified will be used to return all matching results (if it exists).
        /// </summary>
        /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
        /// <param name="minChangeVersion">Used in synchronization to set sequence minimum ChangeVersion</param>
        /// <param name="maxChangeVersion">Used in synchronization to set sequence maximum ChangeVersion</param>
        /// <param name="schoolId">The identifier assigned to a school by the State Education Agency (SEA).</param>
        /// <param name="localEducationAgencyId">The identifier assigned to a local education agency by the State Education Agency (SEA).</param>
        /// <param name="charterApprovalSchoolYear">The school year in which a charter school was initially approved.</param>
        /// <param name="administrativeFundingControlDescriptor">The type of education institution as classified by its funding source, for example public or private.</param>
        /// <param name="charterApprovalAgencyType">The type of agency that approved the establishment or continuation of a charter school.</param>
        /// <param name="charterStatusType">A school or agency providing free public elementary or secondary education to eligible students under a specific charter granted by the state legislature or other appropriate authority and designated by such authority to be a charter school.</param>
        /// <param name="internetAccessType">The type of Internet access available.</param>
        /// <param name="magnetSpecialProgramEmphasisSchoolType">A school that has been designed: 1) to attract students of different racial/ethnic backgrounds for the purpose of reducing, preventing, or eliminating racial isolation; and/or 2) to provide an academic or social focus on a particular theme (e.g., science/math, performing arts, gifted/talented, or foreign language).</param>
        /// <param name="type">The type of education institution as classified by its primary focus.</param>
        /// <param name="statusDescriptor">(MI EXT) Indicates status (OneRoster orgs.status) of this EdOrg.</param>
        /// <param name="titleIPartASchoolDesignationType">Denotes the Title I Part A designation for the school.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<List<School_Readable>> GetSchoolsByExample(int? offset= null, int? limit= null, int? minChangeVersion= null, int? maxChangeVersion= null, int? schoolId= null, int? localEducationAgencyId= null, int? charterApprovalSchoolYear= null, string administrativeFundingControlDescriptor= null, string charterApprovalAgencyType= null, string charterStatusType= null, string internetAccessType= null, string magnetSpecialProgramEmphasisSchoolType= null, string type= null, string statusDescriptor= null, string titleIPartASchoolDesignationType= null) 
        {
            var request = new RestRequest("/schools", Method.GET);
            request.RequestFormat = DataFormat.Json;

            if (offset != null)
                request.AddParameter("offset", offset);
            if (limit != null)
                request.AddParameter("limit", limit);
            if (minChangeVersion != null)
                request.AddParameter("minChangeVersion", minChangeVersion);
            if (maxChangeVersion != null)
                request.AddParameter("maxChangeVersion", maxChangeVersion);
            if (schoolId != null)
                request.AddParameter("schoolId", schoolId);
            if (localEducationAgencyId != null)
                request.AddParameter("localEducationAgencyId", localEducationAgencyId);
            if (charterApprovalSchoolYear != null)
                request.AddParameter("charterApprovalSchoolYear", charterApprovalSchoolYear);
            if (administrativeFundingControlDescriptor != null)
                request.AddParameter("administrativeFundingControlDescriptor", administrativeFundingControlDescriptor);
            if (charterApprovalAgencyType != null)
                request.AddParameter("charterApprovalAgencyType", charterApprovalAgencyType);
            if (charterStatusType != null)
                request.AddParameter("charterStatusType", charterStatusType);
            if (internetAccessType != null)
                request.AddParameter("internetAccessType", internetAccessType);
            if (magnetSpecialProgramEmphasisSchoolType != null)
                request.AddParameter("magnetSpecialProgramEmphasisSchoolType", magnetSpecialProgramEmphasisSchoolType);
            if (type != null)
                request.AddParameter("type", type);
            if (statusDescriptor != null)
                request.AddParameter("statusDescriptor", statusDescriptor);
            if (titleIPartASchoolDesignationType != null)
                request.AddParameter("titleIPartASchoolDesignationType", titleIPartASchoolDesignationType);
            request.AddHeader("Accept", "application/vnd.ed-fi.school.alert.readable+json");
            var response = client.Execute<List<School_Readable>>(request);

            return response;
        }
        /// <summary>
        /// Retrieves a specific resource using the values of the resource's natural key (using the &quot;Get By Key&quot; pattern). This GET operation provides access to resources using the &quot;Get by Key&quot; search pattern. The values of the natural key of the resource must be fully specified, and the service will return the matching result (if it exists).
        /// </summary>
        /// <param name="schoolId">The identifier assigned to a school by the State Education Agency (SEA).</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<School_Readable> GetSchoolByKey(int? schoolId, string IfNoneMatch= null) 
        {
            var request = new RestRequest("/schools", Method.GET);
            request.RequestFormat = DataFormat.Json;

            // verify required params are set
            if (schoolId == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            if (schoolId != null)
                request.AddParameter("schoolId", schoolId);
            request.AddHeader("If-None-Match", IfNoneMatch);
            request.AddHeader("Accept", "application/vnd.ed-fi.school.alert.readable+json");
            var response = client.Execute<School_Readable>(request);

            return response;
        }
        /// <summary>
        /// Retrieves a specific resource using the resource's identifier (using the &quot;Get By Id&quot; pattern). This GET operation retrieves a resource by the specified resource identifier.
        /// </summary>
        /// <param name="id">A resource identifier specifying the resource to be retrieved.</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<School_Readable> GetSchoolsById(string id, string IfNoneMatch= null) 
        {
            var request = new RestRequest("/schools/{id}", Method.GET);
            request.RequestFormat = DataFormat.Json;

            request.AddUrlSegment("id", id);
            // verify required params are set
            if (id == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            request.AddHeader("If-None-Match", IfNoneMatch);
            request.AddHeader("Accept", "application/vnd.ed-fi.school.alert.readable+json");
            var response = client.Execute<School_Readable>(request);

            return response;
        }
        // patch operation generation was suppressed because profiles can not support patch
    }
}

