﻿using System;
using System.Collections.Generic;
using System.Linq;
using Michigan.Ods.Sdk.Helpers;
using Michigan.Ods.Sdk.Profiles.Alert.Models;
using RestSharp;
  using System.Net;
  using System.Text;
  
namespace Michigan.Ods.Sdk.Profiles.Alert.Api 
{
    public class StudentSchoolAttendanceEventsApi 
    {
        private readonly IRestClient client;

        public StudentSchoolAttendanceEventsApi(IRestClient client)
        {
            this.client = client;
        }
      
        /// <summary>
        /// Retrieves resources based with paging capabilities (using the &quot;Get All&quot; pattern). This GET operation provides access to resources using the &quot;Get All&quot; pattern. In this version of the API there is support for paging.
        /// </summary>
        /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
        /// <param name="minChangeVersion">Used in synchronization to set sequence minimum ChangeVersion</param>
        /// <param name="maxChangeVersion">Used in synchronization to set sequence maximum ChangeVersion</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<List<StudentSchoolAttendanceEvent_Readable>> GetStudentSchoolAttendanceEventsAll(int? offset= null, int? limit= null, int? minChangeVersion= null, int? maxChangeVersion= null) 
        {
            var request = new RestRequest("/studentSchoolAttendanceEvents", Method.GET);
            request.RequestFormat = DataFormat.Json;

            if (offset != null)
                request.AddParameter("offset", offset);
            if (limit != null)
                request.AddParameter("limit", limit);
            if (minChangeVersion != null)
                request.AddParameter("minChangeVersion", minChangeVersion);
            if (maxChangeVersion != null)
                request.AddParameter("maxChangeVersion", maxChangeVersion);
            request.AddHeader("Accept", "application/vnd.ed-fi.studentschoolattendanceevent.alert.readable+json");
            var response = client.Execute<List<StudentSchoolAttendanceEvent_Readable>>(request);

            return response;
        }
        /// <summary>
        /// Retrieves resources matching values of an example resource (using the &quot;Get By Example&quot; pattern). This GET operation provides access to resources using the &quot;Get by Example&quot; search pattern.  The values of any properties of the resource that are specified will be used to return all matching results (if it exists).
        /// </summary>
        /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
        /// <param name="minChangeVersion">Used in synchronization to set sequence minimum ChangeVersion</param>
        /// <param name="maxChangeVersion">Used in synchronization to set sequence maximum ChangeVersion</param>
        /// <param name="attendanceEventCategoryDescriptor">A code describing the attendance event, for example:          Present          Unexcused absence          Excused absence          Tardy.</param>
        /// <param name="eventDate">Date for this attendance event.</param>
        /// <param name="schoolId">The identifier assigned to a school by the State Education Agency (SEA).</param>
        /// <param name="schoolYear">The identifier for the school year.</param>
        /// <param name="termDescriptor">The term for the Session during the school year.</param>
        /// <param name="studentUniqueId">A unique alphanumeric code assigned to a student.</param>
        /// <param name="educationalEnvironmentType">The setting in which a child receives education and related services. This attribute is only used if it differs from the EducationalEnvironment of the Section. This is only used in the AttendanceEvent if different from the associated Section.</param>
        /// <param name="attendanceEventReason">The reported reason for a student's absence.</param>
        /// <param name="id"></param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<List<StudentSchoolAttendanceEvent_Readable>> GetStudentSchoolAttendanceEventsByExample(int? offset= null, int? limit= null, int? minChangeVersion= null, int? maxChangeVersion= null, string attendanceEventCategoryDescriptor= null, DateTime? eventDate= null, int? schoolId= null, int? schoolYear= null, string termDescriptor= null, string studentUniqueId= null, string educationalEnvironmentType= null, string attendanceEventReason= null, string id= null) 
        {
            var request = new RestRequest("/studentSchoolAttendanceEvents", Method.GET);
            request.RequestFormat = DataFormat.Json;

            if (offset != null)
                request.AddParameter("offset", offset);
            if (limit != null)
                request.AddParameter("limit", limit);
            if (minChangeVersion != null)
                request.AddParameter("minChangeVersion", minChangeVersion);
            if (maxChangeVersion != null)
                request.AddParameter("maxChangeVersion", maxChangeVersion);
            if (attendanceEventCategoryDescriptor != null)
                request.AddParameter("attendanceEventCategoryDescriptor", attendanceEventCategoryDescriptor);
            if (eventDate != null)
                request.AddParameter("eventDate", eventDate);
            if (schoolId != null)
                request.AddParameter("schoolId", schoolId);
            if (schoolYear != null)
                request.AddParameter("schoolYear", schoolYear);
            if (termDescriptor != null)
                request.AddParameter("termDescriptor", termDescriptor);
            if (studentUniqueId != null)
                request.AddParameter("studentUniqueId", studentUniqueId);
            if (educationalEnvironmentType != null)
                request.AddParameter("educationalEnvironmentType", educationalEnvironmentType);
            if (attendanceEventReason != null)
                request.AddParameter("attendanceEventReason", attendanceEventReason);
            if (id != null)
                request.AddParameter("id", id);
            request.AddHeader("Accept", "application/vnd.ed-fi.studentschoolattendanceevent.alert.readable+json");
            var response = client.Execute<List<StudentSchoolAttendanceEvent_Readable>>(request);

            return response;
        }
        /// <summary>
        /// Retrieves a specific resource using the values of the resource's natural key (using the &quot;Get By Key&quot; pattern). This GET operation provides access to resources using the &quot;Get by Key&quot; search pattern. The values of the natural key of the resource must be fully specified, and the service will return the matching result (if it exists).
        /// </summary>
        /// <param name="attendanceEventCategoryDescriptor">A code describing the attendance event, for example:          Present          Unexcused absence          Excused absence          Tardy.</param>
        /// <param name="eventDate">Date for this attendance event.</param>
        /// <param name="schoolId">The identifier assigned to a school by the State Education Agency (SEA).</param>
        /// <param name="schoolYear">The identifier for the school year.</param>
        /// <param name="studentUniqueId">A unique alphanumeric code assigned to a student.</param>
        /// <param name="termDescriptor">The term for the Session during the school year.</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<StudentSchoolAttendanceEvent_Readable> GetStudentSchoolAttendanceEventByKey(string attendanceEventCategoryDescriptor, DateTime? eventDate, int? schoolId, int? schoolYear, string studentUniqueId, string termDescriptor, string IfNoneMatch= null) 
        {
            var request = new RestRequest("/studentSchoolAttendanceEvents", Method.GET);
            request.RequestFormat = DataFormat.Json;

            // verify required params are set
            if (attendanceEventCategoryDescriptor == null || eventDate == null || schoolId == null || schoolYear == null || studentUniqueId == null || termDescriptor == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            if (attendanceEventCategoryDescriptor != null)
                request.AddParameter("attendanceEventCategoryDescriptor", attendanceEventCategoryDescriptor);
            if (eventDate != null)
                request.AddParameter("eventDate", eventDate);
            if (schoolId != null)
                request.AddParameter("schoolId", schoolId);
            if (schoolYear != null)
                request.AddParameter("schoolYear", schoolYear);
            if (studentUniqueId != null)
                request.AddParameter("studentUniqueId", studentUniqueId);
            if (termDescriptor != null)
                request.AddParameter("termDescriptor", termDescriptor);
            request.AddHeader("If-None-Match", IfNoneMatch);
            request.AddHeader("Accept", "application/vnd.ed-fi.studentschoolattendanceevent.alert.readable+json");
            var response = client.Execute<StudentSchoolAttendanceEvent_Readable>(request);

            return response;
        }
        /// <summary>
        /// Retrieves a specific resource using the resource's identifier (using the &quot;Get By Id&quot; pattern). This GET operation retrieves a resource by the specified resource identifier.
        /// </summary>
        /// <param name="id">A resource identifier specifying the resource to be retrieved.</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<StudentSchoolAttendanceEvent_Readable> GetStudentSchoolAttendanceEventsById(string id, string IfNoneMatch= null) 
        {
            var request = new RestRequest("/studentSchoolAttendanceEvents/{id}", Method.GET);
            request.RequestFormat = DataFormat.Json;

            request.AddUrlSegment("id", id);
            // verify required params are set
            if (id == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            request.AddHeader("If-None-Match", IfNoneMatch);
            request.AddHeader("Accept", "application/vnd.ed-fi.studentschoolattendanceevent.alert.readable+json");
            var response = client.Execute<StudentSchoolAttendanceEvent_Readable>(request);

            return response;
        }
        // patch operation generation was suppressed because profiles can not support patch
    }
}

