﻿using System;
using System.Collections.Generic;
using System.Linq;
using Michigan.Ods.Sdk.Helpers;
using Michigan.Ods.Sdk.Profiles.Assessment.Models;
using RestSharp;
  using System.Net;
  using System.Text;
  
namespace Michigan.Ods.Sdk.Profiles.Assessment.Api 
{
    public class SectionsApi 
    {
        private readonly IRestClient client;

        public SectionsApi(IRestClient client)
        {
            this.client = client;
        }
      
        /// <summary>
        /// Retrieves resources based with paging capabilities (using the &quot;Get All&quot; pattern). This GET operation provides access to resources using the &quot;Get All&quot; pattern. In this version of the API there is support for paging.
        /// </summary>
        /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
        /// <param name="minChangeVersion">Used in synchronization to set sequence minimum ChangeVersion</param>
        /// <param name="maxChangeVersion">Used in synchronization to set sequence maximum ChangeVersion</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<List<Section_Readable>> GetSectionsAll(int? offset= null, int? limit= null, int? minChangeVersion= null, int? maxChangeVersion= null) 
        {
            var request = new RestRequest("/sections", Method.GET);
            request.RequestFormat = DataFormat.Json;

            if (offset != null)
                request.AddParameter("offset", offset);
            if (limit != null)
                request.AddParameter("limit", limit);
            if (minChangeVersion != null)
                request.AddParameter("minChangeVersion", minChangeVersion);
            if (maxChangeVersion != null)
                request.AddParameter("maxChangeVersion", maxChangeVersion);
            request.AddHeader("Accept", "application/vnd.ed-fi.section.assessment.readable+json");
            var response = client.Execute<List<Section_Readable>>(request);

            return response;
        }
        /// <summary>
        /// Retrieves resources matching values of an example resource (using the &quot;Get By Example&quot; pattern). This GET operation provides access to resources using the &quot;Get by Example&quot; search pattern.  The values of any properties of the resource that are specified will be used to return all matching results (if it exists).
        /// </summary>
        /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
        /// <param name="minChangeVersion">Used in synchronization to set sequence minimum ChangeVersion</param>
        /// <param name="maxChangeVersion">Used in synchronization to set sequence maximum ChangeVersion</param>
        /// <param name="uniqueSectionCode">A unique identifier for the Section that is defined by the classroom, the subjects taught, and the instructors who are assigned.</param>
        /// <param name="sequenceOfCourse">When a section is part of a sequence of parts for a course, the number of the sequence. If the course has only one part, the value of this section attribute should be 1.</param>
        /// <param name="schoolId">The identifier assigned to a school by the State Education Agency (SEA).</param>
        /// <param name="classPeriodName">An indication of the portion of a typical daily session in which students receive instruction in a specified subject (e.g., morning, sixth period, block period, or AB schedules).</param>
        /// <param name="localCourseCode">The local code assigned by the School that identifies the course offering provided for the instruction of students.</param>
        /// <param name="schoolYear">The identifier for the school year.</param>
        /// <param name="termDescriptor">The term for the Session during the school year.</param>
        /// <param name="classroomIdentificationCode">A unique number or alphanumeric code assigned to a room by a school, school system, state, or other agency or entity.</param>
        /// <param name="availableCreditType">The type of credits or units of value awarded for the completion of a course.</param>
        /// <param name="educationalEnvironmentType">The setting in which a child receives education and related services; for example:          Center-based instruction          Home-based instruction          Hospital class          Mainstream          Residential care and treatment facility          ...</param>
        /// <param name="instructionLanguageDescriptor">The primary language of instruction, if omitted English is assumed.</param>
        /// <param name="mediumOfInstructionType">The media through which teachers provide instruction to students and students and teachers communicate about instructional matters; for example:          Technology-based instruction in classroom          Correspondence instruction          Face-to-face instruction          Virtual/On-line Distance learning          Center-based instruction          ...</param>
        /// <param name="populationServedType">The type of students the Section is offered and tailored to; for example:          Bilingual students          Remedial education students          Gifted and talented students          Career and Technical Education students          Special education students          ...</param>
        /// <param name="availableCredits">The value of credits or units of value awarded for the completion of a course.</param>
        /// <param name="availableCreditConversion">Conversion factor that when multiplied by the number of credits is equivalent to Carnegie units.</param>
        /// <param name="id"></param>
        /// <param name="msixCourseSectionDescriptor">(MI-EXT) This characteristic is required when reporting migrant students by educating entities when the Local Course ID cannot uniquely identify a given course.</param>
        /// <param name="statusDescriptor">(MI EXT) Indicates the status (OneRoster.classes.status) of the Section.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<List<Section_Readable>> GetSectionsByExample(int? offset= null, int? limit= null, int? minChangeVersion= null, int? maxChangeVersion= null, string uniqueSectionCode= null, int? sequenceOfCourse= null, int? schoolId= null, string classPeriodName= null, string localCourseCode= null, int? schoolYear= null, string termDescriptor= null, string classroomIdentificationCode= null, string availableCreditType= null, string educationalEnvironmentType= null, string instructionLanguageDescriptor= null, string mediumOfInstructionType= null, string populationServedType= null, double? availableCredits= null, double? availableCreditConversion= null, string id= null, string msixCourseSectionDescriptor= null, string statusDescriptor= null) 
        {
            var request = new RestRequest("/sections", Method.GET);
            request.RequestFormat = DataFormat.Json;

            if (offset != null)
                request.AddParameter("offset", offset);
            if (limit != null)
                request.AddParameter("limit", limit);
            if (minChangeVersion != null)
                request.AddParameter("minChangeVersion", minChangeVersion);
            if (maxChangeVersion != null)
                request.AddParameter("maxChangeVersion", maxChangeVersion);
            if (uniqueSectionCode != null)
                request.AddParameter("uniqueSectionCode", uniqueSectionCode);
            if (sequenceOfCourse != null)
                request.AddParameter("sequenceOfCourse", sequenceOfCourse);
            if (schoolId != null)
                request.AddParameter("schoolId", schoolId);
            if (classPeriodName != null)
                request.AddParameter("classPeriodName", classPeriodName);
            if (localCourseCode != null)
                request.AddParameter("localCourseCode", localCourseCode);
            if (schoolYear != null)
                request.AddParameter("schoolYear", schoolYear);
            if (termDescriptor != null)
                request.AddParameter("termDescriptor", termDescriptor);
            if (classroomIdentificationCode != null)
                request.AddParameter("classroomIdentificationCode", classroomIdentificationCode);
            if (availableCreditType != null)
                request.AddParameter("availableCreditType", availableCreditType);
            if (educationalEnvironmentType != null)
                request.AddParameter("educationalEnvironmentType", educationalEnvironmentType);
            if (instructionLanguageDescriptor != null)
                request.AddParameter("instructionLanguageDescriptor", instructionLanguageDescriptor);
            if (mediumOfInstructionType != null)
                request.AddParameter("mediumOfInstructionType", mediumOfInstructionType);
            if (populationServedType != null)
                request.AddParameter("populationServedType", populationServedType);
            if (availableCredits != null)
                request.AddParameter("availableCredits", availableCredits);
            if (availableCreditConversion != null)
                request.AddParameter("availableCreditConversion", availableCreditConversion);
            if (id != null)
                request.AddParameter("id", id);
            if (msixCourseSectionDescriptor != null)
                request.AddParameter("msixCourseSectionDescriptor", msixCourseSectionDescriptor);
            if (statusDescriptor != null)
                request.AddParameter("statusDescriptor", statusDescriptor);
            request.AddHeader("Accept", "application/vnd.ed-fi.section.assessment.readable+json");
            var response = client.Execute<List<Section_Readable>>(request);

            return response;
        }
        /// <summary>
        /// Retrieves a specific resource using the values of the resource's natural key (using the &quot;Get By Key&quot; pattern). This GET operation provides access to resources using the &quot;Get by Key&quot; search pattern. The values of the natural key of the resource must be fully specified, and the service will return the matching result (if it exists).
        /// </summary>
        /// <param name="classPeriodName">An indication of the portion of a typical daily session in which students receive instruction in a specified subject (e.g., morning, sixth period, block period, or AB schedules).</param>
        /// <param name="classroomIdentificationCode">A unique number or alphanumeric code assigned to a room by a school, school system, state, or other agency or entity.</param>
        /// <param name="localCourseCode">The local code assigned by the School that identifies the course offering provided for the instruction of students.</param>
        /// <param name="schoolId">The identifier assigned to a school by the State Education Agency (SEA).</param>
        /// <param name="schoolYear">The identifier for the school year.</param>
        /// <param name="sequenceOfCourse">When a section is part of a sequence of parts for a course, the number of the sequence. If the course has only one part, the value of this section attribute should be 1.</param>
        /// <param name="termDescriptor">The term for the Session during the school year.</param>
        /// <param name="uniqueSectionCode">A unique identifier for the Section that is defined by the classroom, the subjects taught, and the instructors who are assigned.</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<Section_Readable> GetSectionByKey(string classPeriodName, string classroomIdentificationCode, string localCourseCode, int? schoolId, int? schoolYear, int? sequenceOfCourse, string termDescriptor, string uniqueSectionCode, string IfNoneMatch= null) 
        {
            var request = new RestRequest("/sections", Method.GET);
            request.RequestFormat = DataFormat.Json;

            // verify required params are set
            if (classPeriodName == null || classroomIdentificationCode == null || localCourseCode == null || schoolId == null || schoolYear == null || sequenceOfCourse == null || termDescriptor == null || uniqueSectionCode == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            if (classPeriodName != null)
                request.AddParameter("classPeriodName", classPeriodName);
            if (classroomIdentificationCode != null)
                request.AddParameter("classroomIdentificationCode", classroomIdentificationCode);
            if (localCourseCode != null)
                request.AddParameter("localCourseCode", localCourseCode);
            if (schoolId != null)
                request.AddParameter("schoolId", schoolId);
            if (schoolYear != null)
                request.AddParameter("schoolYear", schoolYear);
            if (sequenceOfCourse != null)
                request.AddParameter("sequenceOfCourse", sequenceOfCourse);
            if (termDescriptor != null)
                request.AddParameter("termDescriptor", termDescriptor);
            if (uniqueSectionCode != null)
                request.AddParameter("uniqueSectionCode", uniqueSectionCode);
            request.AddHeader("If-None-Match", IfNoneMatch);
            request.AddHeader("Accept", "application/vnd.ed-fi.section.assessment.readable+json");
            var response = client.Execute<Section_Readable>(request);

            return response;
        }
        /// <summary>
        /// Retrieves a specific resource using the resource's identifier (using the &quot;Get By Id&quot; pattern). This GET operation retrieves a resource by the specified resource identifier.
        /// </summary>
        /// <param name="id">A resource identifier specifying the resource to be retrieved.</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<Section_Readable> GetSectionsById(string id, string IfNoneMatch= null) 
        {
            var request = new RestRequest("/sections/{id}", Method.GET);
            request.RequestFormat = DataFormat.Json;

            request.AddUrlSegment("id", id);
            // verify required params are set
            if (id == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            request.AddHeader("If-None-Match", IfNoneMatch);
            request.AddHeader("Accept", "application/vnd.ed-fi.section.assessment.readable+json");
            var response = client.Execute<Section_Readable>(request);

            return response;
        }
        // patch operation generation was suppressed because profiles can not support patch
    }
}

