﻿using System;
using System.Collections.Generic;
using System.Linq;
using Michigan.Ods.Sdk.Helpers;
using Michigan.Ods.Sdk.Profiles.Assessment.Models;
using RestSharp;
  using System.Net;
  using System.Text;
  
namespace Michigan.Ods.Sdk.Profiles.Assessment.Api 
{
    public class StudentSchoolAssociationsApi 
    {
        private readonly IRestClient client;

        public StudentSchoolAssociationsApi(IRestClient client)
        {
            this.client = client;
        }
      
        /// <summary>
        /// Retrieves resources based with paging capabilities (using the &quot;Get All&quot; pattern). This GET operation provides access to resources using the &quot;Get All&quot; pattern. In this version of the API there is support for paging.
        /// </summary>
        /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
        /// <param name="minChangeVersion">Used in synchronization to set sequence minimum ChangeVersion</param>
        /// <param name="maxChangeVersion">Used in synchronization to set sequence maximum ChangeVersion</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<List<StudentSchoolAssociation_Readable>> GetStudentSchoolAssociationsAll(int? offset= null, int? limit= null, int? minChangeVersion= null, int? maxChangeVersion= null) 
        {
            var request = new RestRequest("/studentSchoolAssociations", Method.GET);
            request.RequestFormat = DataFormat.Json;

            if (offset != null)
                request.AddParameter("offset", offset);
            if (limit != null)
                request.AddParameter("limit", limit);
            if (minChangeVersion != null)
                request.AddParameter("minChangeVersion", minChangeVersion);
            if (maxChangeVersion != null)
                request.AddParameter("maxChangeVersion", maxChangeVersion);
            request.AddHeader("Accept", "application/vnd.ed-fi.studentschoolassociation.assessment.readable+json");
            var response = client.Execute<List<StudentSchoolAssociation_Readable>>(request);

            return response;
        }
        /// <summary>
        /// Retrieves resources matching values of an example resource (using the &quot;Get By Example&quot; pattern). This GET operation provides access to resources using the &quot;Get by Example&quot; search pattern.  The values of any properties of the resource that are specified will be used to return all matching results (if it exists).
        /// </summary>
        /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
        /// <param name="minChangeVersion">Used in synchronization to set sequence minimum ChangeVersion</param>
        /// <param name="maxChangeVersion">Used in synchronization to set sequence maximum ChangeVersion</param>
        /// <param name="entryDate">The month, day, and year on which an individual enters and begins to receive instructional services in a school.</param>
        /// <param name="schoolId">The identifier assigned to a school by the State Education Agency (SEA).</param>
        /// <param name="studentUniqueId">A unique alphanumeric code assigned to a student.</param>
        /// <param name="graduationPlanTypeDescriptor">The type of academic plan the student is following for graduation: for example, Minimum, Recommended, Distinguished, or Standard.</param>
        /// <param name="educationOrganizationId">The identifier assigned to an education agency by the State Education Agency (SEA).  Also known as the State LEA ID.</param>
        /// <param name="graduationSchoolYear">The school year the student is expected to graduate.</param>
        /// <param name="bldgOtherwiseAttendSchoolId">The identifier assigned to a school by the State Education Agency (SEA).</param>
        /// <param name="schoolYear">The school year associated with the student's enrollment.</param>
        /// <param name="classOfSchoolYear">Projected High School graduation year.</param>
        /// <param name="entryGradeLevelReasonType">The primary reason as to why a staff member determined that a student should be promoted or not (or be demoted) at the end of a given school term.</param>
        /// <param name="entryTypeDescriptor">The process by which a student enters a school during a given academic session.</param>
        /// <param name="exitWithdrawTypeDescriptor">The circumstances under which the student exited from membership in an educational institution.</param>
        /// <param name="entryGradeLevelDescriptor">The grade level or primary instructional level at which a student enters and receives services in a school or an educational institution during a given academic session.</param>
        /// <param name="residencyStatusDescriptor">An indication of the location of a persons legal residence relative to (within or outside of) the boundaries of the public school attended and its administrative unit.</param>
        /// <param name="primarySchool">Indicates if a given enrollment record should be considered the primary record for a student. If omitted, the default is true.</param>
        /// <param name="repeatGradeIndicator">An indicator of whether the student is enrolling to repeat a grade level, either by failure or an agreement to hold the student back.</param>
        /// <param name="schoolChoiceTransfer">An indication of whether students transferred in or out of the school did so during the school year under the provisions for public school choice in accordance with Title I, Part A, Section 1116.</param>
        /// <param name="exitWithdrawDate">The month, day, and year of the first day after the date of an individual's last attendance at a school (if known), the day on which an individual graduated, or the date on which it becomes known officially that an individual left school.</param>
        /// <param name="employedWhileEnrolled">An individual who is a paid employee or works in his or her own business, profession, or farm and at the same time is enrolled in secondary, postsecondary, or adult education.</param>
        /// <param name="id"></param>
        /// <param name="districtEntryDate">(MI EXT) The date the student was first enrolled in the District.</param>
        /// <param name="districtResidencyStatusAsOfDate">(MI EXT) To indicate the date of the last change to district residency status.</param>
        /// <param name="districtResidencyStatusDescriptor">(MI EXT) To indicate the student s residency related to the geographic boundaries of the district and/or residency requirements outlined in the Pupil Residency section of the MDE Pupil Accounting Manual.</param>
        /// <param name="enrollmentPercent">(Mi EXT) The pro rata share for which the student receives educational programs and/or services from the operating district.</param>
        /// <param name="exitWithdrawDateAsOfDate">(MI EXT) To support state reporting: captures when ExitWithdrawDate changed.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<List<StudentSchoolAssociation_Readable>> GetStudentSchoolAssociationsByExample(int? offset= null, int? limit= null, int? minChangeVersion= null, int? maxChangeVersion= null, DateTime? entryDate= null, int? schoolId= null, string studentUniqueId= null, string graduationPlanTypeDescriptor= null, int? educationOrganizationId= null, int? graduationSchoolYear= null, int? bldgOtherwiseAttendSchoolId= null, int? schoolYear= null, int? classOfSchoolYear= null, string entryGradeLevelReasonType= null, string entryTypeDescriptor= null, string exitWithdrawTypeDescriptor= null, string entryGradeLevelDescriptor= null, string residencyStatusDescriptor= null, bool? primarySchool= null, bool? repeatGradeIndicator= null, bool? schoolChoiceTransfer= null, DateTime? exitWithdrawDate= null, bool? employedWhileEnrolled= null, string id= null, DateTime? districtEntryDate= null, DateTime? districtResidencyStatusAsOfDate= null, string districtResidencyStatusDescriptor= null, double? enrollmentPercent= null, DateTime? exitWithdrawDateAsOfDate= null) 
        {
            var request = new RestRequest("/studentSchoolAssociations", Method.GET);
            request.RequestFormat = DataFormat.Json;

            if (offset != null)
                request.AddParameter("offset", offset);
            if (limit != null)
                request.AddParameter("limit", limit);
            if (minChangeVersion != null)
                request.AddParameter("minChangeVersion", minChangeVersion);
            if (maxChangeVersion != null)
                request.AddParameter("maxChangeVersion", maxChangeVersion);
            if (entryDate != null)
                request.AddParameter("entryDate", entryDate);
            if (schoolId != null)
                request.AddParameter("schoolId", schoolId);
            if (studentUniqueId != null)
                request.AddParameter("studentUniqueId", studentUniqueId);
            if (graduationPlanTypeDescriptor != null)
                request.AddParameter("graduationPlanTypeDescriptor", graduationPlanTypeDescriptor);
            if (educationOrganizationId != null)
                request.AddParameter("educationOrganizationId", educationOrganizationId);
            if (graduationSchoolYear != null)
                request.AddParameter("graduationSchoolYear", graduationSchoolYear);
            if (bldgOtherwiseAttendSchoolId != null)
                request.AddParameter("bldgOtherwiseAttendSchoolId", bldgOtherwiseAttendSchoolId);
            if (schoolYear != null)
                request.AddParameter("schoolYear", schoolYear);
            if (classOfSchoolYear != null)
                request.AddParameter("classOfSchoolYear", classOfSchoolYear);
            if (entryGradeLevelReasonType != null)
                request.AddParameter("entryGradeLevelReasonType", entryGradeLevelReasonType);
            if (entryTypeDescriptor != null)
                request.AddParameter("entryTypeDescriptor", entryTypeDescriptor);
            if (exitWithdrawTypeDescriptor != null)
                request.AddParameter("exitWithdrawTypeDescriptor", exitWithdrawTypeDescriptor);
            if (entryGradeLevelDescriptor != null)
                request.AddParameter("entryGradeLevelDescriptor", entryGradeLevelDescriptor);
            if (residencyStatusDescriptor != null)
                request.AddParameter("residencyStatusDescriptor", residencyStatusDescriptor);
            if (primarySchool != null)
                request.AddParameter("primarySchool", primarySchool);
            if (repeatGradeIndicator != null)
                request.AddParameter("repeatGradeIndicator", repeatGradeIndicator);
            if (schoolChoiceTransfer != null)
                request.AddParameter("schoolChoiceTransfer", schoolChoiceTransfer);
            if (exitWithdrawDate != null)
                request.AddParameter("exitWithdrawDate", exitWithdrawDate);
            if (employedWhileEnrolled != null)
                request.AddParameter("employedWhileEnrolled", employedWhileEnrolled);
            if (id != null)
                request.AddParameter("id", id);
            if (districtEntryDate != null)
                request.AddParameter("districtEntryDate", districtEntryDate);
            if (districtResidencyStatusAsOfDate != null)
                request.AddParameter("districtResidencyStatusAsOfDate", districtResidencyStatusAsOfDate);
            if (districtResidencyStatusDescriptor != null)
                request.AddParameter("districtResidencyStatusDescriptor", districtResidencyStatusDescriptor);
            if (enrollmentPercent != null)
                request.AddParameter("enrollmentPercent", enrollmentPercent);
            if (exitWithdrawDateAsOfDate != null)
                request.AddParameter("exitWithdrawDateAsOfDate", exitWithdrawDateAsOfDate);
            request.AddHeader("Accept", "application/vnd.ed-fi.studentschoolassociation.assessment.readable+json");
            var response = client.Execute<List<StudentSchoolAssociation_Readable>>(request);

            return response;
        }
        /// <summary>
        /// Retrieves a specific resource using the values of the resource's natural key (using the &quot;Get By Key&quot; pattern). This GET operation provides access to resources using the &quot;Get by Key&quot; search pattern. The values of the natural key of the resource must be fully specified, and the service will return the matching result (if it exists).
        /// </summary>
        /// <param name="entryDate">The month, day, and year on which an individual enters and begins to receive instructional services in a school.</param>
        /// <param name="schoolId">The identifier assigned to a school by the State Education Agency (SEA).</param>
        /// <param name="studentUniqueId">A unique alphanumeric code assigned to a student.</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<StudentSchoolAssociation_Readable> GetStudentSchoolAssociationByKey(DateTime? entryDate, int? schoolId, string studentUniqueId, string IfNoneMatch= null) 
        {
            var request = new RestRequest("/studentSchoolAssociations", Method.GET);
            request.RequestFormat = DataFormat.Json;

            // verify required params are set
            if (entryDate == null || schoolId == null || studentUniqueId == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            if (entryDate != null)
                request.AddParameter("entryDate", entryDate);
            if (schoolId != null)
                request.AddParameter("schoolId", schoolId);
            if (studentUniqueId != null)
                request.AddParameter("studentUniqueId", studentUniqueId);
            request.AddHeader("If-None-Match", IfNoneMatch);
            request.AddHeader("Accept", "application/vnd.ed-fi.studentschoolassociation.assessment.readable+json");
            var response = client.Execute<StudentSchoolAssociation_Readable>(request);

            return response;
        }
        /// <summary>
        /// Retrieves a specific resource using the resource's identifier (using the &quot;Get By Id&quot; pattern). This GET operation retrieves a resource by the specified resource identifier.
        /// </summary>
        /// <param name="id">A resource identifier specifying the resource to be retrieved.</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<StudentSchoolAssociation_Readable> GetStudentSchoolAssociationsById(string id, string IfNoneMatch= null) 
        {
            var request = new RestRequest("/studentSchoolAssociations/{id}", Method.GET);
            request.RequestFormat = DataFormat.Json;

            request.AddUrlSegment("id", id);
            // verify required params are set
            if (id == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            request.AddHeader("If-None-Match", IfNoneMatch);
            request.AddHeader("Accept", "application/vnd.ed-fi.studentschoolassociation.assessment.readable+json");
            var response = client.Execute<StudentSchoolAssociation_Readable>(request);

            return response;
        }
        // patch operation generation was suppressed because profiles can not support patch
    }
}

