﻿using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

namespace Michigan.Ods.Sdk.Profiles.Assessment.Models 
{
    public class AssessmentFamily_Readable 
    {
        /// <summary>
        /// The unique identifier of the resource.
        /// </summary>
        public string id { get; set; }

        /// <summary>
        /// A reference to the related AssessmentFamily resource.
        /// </summary>
        public AssessmentFamilyReference parentAssessmentFamilyReference { get; set; }

        /// <summary>
        /// The description of the content or subject area (e.g., arts, mathematics, reading, stenography, or a foreign language) of an assessment family.
        /// </summary>
        public string academicSubjectDescriptor { get; set; }

        /// <summary>
        /// The typical grade level for which an assessment family is designed. If the assessment spans a range of grades, then this attribute holds the highest grade assessed. If only one grade level is assessed then only this attribute is used. For example: Adult, Prekindergarten, First grade, Second grade, etc.
        /// </summary>
        public string assessedGradeLevelDescriptor { get; set; }

        /// <summary>
        /// The category of an assessment based on format and content. For example: Achievement test, Advanced placement test, Alternate assessment/grade-level standards, Attitudinal test, Cognitive, and perceptual skills test, etc.
        /// </summary>
        public string assessmentCategoryDescriptor { get; set; }

        /// <summary>
        /// The title or name of the AssessmentFamily.
        /// </summary>
        public string title { get; set; }

        /// <summary>
        /// If the assessment spans a range of grades, then this attribute holds the lowest grade assessed. If only one grade level is assessed, then this attribute is omitted.
        /// </summary>
        public string lowestAssessedGradeLevelDescriptor { get; set; }

        /// <summary>
        /// Namespace for the Assessments in this AssessmentFamily.
        /// </summary>
        public string @namespace { get; set; }

        /// <summary>
        /// Reflects the specific nomenclature used for this level of AssessmentFamily.
        /// </summary>
        public string nomenclature { get; set; }

        /// <summary>
        /// The month, day, and year that the conceptual design for the AssessmentFamily was most recently revised substantially.
        /// </summary>
        public DateTime? revisionDate { get; set; }

        /// <summary>
        /// The version identifier for the AssessmentFamily.
        /// </summary>
        public int? version { get; set; }

        /// <summary>
        /// An indication as to whether an assessment conforms to a standard (e.g., local standard, statewide standard, regional standard, association standard).
        /// </summary>
        public AssessmentFamilyContentStandard_Readable contentStandard { get; set; }

        /// <summary>
        /// An unordered collection of assessmentFamilyAssessmentPeriods.  The periods or windows defined in which an assessment is supposed to be administered.
        /// </summary>
        public List<AssessmentFamilyAssessmentPeriod_Readable> assessmentPeriods { get; set; }

        /// <summary>
        /// An unordered collection of assessmentFamilyIdentificationCodes.  A unique number or alphanumeric code assigned to an assessment family by a school, school system, a state, or other agency or entity.
        /// </summary>
        public List<AssessmentFamilyIdentificationCode_Readable> identificationCodes { get; set; }

        /// <summary>
        /// An unordered collection of assessmentFamilyLanguages.  An indication of the languages in which the AssessmentFamily is designed.
        /// </summary>
        public List<AssessmentFamilyLanguage_Readable> languages { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        public string _etag { get; set; }

        }
}

