﻿using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

namespace Michigan.Ods.Sdk.Profiles.Assessment.Models 
{
    public class AssessmentItem_Readable 
    {
        /// <summary>
        /// The unique identifier of the resource.
        /// </summary>
        public string id { get; set; }

        /// <summary>
        /// A reference to the related Assessment resource.
        /// </summary>
        public AssessmentReference assessmentReference { get; set; }

        /// <summary>
        /// Category or type of the AssessmentItem. For example:          Multiple choice          Analytic          Prose          ...
        /// </summary>
        public string categoryType { get; set; }

        /// <summary>
        /// (MI EXT) Identifies the form of the assessment item, for example a regular versus makeup form, multiple choice versus constructed response, etc.
        /// </summary>
        public string form { get; set; }

        /// <summary>
        /// The correct response for the AssessmentItem.
        /// </summary>
        public string correctResponse { get; set; }

        /// <summary>
        /// The duration of time allotted for the AssessmentItem.
        /// </summary>
        public string expectedTimeAssessed { get; set; }

        /// <summary>
        /// A unique number or alphanumeric code assigned to a space, room, site, building, individual, organization, program, or institution by a school, school system, state, or other agency or entity.
        /// </summary>
        public string identificationCode { get; set; }

        /// <summary>
        /// The maximum raw score achievable across all assessment items that are correct and scored at the maximum.
        /// </summary>
        public int? maxRawScore { get; set; }

        /// <summary>
        /// Reflects the specific nomenclature used for AssessmentItem.
        /// </summary>
        public string nomenclature { get; set; }

        /// <summary>
        /// An unordered collection of assessmentItemLearningStandards.  LearningStandard tested by this item.
        /// </summary>
        public List<AssessmentItemLearningStandard_Readable> learningStandards { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        public string _etag { get; set; }

        }
}

