﻿using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

namespace Michigan.Ods.Sdk.Profiles.Assessment.Models 
{
    public class Course_Readable 
    {
        /// <summary>
        /// The unique identifier of the resource.
        /// </summary>
        public string id { get; set; }

        /// <summary>
        /// A reference to the related EducationOrganization resource.
        /// </summary>
        public EducationOrganizationReference educationOrganizationReference { get; set; }

        /// <summary>
        /// The intended major subject area of the course.
        /// </summary>
        public string academicSubjectDescriptor { get; set; }

        /// <summary>
        /// A unique alphanumeric code assigned to a course.
        /// </summary>
        public string code { get; set; }

        /// <summary>
        /// The descriptive name given to a course of study offered in a school or other institution or organization. In departmentalized classes at the elementary, secondary, and postsecondary levels (and for staff development activities), this refers to the name by which a course is identified (e.g., American History, English III). For elementary and other non-departmentalized classes, it refers to any portion of the instruction for which a grade or report is assigned (e.g., reading, composition, spelling, and language arts).
        /// </summary>
        public string title { get; set; }

        /// <summary>
        /// (MI EXT) Indicates the level and rigor of the instruction provided throughout the reported course. Code 07 is to be used for college instructed courses for which the student will receive high school credit. The student may be participating in an Early/Middle College program or a dual enrollment program.
        /// </summary>
        public string typeLevelAndRigorDescriptor { get; set; }

        /// <summary>
        /// (MI EXT) This course is eligible for the 64b dual enrollment funding incentive.
        /// </summary>
        public bool? eligible64bCourse { get; set; }

        /// <summary>
        /// (MI EXT) Indicates status (OneRoster courses.status) of the Course.
        /// </summary>
        public string statusDescriptor { get; set; }

        /// <summary>
        /// An unordered collection of courseIdentificationCodes.  The code that identifies the organization of subject matter and related learning experiences provided for the instruction of students.
        /// </summary>
        public List<CourseIdentificationCode_Readable> identificationCodes { get; set; }

        /// <summary>
        /// An unordered collection of courseLevelCharacteristics.  The type of specific program or designation with which the course is associated (e.g., AP, IB, Dual Credit, CTE).
        /// </summary>
        public List<CourseLevelCharacteristic_Readable> courseLevelCharacteristics { get; set; }

        /// <summary>
        /// An unordered collection of courseOfferedGradeLevels.  The grade levels in which the course is offered.
        /// </summary>
        public List<CourseOfferedGradeLevel_Readable> offeredGradeLevels { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        public string _etag { get; set; }

        }
}

