﻿using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

namespace Michigan.Ods.Sdk.Profiles.Assessment.Models 
{
    public class StudentAssessmentTimeOfTestingSnapshot_Writable 
    {
        /// <summary>
        /// A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table.
        /// </summary>
        public string primaryDisabilityDescriptor { get; set; }

        /// <summary>
        /// A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table.
        /// </summary>
        public string secondaryDisabilityDescriptor { get; set; }

        /// <summary>
        /// A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table.
        /// </summary>
        public string primaryLanguageDescriptor { get; set; }

        /// <summary>
        /// A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table.
        /// </summary>
        public string ethnicityDescriptor { get; set; }

        /// <summary>
        /// A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table.
        /// </summary>
        public string lepInstructionalProgramDescriptor { get; set; }

        /// <summary>
        /// District assigned ID number for a student from MSDS at the time of testing snapshot
        /// </summary>
        public string studentIDNumber { get; set; }

        /// <summary>
        /// 5 digit ISDCode from EEM at the time of the testing snapshot
        /// </summary>
        public string isdCode { get; set; }

        /// <summary>
        /// ISDName from EEM at the time of the testing snapshot
        /// </summary>
        public string isdName { get; set; }

        /// <summary>
        /// 5 digit District Code from EEM at the time of the testing snapshot
        /// </summary>
        public string districtCode { get; set; }

        /// <summary>
        /// District Name from EEM at the time of the testing snapshot
        /// </summary>
        public string districtName { get; set; }

        /// <summary>
        /// 5 digit Building Code from EEM at the time of the testing snapshot
        /// </summary>
        public string buildingCode { get; set; }

        /// <summary>
        /// Building Name from EEM at the time of the testing snapshot
        /// </summary>
        public string buildingName { get; set; }

        /// <summary>
        /// 1 = Y - Private School   0 = N - Public School
        /// </summary>
        public bool? privateIndicator { get; set; }

        /// <summary>
        /// Student Last Name from MSDS at the time of testing snapshot.
        /// </summary>
        public string lastSurname { get; set; }

        /// <summary>
        /// Student First Name from MSDS at the time of testing snapshot.
        /// </summary>
        public string firstName { get; set; }

        /// <summary>
        /// Student Middle Name from MSDS at the time of testing snapshot.
        /// </summary>
        public string middleName { get; set; }

        /// <summary>
        /// Student date of birth in YYYY-MM-DD format
        /// </summary>
        public DateTime? dateOfBirth { get; set; }

        /// <summary>
        /// 1 digit identifier from MSDS at the time of testing.  M - Male   F - Female
        /// </summary>
        public string gender { get; set; }

        /// <summary>
        /// 1 digit identifier from MSDS at the time of testing.  1 = Y - Economically Disadvantaged  0 = N - Not Econmically Disadvantaged
        /// </summary>
        public bool? economicallyDisadvantaged { get; set; }

        /// <summary>
        /// 1 digit identifier from MSDS at the time of testing.  1 = Y - Special Education  0 = N - Not Special Education
        /// </summary>
        public bool? specialEducation { get; set; }

        /// <summary>
        /// 1 digit identifier from MSDS at the time of testing.  1 = Y - English Learner  0 = N - Not English Learner
        /// </summary>
        public bool? englishLearner { get; set; }

        /// <summary>
        /// 1 digit identifier from MSDS at the time of testing.  1 = Y - Migrant  0 = N - Not Migrant Migrant
        /// </summary>
        public bool? migrant { get; set; }

        /// <summary>
        /// 1 digit identifier from MSDS at the time of testing.  1 = Y - Formerly English Learner  0 = N - Not Formerly Learner
        /// </summary>
        public bool? formerlyEnglishLearner { get; set; }

        /// <summary>
        /// 1 digit identifier from MSDS or bubbled on test at the time of testing.  1 = Y - Homeless  0 = N - Not Homeless Homeless
        /// </summary>
        public bool? homeless { get; set; }

        /// <summary>
        /// 1 digit identifier from MSDS or bubbled on test at the time of testing.  1 = Y - Home School  0 = N - Not Home School
        /// </summary>
        public bool? homeSchool { get; set; }

        /// <summary>
        /// Length of Time in LEP/ELL Program in a US school (regardless of country of origin) Valid values are 0-19. Represented in whole number of years by school year. Number of years that a student has been in a bilingual or ESOL program.  Round down: year and 10 months=1  (A kindergarten student is always 0.)   Populated from bubbles on test.
        /// </summary>
        public string englishAcademicYears { get; set; }

        /// <summary>
        /// 1 = Y - Knowledge Of IEP for student  0 = N - Does not have knowledge of IEP for student blank Populated from bubbles on test.
        /// </summary>
        public bool? knowledgeOfIEP { get; set; }

        /// <summary>
        /// 5 digit Building Code from EEM where student received majority of instruction.
        /// </summary>
        public string feederBuildingCode { get; set; }

        /// <summary>
        /// 1 digit identifier that indicates if a student was less than full academic year at the building where tested.  1 = Y - Less than Full Academic Year  0 = N - Full Academic Year
        /// </summary>
        public bool? ltfay { get; set; }

        /// <summary>
        /// Number of years schooling in US
        /// </summary>
        public int? yearsSchoolingUS { get; set; }

        /// <summary>
        /// Space for up to 3, 3-letter codes for more specific program identifiers
        /// </summary>
        public string liepOptionalData { get; set; }

        /// <summary>
        /// Indicates if the student''s parent has refused language services 1 = Y - Parent Refusal 0 = N - No parent Refusal
        /// </summary>
        public bool? liepParentRefusal { get; set; }

        }
}

