﻿using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

namespace Michigan.Ods.Sdk.Profiles.Assessment.Models 
{
    public class StudentAssessment_Writable 
    {
        /// <summary>
        /// The unique identifier of the resource.
        /// </summary>
        public string id { get; set; }

        /// <summary>
        /// A reference to the related Assessment resource.
        /// </summary>
        public AssessmentReference assessmentReference { get; set; }

        /// <summary>
        /// A reference to the related Student resource.
        /// </summary>
        public StudentReference studentReference { get; set; }

        /// <summary>
        /// The month(s), day(s), and year on which an assessment is administered or first day of administration if over multiple days.
        /// </summary>
        public DateTime? administrationDate { get; set; }

        /// <summary>
        /// Assessment Administration End Date, if administered over multiple days.
        /// </summary>
        public DateTime? administrationEndDate { get; set; }

        /// <summary>
        /// The environment in which the test was administered. For example:          Electronic          Classroom          Testing Center          ...
        /// </summary>
        public string administrationEnvironmentType { get; set; }

        /// <summary>
        /// The language in which an assessment is written and/or administered.
        /// </summary>
        public string administrationLanguageDescriptor { get; set; }

        /// <summary>
        /// (MI-EXT) A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table.
        /// </summary>
        public string assessmentReasonDescriptor { get; set; }

        /// <summary>
        /// (MI EXT) Identifies whether the student attempted the assessment and answered enough questions to calculate an overall scale score.
        /// </summary>
        public bool? attemptedness { get; set; }

        /// <summary>
        /// (MI EXT) Student's barcode number providing detailed information regarding the student and the assessment.
        /// </summary>
        public string barcode { get; set; }

        /// <summary>
        /// (MI-EXT) Date record was created in MDE system.
        /// </summary>
        public DateTime? createdDate { get; set; }

        /// <summary>
        /// 1 = Student used a Designated Support  0 = Student did not use a Designated Support
        /// </summary>
        public bool? designatedSupportUsed { get; set; }

        /// <summary>
        /// An unusual event occurred during the administration of the assessment. This could include fire alarm, student became ill, etc.
        /// </summary>
        public string eventCircumstanceType { get; set; }

        /// <summary>
        /// Describes special events that occur before during or after the assessment session that may impact use of results.
        /// </summary>
        public string eventDescription { get; set; }

        /// <summary>
        /// (MI EXT) 1 = Y - Student exempted because of 504   0 = N - Student not exempted because of 504
        /// </summary>
        public bool? exemptedSpecialEd504 { get; set; }

        /// <summary>
        /// 1 = Part 1 of the Online test was not submitted by the student but was forced submit by the system.  0 = Part 1 of the Online tet was submitted by the student.
        /// </summary>
        public bool? forceSubmitPart1Flag { get; set; }

        /// <summary>
        /// 1 = Part 2 of the Online test was not submitted by the student but was forced submit by the system.  0 = Part 2 of the Online tet was submitted by the student.
        /// </summary>
        public bool? forceSubmitPart2Flag { get; set; }

        /// <summary>
        /// 1 = Perrformance task of the Online test was not submitted by the student but was forced submit by the system.  0 = Performance task of the Online tet was submitted by the student.
        /// </summary>
        public bool? forceSubmitPTFlag { get; set; }

        /// <summary>
        /// (MI-EXT) A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table.
        /// </summary>
        public string gradeClusterDescriptor { get; set; }

        /// <summary>
        /// A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table.
        /// </summary>
        public string invalidReasonCodeDescriptor { get; set; }

        /// <summary>
        /// (MI EXT) Identifies whether there was a misadministration of the assessment to the student. Misadministration of an assessment includes
        /// </summary>
        public bool? misadministration { get; set; }

        /// <summary>
        /// (MI-EXT) Date record was modified in the MDE system.
        /// </summary>
        public DateTime? modifiedDate { get; set; }

        /// <summary>
        /// (MI EXT) Identifies whether a student received accommodations that do not qualify as standard accommodations.
        /// </summary>
        public bool? nonstandardAccommodation { get; set; }

        /// <summary>
        /// (MI EXT) Identifies whether the assessment was administered in an online testing environment.
        /// </summary>
        public bool? online { get; set; }

        /// <summary>
        /// (MI EXT) Identifies whether student tested in an incorrect grade.
        /// </summary>
        public bool? outOfLevel { get; set; }

        /// <summary>
        /// (MI-EXT) Date the parent conference occured.
        /// </summary>
        public DateTime? parentConferenceDate { get; set; }

        /// <summary>
        /// (MI-EXT) A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table.
        /// </summary>
        public string parentConferenceMethodDescriptor { get; set; }

        /// <summary>
        /// A specification of which performance level value describes the student proficiency.
        /// </summary>
        public string previousAdministrationPerformanceLevelDescriptor { get; set; }

        /// <summary>
        /// (MI EXT) Identifies whether the student was involved in prohibited behavior while the assessment was being administered.
        /// </summary>
        public bool? prohibitedBehavior { get; set; }

        /// <summary>
        /// The primary reason student is not tested. For example:          Absent          Refusal by parent          Refusal by student          Medical waiver          Illness          Disruptive behavior          LEP Exempt          ...
        /// </summary>
        public string reasonNotTestedType { get; set; }

        /// <summary>
        /// (MI-EXT) A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table.
        /// </summary>
        public string reportedTier { get; set; }

        /// <summary>
        /// Indicator if the test was retaken. For example:          Primary administration          First retest          Second retest          ...
        /// </summary>
        public string retestIndicatorType { get; set; }

        /// <summary>
        /// (MI EXT) 1 = Y - Writing was scribed   0 = N - Writing was not scribed
        /// </summary>
        public bool? scribedStudentResponse { get; set; }

        /// <summary>
        /// The unique number for the assessment form or answer document.
        /// </summary>
        public string serialNumber { get; set; }

        /// <summary>
        /// (MI EXT)  Identifies whether a student received a standard accommodation while the assessment was administered. Standard accommodations are listed and includes: Braille, Text-to-speech passage, Read-aloud, Enlarged Print ...
        /// </summary>
        public bool? standardAccommodation { get; set; }

        /// <summary>
        /// (MI EXT) 1 = Y - Student test is marked as invalid on answer document   0 = N - Student test is not marked as invalid on answer document
        /// </summary>
        public bool? testMarkedInvalid { get; set; }

        /// <summary>
        /// (MI-EXT) A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table.
        /// </summary>
        public string testMarkedInvalidReasonDescriptor { get; set; }

        /// <summary>
        /// (MI-EXT) A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table.
        /// </summary>
        public string testModeDescriptor { get; set; }

        /// <summary>
        /// (MI-EXT) A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table.
        /// </summary>
        public string unableToTestReasonDescriptor { get; set; }

        /// <summary>
        /// (MI EXT) Identifies whether the student received a valid test score for the assessment.
        /// </summary>
        public bool? valid { get; set; }

        /// <summary>
        /// (MI-EXT) 1 = Y - Content is a valid test   0 = N - Content is not a valid test
        /// </summary>
        public bool? validContent { get; set; }

        /// <summary>
        /// The grade level of a student when assessed.
        /// </summary>
        public string whenAssessedGradeLevelDescriptor { get; set; }

        /// <summary>
        /// (MI-EXT) The child's assessment rating on the ability to take appropriate actions to meet his/her needs.
        /// </summary>
        public StudentAssessmentActionAssessmentOutcome_Writable actionAssessmentOutcome { get; set; }

        /// <summary>
        /// (MI-EXT) The child's assessment rating for acquisition and use of knowledge and skills.
        /// </summary>
        public StudentAssessmentKnowledgeSkillAssessmentOutcome_Writable knowledgeSkillAssessmentOutcome { get; set; }

        /// <summary>
        /// (MI-EXT) The child's assessment rating for acquisition and use of knowledge and skills.
        /// </summary>
        public StudentAssessmentOverallSGP_Writable overallSGP { get; set; }

        /// <summary>
        /// (MI EXT) Captures the SchoolYear detail of the StudentAssessment
        /// </summary>
        public StudentAssessmentSchoolYear_Writable schoolYear { get; set; }

        /// <summary>
        /// (MI-EXT) The child's assessment rating for positive social relationships.
        /// </summary>
        public StudentAssessmentSocialRelationshipAssessmentOutcome_Writable socialRelationshipAssessmentOutcome { get; set; }

        /// <summary>
        /// (MI EXT) Captures MDE details that were applicable at the time of testing snapshot.
        /// </summary>
        public StudentAssessmentTimeOfTestingSnapshot_Writable timeOfTestingSnapshot { get; set; }

        /// <summary>
        /// An unordered collection of studentAssessmentAccommodations.  The specific type of special variation used in how an examination is presented, how it is administered, or how the test taker is allowed to respond. This generally refers to changes that do not substantially alter what the examinationmeasures. The proper use of accommodations does not substantially change academic level or performance criteria. For example:          Braille          Enlarged monitor view          Extra time          Large Print          Setting          Oral Administration          ...
        /// </summary>
        public List<StudentAssessmentAccommodation_Writable> accommodations { get; set; }

        /// <summary>
        /// An unordered collection of studentAssessmentEssays.  (MI EXT) This entity represents the analysis or scoring of a student's response on an assessment essay. The analysis results in a value that represents a student's performance on a set of items on a test.
        /// </summary>
        public List<StudentAssessmentEssay_Writable> essays { get; set; }

        /// <summary>
        /// An unordered collection of studentAssessmentItems.  NEEDS DOCUMENTATION
        /// </summary>
        public List<StudentAssessmentItem_Writable> items { get; set; }

        /// <summary>
        /// An unordered collection of studentAssessmentPerformanceLevels.  The performance level(s) achieved for the StudentAssessment.
        /// </summary>
        public List<StudentAssessmentPerformanceLevel_Writable> performanceLevels { get; set; }

        /// <summary>
        /// An unordered collection of studentAssessmentScoreResults.  A meaningful score or statistical expression of the performance of an individual. The results can be expressed as a number, percentile, range, level, etc.
        /// </summary>
        public List<StudentAssessmentScoreResult_Writable> scoreResults { get; set; }

        /// <summary>
        /// An unordered collection of studentAssessmentStudentObjectiveAssessments.  NEEDS DOCUMENTATION
        /// </summary>
        public List<StudentAssessmentStudentObjectiveAssessment_Writable> studentObjectiveAssessments { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        public string _etag { get; set; }

        }
}

