﻿using System;
using System.Collections.Generic;
using System.Linq;
using Michigan.Ods.Sdk.Helpers;
using Michigan.Ods.Sdk.Profiles.LibraryProfile.Models;
using RestSharp;
  using System.Net;
  using System.Text;
  
namespace Michigan.Ods.Sdk.Profiles.LibraryProfile.Api 
{
    public class StaffEducationOrganizationEmploymentAssociationsApi 
    {
        private readonly IRestClient client;

        public StaffEducationOrganizationEmploymentAssociationsApi(IRestClient client)
        {
            this.client = client;
        }
      
        /// <summary>
        /// Retrieves resources based with paging capabilities (using the &quot;Get All&quot; pattern). This GET operation provides access to resources using the &quot;Get All&quot; pattern. In this version of the API there is support for paging.
        /// </summary>
        /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
        /// <param name="minChangeVersion">Used in synchronization to set sequence minimum ChangeVersion</param>
        /// <param name="maxChangeVersion">Used in synchronization to set sequence maximum ChangeVersion</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<List<StaffEducationOrganizationEmploymentAssociation_Readable>> GetStaffEducationOrganizationEmploymentAssociationsAll(int? offset= null, int? limit= null, int? minChangeVersion= null, int? maxChangeVersion= null) 
        {
            var request = new RestRequest("/staffEducationOrganizationEmploymentAssociations", Method.GET);
            request.RequestFormat = DataFormat.Json;

            if (offset != null)
                request.AddParameter("offset", offset);
            if (limit != null)
                request.AddParameter("limit", limit);
            if (minChangeVersion != null)
                request.AddParameter("minChangeVersion", minChangeVersion);
            if (maxChangeVersion != null)
                request.AddParameter("maxChangeVersion", maxChangeVersion);
            request.AddHeader("Accept", "application/vnd.ed-fi.staffeducationorganizationemploymentassociation.libraryprofile.readable+json");
            var response = client.Execute<List<StaffEducationOrganizationEmploymentAssociation_Readable>>(request);

            return response;
        }
        /// <summary>
        /// Retrieves resources matching values of an example resource (using the &quot;Get By Example&quot; pattern). This GET operation provides access to resources using the &quot;Get by Example&quot; search pattern.  The values of any properties of the resource that are specified will be used to return all matching results (if it exists).
        /// </summary>
        /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
        /// <param name="minChangeVersion">Used in synchronization to set sequence minimum ChangeVersion</param>
        /// <param name="maxChangeVersion">Used in synchronization to set sequence maximum ChangeVersion</param>
        /// <param name="employmentStatusDescriptor">Reflects the type of employment or contract; for example:          Probationary          Contractual          Substitute/temporary          Tenured or permanent          Volunteer/no contract          ...</param>
        /// <param name="hireDate">The month, day, and year on which an individual was hired for a position.</param>
        /// <param name="educationOrganizationId">The identifier assigned to an education agency by the State Education Agency (SEA).  Also known as the State LEA ID.</param>
        /// <param name="staffUniqueId">A unique alphanumeric code assigned to a staff.</param>
        /// <param name="separationReasonDescriptor">Reason for terminating the employment; for example:          Employment in education          Employment outside of education          Retirement          Family/personal relocation          Change of assignment.</param>
        /// <param name="separationType">Type of employment separation; for example: Voluntary separation, Involuntary separation,          Mutual agreement          Other, etc.</param>
        /// <param name="endDate">The month, day, and year on which a contract between an individual and a governing authority ends or is terminated under the provisions of the contract (or the date on which the agreement is made invalid).</param>
        /// <param name="department">The department or suborganization the employee/contractor is associated with in the EducationOrganization.</param>
        /// <param name="fullTimeEquivalency">The ratio between the hours of work expected in a position and the hours of work normally expected in a full-time position in the same setting.</param>
        /// <param name="offerDate">Date at which the staff member was made an official offer for this employment.</param>
        /// <param name="hourlyWage">Hourly wage associated with the employment position being reported.</param>
        /// <param name="id"></param>
        /// <param name="accountingFunctionCodeDescriptor">(MI EXT) The appropriate accounting/function code for the employee as determined for accounting purposes by the school district.</param>
        /// <param name="repFullTimeBaseAnnualSalary">(MI EXT) The full-time base annual salary for the employee is the base salary (basic contract) for a first-year full-time employee at this educational level. It does not include increments for years of service or longevity paymensts.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<List<StaffEducationOrganizationEmploymentAssociation_Readable>> GetStaffEducationOrganizationEmploymentAssociationsByExample(int? offset= null, int? limit= null, int? minChangeVersion= null, int? maxChangeVersion= null, string employmentStatusDescriptor= null, DateTime? hireDate= null, int? educationOrganizationId= null, string staffUniqueId= null, string separationReasonDescriptor= null, string separationType= null, DateTime? endDate= null, string department= null, double? fullTimeEquivalency= null, DateTime? offerDate= null, double? hourlyWage= null, string id= null, string accountingFunctionCodeDescriptor= null, int? repFullTimeBaseAnnualSalary= null) 
        {
            var request = new RestRequest("/staffEducationOrganizationEmploymentAssociations", Method.GET);
            request.RequestFormat = DataFormat.Json;

            if (offset != null)
                request.AddParameter("offset", offset);
            if (limit != null)
                request.AddParameter("limit", limit);
            if (minChangeVersion != null)
                request.AddParameter("minChangeVersion", minChangeVersion);
            if (maxChangeVersion != null)
                request.AddParameter("maxChangeVersion", maxChangeVersion);
            if (employmentStatusDescriptor != null)
                request.AddParameter("employmentStatusDescriptor", employmentStatusDescriptor);
            if (hireDate != null)
                request.AddParameter("hireDate", hireDate);
            if (educationOrganizationId != null)
                request.AddParameter("educationOrganizationId", educationOrganizationId);
            if (staffUniqueId != null)
                request.AddParameter("staffUniqueId", staffUniqueId);
            if (separationReasonDescriptor != null)
                request.AddParameter("separationReasonDescriptor", separationReasonDescriptor);
            if (separationType != null)
                request.AddParameter("separationType", separationType);
            if (endDate != null)
                request.AddParameter("endDate", endDate);
            if (department != null)
                request.AddParameter("department", department);
            if (fullTimeEquivalency != null)
                request.AddParameter("fullTimeEquivalency", fullTimeEquivalency);
            if (offerDate != null)
                request.AddParameter("offerDate", offerDate);
            if (hourlyWage != null)
                request.AddParameter("hourlyWage", hourlyWage);
            if (id != null)
                request.AddParameter("id", id);
            if (accountingFunctionCodeDescriptor != null)
                request.AddParameter("accountingFunctionCodeDescriptor", accountingFunctionCodeDescriptor);
            if (repFullTimeBaseAnnualSalary != null)
                request.AddParameter("repFullTimeBaseAnnualSalary", repFullTimeBaseAnnualSalary);
            request.AddHeader("Accept", "application/vnd.ed-fi.staffeducationorganizationemploymentassociation.libraryprofile.readable+json");
            var response = client.Execute<List<StaffEducationOrganizationEmploymentAssociation_Readable>>(request);

            return response;
        }
        /// <summary>
        /// Retrieves a specific resource using the values of the resource's natural key (using the &quot;Get By Key&quot; pattern). This GET operation provides access to resources using the &quot;Get by Key&quot; search pattern. The values of the natural key of the resource must be fully specified, and the service will return the matching result (if it exists).
        /// </summary>
        /// <param name="educationOrganizationId">The identifier assigned to an education agency by the State Education Agency (SEA).  Also known as the State LEA ID.</param>
        /// <param name="employmentStatusDescriptor">Reflects the type of employment or contract; for example:          Probationary          Contractual          Substitute/temporary          Tenured or permanent          Volunteer/no contract          ...</param>
        /// <param name="hireDate">The month, day, and year on which an individual was hired for a position.</param>
        /// <param name="staffUniqueId">A unique alphanumeric code assigned to a staff.</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<StaffEducationOrganizationEmploymentAssociation_Readable> GetStaffEducationOrganizationEmploymentAssociationByKey(int? educationOrganizationId, string employmentStatusDescriptor, DateTime? hireDate, string staffUniqueId, string IfNoneMatch= null) 
        {
            var request = new RestRequest("/staffEducationOrganizationEmploymentAssociations", Method.GET);
            request.RequestFormat = DataFormat.Json;

            // verify required params are set
            if (educationOrganizationId == null || employmentStatusDescriptor == null || hireDate == null || staffUniqueId == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            if (educationOrganizationId != null)
                request.AddParameter("educationOrganizationId", educationOrganizationId);
            if (employmentStatusDescriptor != null)
                request.AddParameter("employmentStatusDescriptor", employmentStatusDescriptor);
            if (hireDate != null)
                request.AddParameter("hireDate", hireDate);
            if (staffUniqueId != null)
                request.AddParameter("staffUniqueId", staffUniqueId);
            request.AddHeader("If-None-Match", IfNoneMatch);
            request.AddHeader("Accept", "application/vnd.ed-fi.staffeducationorganizationemploymentassociation.libraryprofile.readable+json");
            var response = client.Execute<StaffEducationOrganizationEmploymentAssociation_Readable>(request);

            return response;
        }
        /// <summary>
        /// Retrieves a specific resource using the resource's identifier (using the &quot;Get By Id&quot; pattern). This GET operation retrieves a resource by the specified resource identifier.
        /// </summary>
        /// <param name="id">A resource identifier specifying the resource to be retrieved.</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<StaffEducationOrganizationEmploymentAssociation_Readable> GetStaffEducationOrganizationEmploymentAssociationsById(string id, string IfNoneMatch= null) 
        {
            var request = new RestRequest("/staffEducationOrganizationEmploymentAssociations/{id}", Method.GET);
            request.RequestFormat = DataFormat.Json;

            request.AddUrlSegment("id", id);
            // verify required params are set
            if (id == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            request.AddHeader("If-None-Match", IfNoneMatch);
            request.AddHeader("Accept", "application/vnd.ed-fi.staffeducationorganizationemploymentassociation.libraryprofile.readable+json");
            var response = client.Execute<StaffEducationOrganizationEmploymentAssociation_Readable>(request);

            return response;
        }
        // patch operation generation was suppressed because profiles can not support patch
    }
}

