﻿using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

namespace Michigan.Ods.Sdk.Profiles.LibraryProfile.Models 
{
    public class Student_Readable 
    {
        /// <summary>
        /// The unique identifier of the resource.
        /// </summary>
        public string id { get; set; }

        /// <summary>
        /// (MI-EXT) The date of the most recent changes to Address.
        /// </summary>
        public DateTime? addressAsOfDate { get; set; }

        /// <summary>
        /// The month, day, and year on which an individual was born.
        /// </summary>
        public DateTime? birthDate { get; set; }

        /// <summary>
        /// A name given to an individual at birth, baptism, or during another naming ceremony, or through legal change.
        /// </summary>
        public string firstName { get; set; }

        /// <summary>
        /// (MI EXT) A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table.
        /// </summary>
        public string foodServiceStatusDescriptor { get; set; }

        /// <summary>
        /// An appendage, if any, used to denote an individual's generation in his family (e.g., Jr., Sr., III).
        /// </summary>
        public string generationCodeSuffix { get; set; }

        /// <summary>
        /// A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table.
        /// </summary>
        public string homelessDescriptor { get; set; }

        /// <summary>
        /// The name borne in common by members of a family.
        /// </summary>
        public string lastSurname { get; set; }

        /// <summary>
        /// The login ID for the user; used for security access control interface.
        /// </summary>
        public string loginId { get; set; }

        /// <summary>
        /// (MI EXT) Medical notes such as "Allergic to Bee Stings" - For Transportation records.
        /// </summary>
        public string medicalNotes { get; set; }

        /// <summary>
        /// A secondary name given to an individual at birth, baptism, or during another naming ceremony.
        /// </summary>
        public string middleName { get; set; }

        /// <summary>
        /// A prefix used to denote the title, degree, position, or seniority of the person.
        /// </summary>
        public string personalTitlePrefix { get; set; }

        /// <summary>
        /// (MI-EXT) The date of the most recent change in Race.
        /// </summary>
        public DateTime? raceAsOfDate { get; set; }

        /// <summary>
        /// (MI-EXT) The date of the most recent change in SchoolFoodServicesEligibility.
        /// </summary>
        public DateTime? schoolFoodServicesEligibilityAsOfDate { get; set; }

        /// <summary>
        /// A person's gender.
        /// </summary>
        public string sexType { get; set; }

        /// <summary>
        /// (MI EXT) Indicates status (OneRoster users.status) of the Student.
        /// </summary>
        public string statusDescriptor { get; set; }

        /// <summary>
        /// A unique alphanumeric code assigned to a student.
        /// </summary>
        public string studentUniqueId { get; set; }

        /// <summary>
        /// (MI EXT) Indicates the Student username (OneRoster users.username)
        /// </summary>
        public string userName { get; set; }

        /// <summary>
        /// (MI-EXT) The four-digit calendar year in which the student first attended school in the United States.
        /// </summary>
        public int? yearOfEntry { get; set; }

        /// <summary>
        /// An unordered collection of studentAddresses.  The set of elements that describes an address, including the street address, city, state, and ZIP code.
        /// </summary>
        public List<StudentAddress_Readable> addresses { get; set; }

        /// <summary>
        /// An unordered collection of studentElectronicMails.  The numbers, letters, and symbols used to identify an electronic mail (e-mail) user within the network to which the individual or organization belongs.
        /// </summary>
        public List<StudentElectronicMail_Readable> electronicMails { get; set; }

        /// <summary>
        /// An unordered collection of studentIdentificationCodes.  A coding scheme that is used for identification and record-keeping purposes by schools, social services, or other agencies to refer to a student.
        /// </summary>
        public List<StudentIdentificationCode_Readable> identificationCodes { get; set; }

        /// <summary>
        /// An unordered collection of studentLanguages.  The language(s) the individual uses to communicate.
        /// </summary>
        public List<StudentLanguage_Readable> languages { get; set; }

        /// <summary>
        /// An unordered collection of studentTelephones.  The 10-digit telephone number, including the area code, for the person.
        /// </summary>
        public List<StudentTelephone_Readable> telephones { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        public string _etag { get; set; }

        }
}

