﻿using System;
using System.Collections.Generic;
using System.Linq;
using Michigan.Ods.Sdk.Helpers;
using Michigan.Ods.Sdk.Profiles.Nutrition.Models;
using RestSharp;
  using System.Net;
  using System.Text;
  
namespace Michigan.Ods.Sdk.Profiles.Nutrition.Api 
{
    public class ParentsApi 
    {
        private readonly IRestClient client;

        public ParentsApi(IRestClient client)
        {
            this.client = client;
        }
      
        /// <summary>
        /// Retrieves resources based with paging capabilities (using the &quot;Get All&quot; pattern). This GET operation provides access to resources using the &quot;Get All&quot; pattern. In this version of the API there is support for paging.
        /// </summary>
        /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
        /// <param name="minChangeVersion">Used in synchronization to set sequence minimum ChangeVersion</param>
        /// <param name="maxChangeVersion">Used in synchronization to set sequence maximum ChangeVersion</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<List<Parent_Readable>> GetParentsAll(int? offset= null, int? limit= null, int? minChangeVersion= null, int? maxChangeVersion= null) 
        {
            var request = new RestRequest("/parents", Method.GET);
            request.RequestFormat = DataFormat.Json;

            if (offset != null)
                request.AddParameter("offset", offset);
            if (limit != null)
                request.AddParameter("limit", limit);
            if (minChangeVersion != null)
                request.AddParameter("minChangeVersion", minChangeVersion);
            if (maxChangeVersion != null)
                request.AddParameter("maxChangeVersion", maxChangeVersion);
            request.AddHeader("Accept", "application/vnd.ed-fi.parent.nutrition.readable+json");
            var response = client.Execute<List<Parent_Readable>>(request);

            return response;
        }
        /// <summary>
        /// Retrieves resources matching values of an example resource (using the &quot;Get By Example&quot; pattern). This GET operation provides access to resources using the &quot;Get by Example&quot; search pattern.  The values of any properties of the resource that are specified will be used to return all matching results (if it exists).
        /// </summary>
        /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
        /// <param name="minChangeVersion">Used in synchronization to set sequence minimum ChangeVersion</param>
        /// <param name="maxChangeVersion">Used in synchronization to set sequence maximum ChangeVersion</param>
        /// <param name="parentUniqueId">A unique alphanumeric code assigned to a parent.</param>
        /// <param name="sexType">A person's gender.</param>
        /// <param name="personalTitlePrefix">A prefix used to denote the title, degree, position, or seniority of the person.</param>
        /// <param name="firstName">A name given to an individual at birth, baptism, or during another naming ceremony, or through legal change.</param>
        /// <param name="middleName">A secondary name given to an individual at birth, baptism, or during another naming ceremony.</param>
        /// <param name="lastSurname">The name borne in common by members of a family.</param>
        /// <param name="generationCodeSuffix">An appendage, if any, used to denote an individual's generation in his family (e.g., Jr., Sr., III).</param>
        /// <param name="maidenName">The person's maiden name.</param>
        /// <param name="loginId">The login ID for the user; used for security access control interface.</param>
        /// <param name="id"></param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<List<Parent_Readable>> GetParentsByExample(int? offset= null, int? limit= null, int? minChangeVersion= null, int? maxChangeVersion= null, string parentUniqueId= null, string sexType= null, string personalTitlePrefix= null, string firstName= null, string middleName= null, string lastSurname= null, string generationCodeSuffix= null, string maidenName= null, string loginId= null, string id= null) 
        {
            var request = new RestRequest("/parents", Method.GET);
            request.RequestFormat = DataFormat.Json;

            if (offset != null)
                request.AddParameter("offset", offset);
            if (limit != null)
                request.AddParameter("limit", limit);
            if (minChangeVersion != null)
                request.AddParameter("minChangeVersion", minChangeVersion);
            if (maxChangeVersion != null)
                request.AddParameter("maxChangeVersion", maxChangeVersion);
            if (parentUniqueId != null)
                request.AddParameter("parentUniqueId", parentUniqueId);
            if (sexType != null)
                request.AddParameter("sexType", sexType);
            if (personalTitlePrefix != null)
                request.AddParameter("personalTitlePrefix", personalTitlePrefix);
            if (firstName != null)
                request.AddParameter("firstName", firstName);
            if (middleName != null)
                request.AddParameter("middleName", middleName);
            if (lastSurname != null)
                request.AddParameter("lastSurname", lastSurname);
            if (generationCodeSuffix != null)
                request.AddParameter("generationCodeSuffix", generationCodeSuffix);
            if (maidenName != null)
                request.AddParameter("maidenName", maidenName);
            if (loginId != null)
                request.AddParameter("loginId", loginId);
            if (id != null)
                request.AddParameter("id", id);
            request.AddHeader("Accept", "application/vnd.ed-fi.parent.nutrition.readable+json");
            var response = client.Execute<List<Parent_Readable>>(request);

            return response;
        }
        /// <summary>
        /// Retrieves a specific resource using the values of the resource's natural key (using the &quot;Get By Key&quot; pattern). This GET operation provides access to resources using the &quot;Get by Key&quot; search pattern. The values of the natural key of the resource must be fully specified, and the service will return the matching result (if it exists).
        /// </summary>
        /// <param name="parentUniqueId">A unique alphanumeric code assigned to a parent.</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<Parent_Readable> GetParentByKey(string parentUniqueId, string IfNoneMatch= null) 
        {
            var request = new RestRequest("/parents", Method.GET);
            request.RequestFormat = DataFormat.Json;

            // verify required params are set
            if (parentUniqueId == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            if (parentUniqueId != null)
                request.AddParameter("parentUniqueId", parentUniqueId);
            request.AddHeader("If-None-Match", IfNoneMatch);
            request.AddHeader("Accept", "application/vnd.ed-fi.parent.nutrition.readable+json");
            var response = client.Execute<Parent_Readable>(request);

            return response;
        }
        /// <summary>
        /// Retrieves a specific resource using the resource's identifier (using the &quot;Get By Id&quot; pattern). This GET operation retrieves a resource by the specified resource identifier.
        /// </summary>
        /// <param name="id">A resource identifier specifying the resource to be retrieved.</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<Parent_Readable> GetParentsById(string id, string IfNoneMatch= null) 
        {
            var request = new RestRequest("/parents/{id}", Method.GET);
            request.RequestFormat = DataFormat.Json;

            request.AddUrlSegment("id", id);
            // verify required params are set
            if (id == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            request.AddHeader("If-None-Match", IfNoneMatch);
            request.AddHeader("Accept", "application/vnd.ed-fi.parent.nutrition.readable+json");
            var response = client.Execute<Parent_Readable>(request);

            return response;
        }
        // patch operation generation was suppressed because profiles can not support patch
    }
}

