﻿using System;
using System.Collections.Generic;
using System.Linq;
using Michigan.Ods.Sdk.Helpers;
using Michigan.Ods.Sdk.Profiles.SIS.Models;
using RestSharp;
  using System.Net;
  using System.Text;
  
namespace Michigan.Ods.Sdk.Profiles.SIS.Api 
{
    public class EemCountiesApi 
    {
        private readonly IRestClient client;

        public EemCountiesApi(IRestClient client)
        {
            this.client = client;
        }
      
        /// <summary>
        /// Retrieves resources based with paging capabilities (using the &quot;Get All&quot; pattern). This GET operation provides access to resources using the &quot;Get All&quot; pattern. In this version of the API there is support for paging.
        /// </summary>
        /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
        /// <param name="minChangeVersion">Used in synchronization to set sequence minimum ChangeVersion</param>
        /// <param name="maxChangeVersion">Used in synchronization to set sequence maximum ChangeVersion</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<List<EemCounty_Readable>> GetEEMCountiesAll(int? offset= null, int? limit= null, int? minChangeVersion= null, int? maxChangeVersion= null) 
        {
            var request = new RestRequest("/eemCounties", Method.GET);
            request.RequestFormat = DataFormat.Json;

            if (offset != null)
                request.AddParameter("offset", offset);
            if (limit != null)
                request.AddParameter("limit", limit);
            if (minChangeVersion != null)
                request.AddParameter("minChangeVersion", minChangeVersion);
            if (maxChangeVersion != null)
                request.AddParameter("maxChangeVersion", maxChangeVersion);
            request.AddHeader("Accept", "application/vnd.ed-fi.eemcounty.sis.readable+json");
            var response = client.Execute<List<EemCounty_Readable>>(request);

            return response;
        }
        /// <summary>
        /// Retrieves resources matching values of an example resource (using the &quot;Get By Example&quot; pattern). This GET operation provides access to resources using the &quot;Get by Example&quot; search pattern.  The values of any properties of the resource that are specified will be used to return all matching results (if it exists).
        /// </summary>
        /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
        /// <param name="minChangeVersion">Used in synchronization to set sequence minimum ChangeVersion</param>
        /// <param name="maxChangeVersion">Used in synchronization to set sequence maximum ChangeVersion</param>
        /// <param name="eemCountyId">EEM County identifier.</param>
        /// <param name="eemProsperityRegionDescriptor">The EEM Prosperity Region of the County.</param>
        /// <param name="isUPCounty">1 = is UP County; 0 or NULL = is not UP County;</param>
        /// <param name="code">The official state-assigned two-digit code number of the County.</param>
        /// <param name="name">The County name.</param>
        /// <param name="eemfipsCode">The FIPS code associated with the County.</param>
        /// <param name="eempsaDistrictNumber">The PSA District number of the County.</param>
        /// <param name="id"></param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<List<EemCounty_Readable>> GetEEMCountiesByExample(int? offset= null, int? limit= null, int? minChangeVersion= null, int? maxChangeVersion= null, int? eemCountyId= null, string eemProsperityRegionDescriptor= null, bool? isUPCounty= null, int? code= null, string name= null, string eemfipsCode= null, int? eempsaDistrictNumber= null, string id= null) 
        {
            var request = new RestRequest("/eemCounties", Method.GET);
            request.RequestFormat = DataFormat.Json;

            if (offset != null)
                request.AddParameter("offset", offset);
            if (limit != null)
                request.AddParameter("limit", limit);
            if (minChangeVersion != null)
                request.AddParameter("minChangeVersion", minChangeVersion);
            if (maxChangeVersion != null)
                request.AddParameter("maxChangeVersion", maxChangeVersion);
            if (eemCountyId != null)
                request.AddParameter("eemCountyId", eemCountyId);
            if (eemProsperityRegionDescriptor != null)
                request.AddParameter("eemProsperityRegionDescriptor", eemProsperityRegionDescriptor);
            if (isUPCounty != null)
                request.AddParameter("isUPCounty", isUPCounty);
            if (code != null)
                request.AddParameter("code", code);
            if (name != null)
                request.AddParameter("name", name);
            if (eemfipsCode != null)
                request.AddParameter("eemfipsCode", eemfipsCode);
            if (eempsaDistrictNumber != null)
                request.AddParameter("eempsaDistrictNumber", eempsaDistrictNumber);
            if (id != null)
                request.AddParameter("id", id);
            request.AddHeader("Accept", "application/vnd.ed-fi.eemcounty.sis.readable+json");
            var response = client.Execute<List<EemCounty_Readable>>(request);

            return response;
        }
        /// <summary>
        /// Retrieves a specific resource using the values of the resource's natural key (using the &quot;Get By Key&quot; pattern). This GET operation provides access to resources using the &quot;Get by Key&quot; search pattern. The values of the natural key of the resource must be fully specified, and the service will return the matching result (if it exists).
        /// </summary>
        /// <param name="eemCountyId">EEM County identifier.</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<EemCounty_Readable> GetEEMCountyByKey(int? eemCountyId, string IfNoneMatch= null) 
        {
            var request = new RestRequest("/eemCounties", Method.GET);
            request.RequestFormat = DataFormat.Json;

            // verify required params are set
            if (eemCountyId == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            if (eemCountyId != null)
                request.AddParameter("eemCountyId", eemCountyId);
            request.AddHeader("If-None-Match", IfNoneMatch);
            request.AddHeader("Accept", "application/vnd.ed-fi.eemcounty.sis.readable+json");
            var response = client.Execute<EemCounty_Readable>(request);

            return response;
        }
        /// <summary>
        /// Retrieves a specific resource using the resource's identifier (using the &quot;Get By Id&quot; pattern). This GET operation retrieves a resource by the specified resource identifier.
        /// </summary>
        /// <param name="id">A resource identifier specifying the resource to be retrieved.</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<EemCounty_Readable> GetEEMCountiesById(string id, string IfNoneMatch= null) 
        {
            var request = new RestRequest("/eemCounties/{id}", Method.GET);
            request.RequestFormat = DataFormat.Json;

            request.AddUrlSegment("id", id);
            // verify required params are set
            if (id == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            request.AddHeader("If-None-Match", IfNoneMatch);
            request.AddHeader("Accept", "application/vnd.ed-fi.eemcounty.sis.readable+json");
            var response = client.Execute<EemCounty_Readable>(request);

            return response;
        }
        // patch operation generation was suppressed because profiles can not support patch
    }
}

