﻿using System;
using System.Collections.Generic;
using System.Linq;
using Michigan.Ods.Sdk.Helpers;
using Michigan.Ods.Sdk.Profiles.SIS.Models;
using RestSharp;
  using System.Net;
  using System.Text;
  
namespace Michigan.Ods.Sdk.Profiles.SIS.Api 
{
    public class InterventionsApi 
    {
        private readonly IRestClient client;

        public InterventionsApi(IRestClient client)
        {
            this.client = client;
        }
      
        /// <summary>
        /// Retrieves resources based with paging capabilities (using the &quot;Get All&quot; pattern). This GET operation provides access to resources using the &quot;Get All&quot; pattern. In this version of the API there is support for paging.
        /// </summary>
        /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
        /// <param name="minChangeVersion">Used in synchronization to set sequence minimum ChangeVersion</param>
        /// <param name="maxChangeVersion">Used in synchronization to set sequence maximum ChangeVersion</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<List<Intervention_Readable>> GetInterventionsAll(int? offset= null, int? limit= null, int? minChangeVersion= null, int? maxChangeVersion= null) 
        {
            var request = new RestRequest("/interventions", Method.GET);
            request.RequestFormat = DataFormat.Json;

            if (offset != null)
                request.AddParameter("offset", offset);
            if (limit != null)
                request.AddParameter("limit", limit);
            if (minChangeVersion != null)
                request.AddParameter("minChangeVersion", minChangeVersion);
            if (maxChangeVersion != null)
                request.AddParameter("maxChangeVersion", maxChangeVersion);
            request.AddHeader("Accept", "application/vnd.ed-fi.intervention.sis.readable+json");
            var response = client.Execute<List<Intervention_Readable>>(request);

            return response;
        }
        /// <summary>
        /// Retrieves resources matching values of an example resource (using the &quot;Get By Example&quot; pattern). This GET operation provides access to resources using the &quot;Get by Example&quot; search pattern.  The values of any properties of the resource that are specified will be used to return all matching results (if it exists).
        /// </summary>
        /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
        /// <param name="minChangeVersion">Used in synchronization to set sequence minimum ChangeVersion</param>
        /// <param name="maxChangeVersion">Used in synchronization to set sequence maximum ChangeVersion</param>
        /// <param name="identificationCode">A unique number or alphanumeric code assigned to an intervention.</param>
        /// <param name="educationOrganizationId">The identifier assigned to an education agency by the State Education Agency (SEA).  Also known as the State LEA ID.</param>
        /// <param name="classType">The way in which an intervention is used: curriculum, supplement, or practice.</param>
        /// <param name="beginDate">The start date for the intervention implementation.</param>
        /// <param name="endDate">The end date for the intervention implementation.</param>
        /// <param name="id"></param>
        /// <param name="academicSubsetDescriptor">(MI EXT) Indicates the academic subset.</param>
        /// <param name="costStatement">(MI EXT) A statement that provides information regarding the cost of this Intervention.</param>
        /// <param name="deleted">(MI EXT) Indicates if this Intervention has been deleted. 0=&quot;No&quot; 1=&quot;Yes&quot; NULL=&quot;Unknown&quot;</param>
        /// <param name="durationLength">(MI EXT) Duration length.</param>
        /// <param name="durationPeriod">(MI EXT) Duration period.</param>
        /// <param name="effectivenessMeasurementMethodDescriptor">(MI EXT) Indicates the method of the effectiveness measurement.</param>
        /// <param name="eligibilityStatement">(MI EXT) A statement that provides information regarding the eligibility of this Intervention.</param>
        /// <param name="hasCost">(MI EXT) 0=&quot;No&quot; 1=&quot;Yes&quot; NULL=&quot;Unknown&quot;</param>
        /// <param name="hasDuration">(MI EXT) 0=&quot;No&quot; 1=&quot;Yes&quot; NULL=&quot;Unknown&quot;</param>
        /// <param name="hasEligibility">(MI EXT) 0=&quot;No&quot; 1=&quot;Yes&quot; NULL=&quot;Unknown&quot;</param>
        /// <param name="contactElectronicMailAddress">(MI EXT) Electronic mail (e-mail) of the Contact for this Intervention.</param>
        /// <param name="contactName">(MI EXT) Name of the Contact for this Intervention.</param>
        /// <param name="contactTelephoneNumber">(MI EXT) Telephone number of the Contact for this Intervention.</param>
        /// <param name="description">(MI EXT) A description of this Intervention.</param>
        /// <param name="name">(MI EXT) The name of this Intervention.</param>
        /// <param name="locationReferral">(MI EXT) 0=&quot;No&quot; 1=&quot;Yes&quot; NULL=&quot;Unknown&quot;</param>
        /// <param name="originatingSourceDescriptor">(MI EXT) Indicates the source where Intervention or InterventionPrescription originated.</param>
        /// <param name="parentPermissionRequired">(MI EXT) 0=&quot;No&quot; 1=&quot;Yes&quot; NULL=&quot;Unknown&quot;</param>
        /// <param name="periodType">(MI EXT) A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<List<Intervention_Readable>> GetInterventionsByExample(int? offset= null, int? limit= null, int? minChangeVersion= null, int? maxChangeVersion= null, string identificationCode= null, int? educationOrganizationId= null, string classType= null, DateTime? beginDate= null, DateTime? endDate= null, string id= null, string academicSubsetDescriptor= null, string costStatement= null, bool? deleted= null, int? durationLength= null, string durationPeriod= null, string effectivenessMeasurementMethodDescriptor= null, string eligibilityStatement= null, bool? hasCost= null, bool? hasDuration= null, bool? hasEligibility= null, string contactElectronicMailAddress= null, string contactName= null, string contactTelephoneNumber= null, string description= null, string name= null, bool? locationReferral= null, string originatingSourceDescriptor= null, bool? parentPermissionRequired= null, string periodType= null) 
        {
            var request = new RestRequest("/interventions", Method.GET);
            request.RequestFormat = DataFormat.Json;

            if (offset != null)
                request.AddParameter("offset", offset);
            if (limit != null)
                request.AddParameter("limit", limit);
            if (minChangeVersion != null)
                request.AddParameter("minChangeVersion", minChangeVersion);
            if (maxChangeVersion != null)
                request.AddParameter("maxChangeVersion", maxChangeVersion);
            if (identificationCode != null)
                request.AddParameter("identificationCode", identificationCode);
            if (educationOrganizationId != null)
                request.AddParameter("educationOrganizationId", educationOrganizationId);
            if (classType != null)
                request.AddParameter("classType", classType);
            if (beginDate != null)
                request.AddParameter("beginDate", beginDate);
            if (endDate != null)
                request.AddParameter("endDate", endDate);
            if (id != null)
                request.AddParameter("id", id);
            if (academicSubsetDescriptor != null)
                request.AddParameter("academicSubsetDescriptor", academicSubsetDescriptor);
            if (costStatement != null)
                request.AddParameter("costStatement", costStatement);
            if (deleted != null)
                request.AddParameter("deleted", deleted);
            if (durationLength != null)
                request.AddParameter("durationLength", durationLength);
            if (durationPeriod != null)
                request.AddParameter("durationPeriod", durationPeriod);
            if (effectivenessMeasurementMethodDescriptor != null)
                request.AddParameter("effectivenessMeasurementMethodDescriptor", effectivenessMeasurementMethodDescriptor);
            if (eligibilityStatement != null)
                request.AddParameter("eligibilityStatement", eligibilityStatement);
            if (hasCost != null)
                request.AddParameter("hasCost", hasCost);
            if (hasDuration != null)
                request.AddParameter("hasDuration", hasDuration);
            if (hasEligibility != null)
                request.AddParameter("hasEligibility", hasEligibility);
            if (contactElectronicMailAddress != null)
                request.AddParameter("contactElectronicMailAddress", contactElectronicMailAddress);
            if (contactName != null)
                request.AddParameter("contactName", contactName);
            if (contactTelephoneNumber != null)
                request.AddParameter("contactTelephoneNumber", contactTelephoneNumber);
            if (description != null)
                request.AddParameter("description", description);
            if (name != null)
                request.AddParameter("name", name);
            if (locationReferral != null)
                request.AddParameter("locationReferral", locationReferral);
            if (originatingSourceDescriptor != null)
                request.AddParameter("originatingSourceDescriptor", originatingSourceDescriptor);
            if (parentPermissionRequired != null)
                request.AddParameter("parentPermissionRequired", parentPermissionRequired);
            if (periodType != null)
                request.AddParameter("periodType", periodType);
            request.AddHeader("Accept", "application/vnd.ed-fi.intervention.sis.readable+json");
            var response = client.Execute<List<Intervention_Readable>>(request);

            return response;
        }
        /// <summary>
        /// Retrieves a specific resource using the values of the resource's natural key (using the &quot;Get By Key&quot; pattern). This GET operation provides access to resources using the &quot;Get by Key&quot; search pattern. The values of the natural key of the resource must be fully specified, and the service will return the matching result (if it exists).
        /// </summary>
        /// <param name="educationOrganizationId">The identifier assigned to an education agency by the State Education Agency (SEA).  Also known as the State LEA ID.</param>
        /// <param name="identificationCode">A unique number or alphanumeric code assigned to an intervention.</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<Intervention_Readable> GetInterventionByKey(int? educationOrganizationId, string identificationCode, string IfNoneMatch= null) 
        {
            var request = new RestRequest("/interventions", Method.GET);
            request.RequestFormat = DataFormat.Json;

            // verify required params are set
            if (educationOrganizationId == null || identificationCode == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            if (educationOrganizationId != null)
                request.AddParameter("educationOrganizationId", educationOrganizationId);
            if (identificationCode != null)
                request.AddParameter("identificationCode", identificationCode);
            request.AddHeader("If-None-Match", IfNoneMatch);
            request.AddHeader("Accept", "application/vnd.ed-fi.intervention.sis.readable+json");
            var response = client.Execute<Intervention_Readable>(request);

            return response;
        }
        /// <summary>
        /// Creates or updates resources based on the natural key values of the supplied resource. The POST operation can be used to create or update resources. In database terms, this is often referred to as an &quot;upsert&quot; operation (insert + update).  Clients should NOT include the resource &quot;id&quot; in the JSON body because it will result in an error (you must use a PUT operation to update a resource by &quot;id&quot;). The web service will identify whether the resource already exists based on the natural key values provided, and update or create the resource appropriately.
        /// </summary>
        /// <param name="body">The JSON representation of the &quot;intervention&quot; resource to be created or updated.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse PostInterventions(Intervention_Writable body) 
        {
            var request = new RestRequest("/interventions", Method.POST);
            request.RequestFormat = DataFormat.Json;

            // verify required params are set
            if (body == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            request.AddBody(body);
            request.Parameters.First(param => param.Type == ParameterType.RequestBody).Name = "application/vnd.ed-fi.intervention.sis.writable+json";
            var response = client.Execute(request);

            var location = response.Headers.FirstOrDefault(x => x.Name == "Location");

            if (location != null && !string.IsNullOrWhiteSpace(location.Value.ToString()))
                body.id = location.Value.ToString().Split('/').Last();
            return response;
        }
        /// <summary>
        /// Retrieves a specific resource using the resource's identifier (using the &quot;Get By Id&quot; pattern). This GET operation retrieves a resource by the specified resource identifier.
        /// </summary>
        /// <param name="id">A resource identifier specifying the resource to be retrieved.</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<Intervention_Readable> GetInterventionsById(string id, string IfNoneMatch= null) 
        {
            var request = new RestRequest("/interventions/{id}", Method.GET);
            request.RequestFormat = DataFormat.Json;

            request.AddUrlSegment("id", id);
            // verify required params are set
            if (id == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            request.AddHeader("If-None-Match", IfNoneMatch);
            request.AddHeader("Accept", "application/vnd.ed-fi.intervention.sis.readable+json");
            var response = client.Execute<Intervention_Readable>(request);

            return response;
        }
        /// <summary>
        /// Updates or creates a resource based on the resource identifier. The PUT operation is used to update or create a resource by identifier.  If the resource doesn't exist, the resource will be created using that identifier.  Additionally, natural key values cannot be changed using this operation, and will not be modified in the database.  If the resource &quot;id&quot; is provided in the JSON body, it will be ignored as well.
        /// </summary>
        /// <param name="id">A resource identifier specifying the resource to be updated.</param>
        /// <param name="IfMatch">The ETag header value used to prevent the PUT from updating a resource modified by another consumer.</param>
        /// <param name="body">The JSON representation of the &quot;intervention&quot; resource to be updated.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse PutIntervention(string id, string IfMatch, Intervention_Writable body) 
        {
            var request = new RestRequest("/interventions/{id}", Method.PUT);
            request.RequestFormat = DataFormat.Json;

            request.AddUrlSegment("id", id);
            // verify required params are set
            if (id == null || body == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            request.AddHeader("If-Match", IfMatch);
            request.AddBody(body);
            request.Parameters.First(param => param.Type == ParameterType.RequestBody).Name = "application/vnd.ed-fi.intervention.sis.writable+json";
            var response = client.Execute(request);

            var location = response.Headers.FirstOrDefault(x => x.Name == "Location");

            if (location != null && !string.IsNullOrWhiteSpace(location.Value.ToString()))
                body.id = location.Value.ToString().Split('/').Last();
            return response;
        }
        /// <summary>
        /// Deletes an existing resource using the resource identifier. The DELETE operation is used to delete an existing resource by identifier.  If the resource doesn't exist, an error will result (the resource will not be found).
        /// </summary>
        /// <param name="id">A resource identifier specifying the resource to be deleted.</param>
        /// <param name="IfMatch">The ETag header value used to prevent the DELETE from removing a resource modified by another consumer.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse DeleteInterventionById(string id, string IfMatch= null) 
        {
            var request = new RestRequest("/interventions/{id}", Method.DELETE);
            request.RequestFormat = DataFormat.Json;

            request.AddUrlSegment("id", id);
            // verify required params are set
            if (id == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            request.AddHeader("If-Match", IfMatch);
            var response = client.Execute(request);

            return response;
        }
        // patch operation generation was suppressed because profiles can not support patch
    }
}

