﻿using System;
using System.Collections.Generic;
using System.Linq;
using Michigan.Ods.Sdk.Helpers;
using Michigan.Ods.Sdk.Profiles.SIS.Models;
using RestSharp;
  using System.Net;
  using System.Text;
  
namespace Michigan.Ods.Sdk.Profiles.SIS.Api 
{
    public class StudentParentAssociationsApi 
    {
        private readonly IRestClient client;

        public StudentParentAssociationsApi(IRestClient client)
        {
            this.client = client;
        }
      
        /// <summary>
        /// Retrieves resources based with paging capabilities (using the &quot;Get All&quot; pattern). This GET operation provides access to resources using the &quot;Get All&quot; pattern. In this version of the API there is support for paging.
        /// </summary>
        /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
        /// <param name="minChangeVersion">Used in synchronization to set sequence minimum ChangeVersion</param>
        /// <param name="maxChangeVersion">Used in synchronization to set sequence maximum ChangeVersion</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<List<StudentParentAssociation_Readable>> GetStudentParentAssociationsAll(int? offset= null, int? limit= null, int? minChangeVersion= null, int? maxChangeVersion= null) 
        {
            var request = new RestRequest("/studentParentAssociations", Method.GET);
            request.RequestFormat = DataFormat.Json;

            if (offset != null)
                request.AddParameter("offset", offset);
            if (limit != null)
                request.AddParameter("limit", limit);
            if (minChangeVersion != null)
                request.AddParameter("minChangeVersion", minChangeVersion);
            if (maxChangeVersion != null)
                request.AddParameter("maxChangeVersion", maxChangeVersion);
            request.AddHeader("Accept", "application/vnd.ed-fi.studentparentassociation.sis.readable+json");
            var response = client.Execute<List<StudentParentAssociation_Readable>>(request);

            return response;
        }
        /// <summary>
        /// Retrieves resources matching values of an example resource (using the &quot;Get By Example&quot; pattern). This GET operation provides access to resources using the &quot;Get by Example&quot; search pattern.  The values of any properties of the resource that are specified will be used to return all matching results (if it exists).
        /// </summary>
        /// <param name="offset">Indicates how many items should be skipped before returning results.</param>
        /// <param name="limit">Indicates the maximum number of items that should be returned in the results (defaults to 25).</param>
        /// <param name="minChangeVersion">Used in synchronization to set sequence minimum ChangeVersion</param>
        /// <param name="maxChangeVersion">Used in synchronization to set sequence maximum ChangeVersion</param>
        /// <param name="parentUniqueId">A unique alphanumeric code assigned to a parent.</param>
        /// <param name="studentUniqueId">A unique alphanumeric code assigned to a student.</param>
        /// <param name="relationType">The nature of an individual's relationship to a student; for example:          Father, Mother, Step Father, Step Mother, Foster Father, Foster Mother, Guardian, etc.</param>
        /// <param name="primaryContactStatus">Indicator of whether the person is a primary parental contact for the Student.</param>
        /// <param name="livesWith">Indicator of whether the Student lives with the associated parent.</param>
        /// <param name="emergencyContactStatus">Indicator of whether the person is a designated emergency contact for the Student.</param>
        /// <param name="contactPriority">The numeric order of the preferred sequence or priority of contact.</param>
        /// <param name="contactRestrictions">Restrictions for student and/or teacher contact with the individual (e.g., the student may not be picked up by the individual).</param>
        /// <param name="id"></param>
        /// <param name="householdId">(MI EXT) Household identifier for Transportation records.</param>
        /// <param name="leaNotification">(MI EXT) Notification to the LEA where the toddler resides must occur at least 90 days prior to the toddler's third birthday for toddlers potentially eligible for Part B preschool services.</param>
        /// <param name="primaryPartBReferralContactStatus">(MI EXT) Indicator of whether the person is the primary parental contact for the student for the Part B Referral.</param>
        /// <param name="seaNotification">(MI EXT) Notification to the SEA must occur at least 90 days prior to the toddler's third birthday for toddlers potentially eligible for Part B preschool services.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<List<StudentParentAssociation_Readable>> GetStudentParentAssociationsByExample(int? offset= null, int? limit= null, int? minChangeVersion= null, int? maxChangeVersion= null, string parentUniqueId= null, string studentUniqueId= null, string relationType= null, bool? primaryContactStatus= null, bool? livesWith= null, bool? emergencyContactStatus= null, int? contactPriority= null, string contactRestrictions= null, string id= null, string householdId= null, bool? leaNotification= null, bool? primaryPartBReferralContactStatus= null, bool? seaNotification= null) 
        {
            var request = new RestRequest("/studentParentAssociations", Method.GET);
            request.RequestFormat = DataFormat.Json;

            if (offset != null)
                request.AddParameter("offset", offset);
            if (limit != null)
                request.AddParameter("limit", limit);
            if (minChangeVersion != null)
                request.AddParameter("minChangeVersion", minChangeVersion);
            if (maxChangeVersion != null)
                request.AddParameter("maxChangeVersion", maxChangeVersion);
            if (parentUniqueId != null)
                request.AddParameter("parentUniqueId", parentUniqueId);
            if (studentUniqueId != null)
                request.AddParameter("studentUniqueId", studentUniqueId);
            if (relationType != null)
                request.AddParameter("relationType", relationType);
            if (primaryContactStatus != null)
                request.AddParameter("primaryContactStatus", primaryContactStatus);
            if (livesWith != null)
                request.AddParameter("livesWith", livesWith);
            if (emergencyContactStatus != null)
                request.AddParameter("emergencyContactStatus", emergencyContactStatus);
            if (contactPriority != null)
                request.AddParameter("contactPriority", contactPriority);
            if (contactRestrictions != null)
                request.AddParameter("contactRestrictions", contactRestrictions);
            if (id != null)
                request.AddParameter("id", id);
            if (householdId != null)
                request.AddParameter("householdId", householdId);
            if (leaNotification != null)
                request.AddParameter("leaNotification", leaNotification);
            if (primaryPartBReferralContactStatus != null)
                request.AddParameter("primaryPartBReferralContactStatus", primaryPartBReferralContactStatus);
            if (seaNotification != null)
                request.AddParameter("seaNotification", seaNotification);
            request.AddHeader("Accept", "application/vnd.ed-fi.studentparentassociation.sis.readable+json");
            var response = client.Execute<List<StudentParentAssociation_Readable>>(request);

            return response;
        }
        /// <summary>
        /// Retrieves a specific resource using the values of the resource's natural key (using the &quot;Get By Key&quot; pattern). This GET operation provides access to resources using the &quot;Get by Key&quot; search pattern. The values of the natural key of the resource must be fully specified, and the service will return the matching result (if it exists).
        /// </summary>
        /// <param name="parentUniqueId">A unique alphanumeric code assigned to a parent.</param>
        /// <param name="studentUniqueId">A unique alphanumeric code assigned to a student.</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<StudentParentAssociation_Readable> GetStudentParentAssociationByKey(string parentUniqueId, string studentUniqueId, string IfNoneMatch= null) 
        {
            var request = new RestRequest("/studentParentAssociations", Method.GET);
            request.RequestFormat = DataFormat.Json;

            // verify required params are set
            if (parentUniqueId == null || studentUniqueId == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            if (parentUniqueId != null)
                request.AddParameter("parentUniqueId", parentUniqueId);
            if (studentUniqueId != null)
                request.AddParameter("studentUniqueId", studentUniqueId);
            request.AddHeader("If-None-Match", IfNoneMatch);
            request.AddHeader("Accept", "application/vnd.ed-fi.studentparentassociation.sis.readable+json");
            var response = client.Execute<StudentParentAssociation_Readable>(request);

            return response;
        }
        /// <summary>
        /// Creates or updates resources based on the natural key values of the supplied resource. The POST operation can be used to create or update resources. In database terms, this is often referred to as an &quot;upsert&quot; operation (insert + update).  Clients should NOT include the resource &quot;id&quot; in the JSON body because it will result in an error (you must use a PUT operation to update a resource by &quot;id&quot;). The web service will identify whether the resource already exists based on the natural key values provided, and update or create the resource appropriately.
        /// </summary>
        /// <param name="body">The JSON representation of the &quot;studentParentAssociation&quot; resource to be created or updated.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse PostStudentParentAssociations(StudentParentAssociation_Writable body) 
        {
            var request = new RestRequest("/studentParentAssociations", Method.POST);
            request.RequestFormat = DataFormat.Json;

            // verify required params are set
            if (body == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            request.AddBody(body);
            request.Parameters.First(param => param.Type == ParameterType.RequestBody).Name = "application/vnd.ed-fi.studentparentassociation.sis.writable+json";
            var response = client.Execute(request);

            var location = response.Headers.FirstOrDefault(x => x.Name == "Location");

            if (location != null && !string.IsNullOrWhiteSpace(location.Value.ToString()))
                body.id = location.Value.ToString().Split('/').Last();
            return response;
        }
        /// <summary>
        /// Retrieves a specific resource using the resource's identifier (using the &quot;Get By Id&quot; pattern). This GET operation retrieves a resource by the specified resource identifier.
        /// </summary>
        /// <param name="id">A resource identifier specifying the resource to be retrieved.</param>
        /// <param name="IfNoneMatch">The previously returned ETag header value, used here to prevent the unnecessary data transfer of an unchanged resource.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse<StudentParentAssociation_Readable> GetStudentParentAssociationsById(string id, string IfNoneMatch= null) 
        {
            var request = new RestRequest("/studentParentAssociations/{id}", Method.GET);
            request.RequestFormat = DataFormat.Json;

            request.AddUrlSegment("id", id);
            // verify required params are set
            if (id == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            request.AddHeader("If-None-Match", IfNoneMatch);
            request.AddHeader("Accept", "application/vnd.ed-fi.studentparentassociation.sis.readable+json");
            var response = client.Execute<StudentParentAssociation_Readable>(request);

            return response;
        }
        /// <summary>
        /// Updates or creates a resource based on the resource identifier. The PUT operation is used to update or create a resource by identifier.  If the resource doesn't exist, the resource will be created using that identifier.  Additionally, natural key values cannot be changed using this operation, and will not be modified in the database.  If the resource &quot;id&quot; is provided in the JSON body, it will be ignored as well.
        /// </summary>
        /// <param name="id">A resource identifier specifying the resource to be updated.</param>
        /// <param name="IfMatch">The ETag header value used to prevent the PUT from updating a resource modified by another consumer.</param>
        /// <param name="body">The JSON representation of the &quot;studentParentAssociation&quot; resource to be updated.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse PutStudentParentAssociation(string id, string IfMatch, StudentParentAssociation_Writable body) 
        {
            var request = new RestRequest("/studentParentAssociations/{id}", Method.PUT);
            request.RequestFormat = DataFormat.Json;

            request.AddUrlSegment("id", id);
            // verify required params are set
            if (id == null || body == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            request.AddHeader("If-Match", IfMatch);
            request.AddBody(body);
            request.Parameters.First(param => param.Type == ParameterType.RequestBody).Name = "application/vnd.ed-fi.studentparentassociation.sis.writable+json";
            var response = client.Execute(request);

            var location = response.Headers.FirstOrDefault(x => x.Name == "Location");

            if (location != null && !string.IsNullOrWhiteSpace(location.Value.ToString()))
                body.id = location.Value.ToString().Split('/').Last();
            return response;
        }
        /// <summary>
        /// Deletes an existing resource using the resource identifier. The DELETE operation is used to delete an existing resource by identifier.  If the resource doesn't exist, an error will result (the resource will not be found).
        /// </summary>
        /// <param name="id">A resource identifier specifying the resource to be deleted.</param>
        /// <param name="IfMatch">The ETag header value used to prevent the DELETE from removing a resource modified by another consumer.</param>
        /// <returns>A RestSharp <see cref="IRestResponse"/> instance containing the API response details.</returns>
        public IRestResponse DeleteStudentParentAssociationById(string id, string IfMatch= null) 
        {
            var request = new RestRequest("/studentParentAssociations/{id}", Method.DELETE);
            request.RequestFormat = DataFormat.Json;

            request.AddUrlSegment("id", id);
            // verify required params are set
            if (id == null ) 
               throw new ArgumentException("API method call is missing required parameters");
            request.AddHeader("If-Match", IfMatch);
            var response = client.Execute(request);

            return response;
        }
        // patch operation generation was suppressed because profiles can not support patch
    }
}

