﻿using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

namespace Michigan.Ods.Sdk.Profiles.SIS.Models 
{
    public class Cohort_Writable 
    {
        /// <summary>
        /// The unique identifier of the resource.
        /// </summary>
        public string id { get; set; }

        /// <summary>
        /// A reference to the related EducationOrganization resource.
        /// </summary>
        public EducationOrganizationReference educationOrganizationReference { get; set; }

        /// <summary>
        /// The academic subject associated with an academic intervention.
        /// </summary>
        public string academicSubjectDescriptor { get; set; }

        /// <summary>
        /// The description of the Cohort and its purpose.
        /// </summary>
        public string description { get; set; }

        /// <summary>
        /// The name or ID for the Cohort.
        /// </summary>
        public string identifier { get; set; }

        /// <summary>
        /// The scope of cohort (e.g., school, district, classroom).
        /// </summary>
        public string scopeType { get; set; }

        /// <summary>
        /// The type of cohort (e.g., academic intervention, classroom breakout).
        /// </summary>
        public string type { get; set; }

        /// <summary>
        /// (MI EXT) Field for use in researching or tracking student assessment information.
        /// </summary>
        public string otherResearchCode1 { get; set; }

        /// <summary>
        /// (MI EXT) Field for use in researching or tracking student assessment information.
        /// </summary>
        public string otherResearchCode2 { get; set; }

        /// <summary>
        /// An unordered collection of cohortAssessments.  (MI EXT) Provide user information to lookup and link to an existing assessment.
        /// </summary>
        public List<CohortAssessment_Writable> assessments { get; set; }

        /// <summary>
        /// An unordered collection of cohortPrograms.  The (optional) program associated with this Cohort (e.g., special education).
        /// </summary>
        public List<CohortProgram_Writable> programs { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        public string _etag { get; set; }

        }
}

