﻿using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

namespace Michigan.Ods.Sdk.Profiles.SIS.Models 
{
    public class EducationContent_Writable 
    {
        /// <summary>
        /// The unique identifier of the resource.
        /// </summary>
        public string id { get; set; }

        /// <summary>
        /// A reference to the related LearningStandard resource.
        /// </summary>
        public LearningStandardReference learningStandardReference { get; set; }

        /// <summary>
        /// Indicates whether there are additional un-named authors. In a research report, this is often marked by the abbreviation "et al".
        /// </summary>
        public bool? additionalAuthorsIndicator { get; set; }

        /// <summary>
        /// The predominate type or kind characterizing the learning resource.
        /// </summary>
        public string contentClassType { get; set; }

        /// <summary>
        /// A unique identifier for the EducationContent.
        /// </summary>
        public string contentIdentifier { get; set; }

        /// <summary>
        /// An amount that has to be paid or spent to buy or obtain the EducationContent.
        /// </summary>
        public double? cost { get; set; }

        /// <summary>
        /// The rate by which the cost applies.
        /// </summary>
        public string costRateType { get; set; }

        /// <summary>
        /// An extended written representation of the education content.
        /// </summary>
        public string description { get; set; }

        /// <summary>
        /// The predominate mode of learning supported by the learning resource. Acceptable values are active, expositive, or mixed.
        /// </summary>
        public string interactivityStyleType { get; set; }

        /// <summary>
        /// The URI (typical a URL) pointing to the metadata entry in a LRMI metadata repository, which describes this content item.
        /// </summary>
        public string learningResourceMetadataURI { get; set; }

        /// <summary>
        /// Namespace for the EducationContent.
        /// </summary>
        public string @namespace { get; set; }

        /// <summary>
        /// The date on which this content was first published.
        /// </summary>
        public DateTime? publicationDate { get; set; }

        /// <summary>
        /// The year at which this content was first published.
        /// </summary>
        public int? publicationYear { get; set; }

        /// <summary>
        /// The organization credited with publishing the resource.
        /// </summary>
        public string publisher { get; set; }

        /// <summary>
        /// A short description or name of the entity.
        /// </summary>
        public string shortDescription { get; set; }

        /// <summary>
        /// Approximate or typical time it takes to work with or through this learning resource for the typical intended target audience.
        /// </summary>
        public string timeRequired { get; set; }

        /// <summary>
        /// The URL where the owner specifies permissions for using the resource.
        /// </summary>
        public string useRightsURL { get; set; }

        /// <summary>
        /// The version identifier for the content.
        /// </summary>
        public string version { get; set; }

        /// <summary>
        /// An unordered collection of educationContentAppropriateGradeLevels.  Grade levels for which this education content is applicable-if omitted, considered generally applicable.
        /// </summary>
        public List<EducationContentAppropriateGradeLevel_Writable> appropriateGradeLevels { get; set; }

        /// <summary>
        /// An unordered collection of educationContentAppropriateSexes.  Sexes for which this education content is applicable. If omitted, considered generally applicable.
        /// </summary>
        public List<EducationContentAppropriateSex_Writable> appropriateSexes { get; set; }

        /// <summary>
        /// An unordered collection of educationContentAuthors.  The individual credited with the creation of the resource.
        /// </summary>
        public List<EducationContentAuthor_Writable> authors { get; set; }

        /// <summary>
        /// An unordered collection of educationContentDerivativeSourceEducationContents.  Relates the education content source to the education content.
        /// </summary>
        public List<EducationContentDerivativeSourceEducationContent_Writable> derivativeSourceEducationContents { get; set; }

        /// <summary>
        /// An unordered collection of educationContentDerivativeSourceLearningResourceMetadataURIs.  The URI (typical a URL) pointing to the metadata entry in a LRMI metadata repository, which describes this content item.
        /// </summary>
        public List<EducationContentDerivativeSourceLearningResourceMetadataURI_Writable> derivativeSourceLearningResourceMetadataURIs { get; set; }

        /// <summary>
        /// An unordered collection of educationContentDerivativeSourceURIs.  The URI (typical a URL) pointing to an education content item.
        /// </summary>
        public List<EducationContentDerivativeSourceURI_Writable> derivativeSourceURIs { get; set; }

        /// <summary>
        /// An unordered collection of educationContentLanguages.  An indication of the languages in which the Education Content is designed.
        /// </summary>
        public List<EducationContentLanguage_Writable> languages { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        public string _etag { get; set; }

        }
}

