﻿using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

namespace Michigan.Ods.Sdk.Profiles.SIS.Models 
{
    public class Expenditure_Readable 
    {
        /// <summary>
        /// The unique identifier of the resource.
        /// </summary>
        public string id { get; set; }

        /// <summary>
        /// A reference to the related EducationOrganization resource.
        /// </summary>
        public EducationOrganizationReference educationOrganizationReference { get; set; }

        /// <summary>
        /// A reference to the related School resource.
        /// </summary>
        public SchoolReference schoolReference { get; set; }

        /// <summary>
        /// Amount for the expenditure submitted.
        /// </summary>
        public double? amount { get; set; }

        /// <summary>
        /// A dimension available to further define a school district's activities.
        /// </summary>
        public string otherOptional { get; set; }

        /// <summary>
        /// Denotes the fiscal year for which the expenditure is submitted.
        /// </summary>
        public int? fiscalYear { get; set; }

        /// <summary>
        /// To indicate the basic account identifier for expenditures. It describes the activity for which a service or material is acquired.
        /// </summary>
        public string functionCodeDescriptor { get; set; }

        /// <summary>
        /// The independent fisal and accounting entity for expenditure transaction.
        /// </summary>
        public string fundCodeDescriptor { get; set; }

        /// <summary>
        /// To indicate various grant programs/funding source.
        /// </summary>
        public string grantCodeDescriptor { get; set; }

        /// <summary>
        /// To indicate the service or commodity obtained as the result of a specific expenditure.
        /// </summary>
        public string objectCodeDescriptor { get; set; }

        /// <summary>
        /// To indicate a plan of activities and procedures designed to accomplish predetermined objective or set of allied objectives.
        /// </summary>
        public string programCodeDescriptor { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        public string _etag { get; set; }

        }
}

