﻿using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

namespace Michigan.Ods.Sdk.Profiles.SIS.Models 
{
    public class GradebookEntry_Writable 
    {
        /// <summary>
        /// The unique identifier of the resource.
        /// </summary>
        public string id { get; set; }

        /// <summary>
        /// A reference to the related GradingPeriod resource.
        /// </summary>
        public GradingPeriodReference gradingPeriodReference { get; set; }

        /// <summary>
        /// A reference to the related Section resource.
        /// </summary>
        public SectionReference sectionReference { get; set; }

        /// <summary>
        /// The date the assignment, homework, or assessment was assigned or executed.
        /// </summary>
        public DateTime? dateAssigned { get; set; }

        /// <summary>
        /// A description of the assignment, homework, or classroom assessment.
        /// </summary>
        public string description { get; set; }

        /// <summary>
        /// (MI EXT) Indicates due date (OneRoster lineitems.dueDate) of the GradebookEntry.
        /// </summary>
        public DateTime? dueDate { get; set; }

        /// <summary>
        /// The name or title of the activity to be recorded in the GradebookEntry.
        /// </summary>
        public string title { get; set; }

        /// <summary>
        /// The type of the GradebookEntry; for example, homework, assignment, quiz, unit test, oral presentation, etc.
        /// </summary>
        public string type { get; set; }

        /// <summary>
        /// (MI EXT) The maximum value permitted for the score (inclusive) e.g. 100.0
        /// </summary>
        public double? resultValueMax { get; set; }

        /// <summary>
        /// (MI EXT) The minimum value permitted for the score (inclusive) e.g. 0.0.
        /// </summary>
        public double? resultValueMin { get; set; }

        /// <summary>
        /// (MI EXT) Indicates status (OneRoster lineitems.status) of the GradebookEntry.
        /// </summary>
        public string statusDescriptor { get; set; }

        /// <summary>
        /// An unordered collection of gradebookEntryLearningObjectives.  LearningObjectives associated with the GradebookEntry.
        /// </summary>
        public List<GradebookEntryLearningObjective_Writable> learningObjectives { get; set; }

        /// <summary>
        /// An unordered collection of gradebookEntryLearningStandards.  LearningStandard(s) associated with the GradebookEntry.
        /// </summary>
        public List<GradebookEntryLearningStandard_Writable> learningStandards { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        public string _etag { get; set; }

        }
}

