﻿using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

namespace Michigan.Ods.Sdk.Profiles.SIS.Models 
{
    public class InterventionStudy_Readable 
    {
        /// <summary>
        /// The unique identifier of the resource.
        /// </summary>
        public string id { get; set; }

        /// <summary>
        /// A reference to the related EducationOrganization resource.
        /// </summary>
        public EducationOrganizationReference educationOrganizationReference { get; set; }

        /// <summary>
        /// A reference to the related InterventionPrescription resource.
        /// </summary>
        public InterventionPrescriptionReference interventionPrescriptionReference { get; set; }

        /// <summary>
        /// The way in which an intervention was implemented: individual, small group, whole class, or whole school.
        /// </summary>
        public string deliveryMethodType { get; set; }

        /// <summary>
        /// The way in which an intervention is used: curriculum, supplement, or practice.
        /// </summary>
        public string interventionClassType { get; set; }

        /// <summary>
        /// A unique number or alphanumeric code assigned to an intervention study.
        /// </summary>
        public string identificationCode { get; set; }

        /// <summary>
        /// The number of participants observed in the study.
        /// </summary>
        public int? participants { get; set; }

        /// <summary>
        /// An unordered collection of interventionStudyAppropriateGradeLevels.  Grade levels participating in this study.
        /// </summary>
        public List<InterventionStudyAppropriateGradeLevel_Readable> appropriateGradeLevels { get; set; }

        /// <summary>
        /// An unordered collection of interventionStudyAppropriateSexes.  Sexes participating in this study. If omitted, considered generally applicable.
        /// </summary>
        public List<InterventionStudyAppropriateSex_Readable> appropriateSexes { get; set; }

        /// <summary>
        /// An unordered collection of interventionStudyEducationContents.  Relates the education content source to the education content.
        /// </summary>
        public List<InterventionStudyEducationContent_Readable> educationContents { get; set; }

        /// <summary>
        /// An unordered collection of interventionStudyInterventionEffectivenesses.  Measurement of the effectiveness of the InterventionStudy per diagnosis.
        /// </summary>
        public List<InterventionStudyInterventionEffectiveness_Readable> interventionEffectivenesses { get; set; }

        /// <summary>
        /// An unordered collection of interventionStudyLearningResourceMetadataURIs.  The URI (typical a URL) pointing to the metadata entry in a LRMI metadata repository, which describes this content item.
        /// </summary>
        public List<InterventionStudyLearningResourceMetadataURI_Readable> learningResourceMetadataURIs { get; set; }

        /// <summary>
        /// An unordered collection of interventionStudyPopulationServeds.  A subset of students that are the focus of the InterventionStudy.
        /// </summary>
        public List<InterventionStudyPopulationServed_Readable> populationServeds { get; set; }

        /// <summary>
        /// An unordered collection of interventionStudyStateAbbreviations.  The abbreviation for the state (within the United States) or outlying area, the school system of which the participants of the study are considered to be a part.
        /// </summary>
        public List<InterventionStudyStateAbbreviation_Readable> stateAbbreviations { get; set; }

        /// <summary>
        /// An unordered collection of interventionStudyURIs.  The URI (typical a URL) pointing to an education content item.
        /// </summary>
        public List<InterventionStudyURI_Readable> uris { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        public string _etag { get; set; }

        }
}

