﻿using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

namespace Michigan.Ods.Sdk.Profiles.SIS.Models 
{
    public class LearningObjective_Readable 
    {
        /// <summary>
        /// The unique identifier of the resource.
        /// </summary>
        public string id { get; set; }

        /// <summary>
        /// A reference to the related LearningObjective resource.
        /// </summary>
        public LearningObjectiveReference parentLearningObjectiveReference { get; set; }

        /// <summary>
        /// The description of the content or subject area (e.g., arts, mathematics, reading, stenography, or a foreign language) of an assessment.
        /// </summary>
        public string academicSubjectDescriptor { get; set; }

        /// <summary>
        /// The description of the LearningObjective.
        /// </summary>
        public string description { get; set; }

        /// <summary>
        /// The identifier for the specific learning objective in the context of a standard (e.g., 111.15.3.1.A).
        /// </summary>
        public string learningObjectiveId { get; set; }

        /// <summary>
        /// Namespace for the LearningObjective.
        /// </summary>
        public string @namespace { get; set; }

        /// <summary>
        /// Reflects the specific nomenclature used for the LearningObjective.
        /// </summary>
        public string nomenclature { get; set; }

        /// <summary>
        /// The designated title of the LearningObjective.
        /// </summary>
        public string objective { get; set; }

        /// <summary>
        /// The grade level for which the LearningObjective is targeted.
        /// </summary>
        public string objectiveGradeLevelDescriptor { get; set; }

        /// <summary>
        /// One or more statements that describes the criteria used by teachers and students to check for attainment of a learning objective. This criteria gives clear indications as to the degree to which learning is moving through the Zone or Proximal Development toward independent achievement of the LearningObjective.
        /// </summary>
        public string successCriteria { get; set; }

        /// <summary>
        /// A reference to the type of content standard (e.g., state, national)
        /// </summary>
        public LearningObjectiveContentStandard_Readable contentStandard { get; set; }

        /// <summary>
        /// An unordered collection of learningObjectiveLearningStandards.  LearningStandard(s) included in this objective.
        /// </summary>
        public List<LearningObjectiveLearningStandard_Readable> learningStandards { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        public string _etag { get; set; }

        }
}

