﻿using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

namespace Michigan.Ods.Sdk.Profiles.SIS.Models 
{
    public class ObjectiveAssessment_Readable 
    {
        /// <summary>
        /// The unique identifier of the resource.
        /// </summary>
        public string id { get; set; }

        /// <summary>
        /// A reference to the related Assessment resource.
        /// </summary>
        public AssessmentReference assessmentReference { get; set; }

        /// <summary>
        /// A reference to the related ObjectiveAssessment resource.
        /// </summary>
        public ObjectiveAssessmentReference parentObjectiveAssessmentReference { get; set; }

        /// <summary>
        /// The description of the ObjectiveAssessment (e.g., vocabulary, measurement, or geometry).
        /// </summary>
        public string description { get; set; }

        /// <summary>
        /// A unique number or alphanumeric code assigned to a space, room, site, building, individual, organization, program, or institution by a school, school system, a state, or other agency or entity.
        /// </summary>
        public string identificationCode { get; set; }

        /// <summary>
        /// The maximum raw score achievable across all assessment items that are correct and scored at the maximum.
        /// </summary>
        public int? maxRawScore { get; set; }

        /// <summary>
        /// Reflects the specific nomenclature used for this level of ObjectiveAssessment.
        /// </summary>
        public string nomenclature { get; set; }

        /// <summary>
        /// The percentage of the Assessment that tests this objective.
        /// </summary>
        public double? percentOfAssessment { get; set; }

        /// <summary>
        /// (MI EXT) Indicates the level (e.g. Level 1, Level 2).
        /// </summary>
        public string reportingLevelDescriptor { get; set; }

        /// <summary>
        /// (MI EXT) The label that is used for reporting level.
        /// </summary>
        public string reportingLevelLabel { get; set; }

        /// <summary>
        /// (MI EXT) The short description of reporting level.
        /// </summary>
        public string reportingLevelShortDescription { get; set; }

        /// <summary>
        /// An unordered collection of objectiveAssessmentAssessmentItems.  References individual test items, if appropriate.
        /// </summary>
        public List<ObjectiveAssessmentAssessmentItem_Readable> assessmentItems { get; set; }

        /// <summary>
        /// An unordered collection of objectiveAssessmentLearningObjectives.  References the LearningObjective(s) the ObjectiveAssessment tests.
        /// </summary>
        public List<ObjectiveAssessmentLearningObjective_Readable> learningObjectives { get; set; }

        /// <summary>
        /// An unordered collection of objectiveAssessmentLearningStandards.  LearningStandard tested by this ObjectiveAssessment.
        /// </summary>
        public List<ObjectiveAssessmentLearningStandard_Readable> learningStandards { get; set; }

        /// <summary>
        /// An unordered collection of objectiveAssessmentPerformanceLevels.  Definition of the performance levels and the associated cut scores. Three styles are supported:          1. Specification of performance level by minimum and maximum score              2. Specification of performance level by cut score, using only minimum score              3. Specification of performance level without any mapping to scores.
        /// </summary>
        public List<ObjectiveAssessmentPerformanceLevel_Readable> performanceLevels { get; set; }

        /// <summary>
        /// An unordered collection of objectiveAssessmentScores.  Definition of the scores to be expected from this objective assessment.
        /// </summary>
        public List<ObjectiveAssessmentScore_Readable> scores { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        public string _etag { get; set; }

        }
}

