﻿using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

namespace Michigan.Ods.Sdk.Profiles.SIS.Models 
{
    public class StaffEducationOrganizationEmploymentAssociation_Readable 
    {
        /// <summary>
        /// The unique identifier of the resource.
        /// </summary>
        public string id { get; set; }

        /// <summary>
        /// A reference to the related EducationOrganization resource.
        /// </summary>
        public EducationOrganizationReference educationOrganizationReference { get; set; }

        /// <summary>
        /// A reference to the related Staff resource.
        /// </summary>
        public StaffReference staffReference { get; set; }

        /// <summary>
        /// (MI EXT) The appropriate accounting/function code for the employee as determined for accounting purposes by the school district.
        /// </summary>
        public string accountingFunctionCodeDescriptor { get; set; }

        /// <summary>
        /// The department or suborganization the employee/contractor is associated with in the EducationOrganization.
        /// </summary>
        public string department { get; set; }

        /// <summary>
        /// Reflects the type of employment or contract; for example:          Probationary          Contractual          Substitute/temporary          Tenured or permanent          Volunteer/no contract          ...
        /// </summary>
        public string employmentStatusDescriptor { get; set; }

        /// <summary>
        /// The month, day, and year on which a contract between an individual and a governing authority ends or is terminated under the provisions of the contract (or the date on which the agreement is made invalid).
        /// </summary>
        public DateTime? endDate { get; set; }

        /// <summary>
        /// The ratio between the hours of work expected in a position and the hours of work normally expected in a full-time position in the same setting.
        /// </summary>
        public double? fullTimeEquivalency { get; set; }

        /// <summary>
        /// The month, day, and year on which an individual was hired for a position.
        /// </summary>
        public DateTime? hireDate { get; set; }

        /// <summary>
        /// Hourly wage associated with the employment position being reported.
        /// </summary>
        public double? hourlyWage { get; set; }

        /// <summary>
        /// Date at which the staff member was made an official offer for this employment.
        /// </summary>
        public DateTime? offerDate { get; set; }

        /// <summary>
        /// (MI EXT) The full-time base annual salary for the employee is the base salary (basic contract) for a first-year full-time employee at this educational level. It does not include increments for years of service or longevity paymensts.
        /// </summary>
        public int? repFullTimeBaseAnnualSalary { get; set; }

        /// <summary>
        /// Reason for terminating the employment; for example:          Employment in education          Employment outside of education          Retirement          Family/personal relocation          Change of assignment.
        /// </summary>
        public string separationReasonDescriptor { get; set; }

        /// <summary>
        /// Type of employment separation; for example: Voluntary separation, Involuntary separation,          Mutual agreement          Other, etc.
        /// </summary>
        public string separationType { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        public string _etag { get; set; }

        }
}

