﻿using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

namespace Michigan.Ods.Sdk.Profiles.SIS.Models 
{
    public class StudentEarlyLearningProgramAssociation_Writable 
    {
        /// <summary>
        /// The unique identifier of the resource.
        /// </summary>
        public string id { get; set; }

        /// <summary>
        /// A reference to the related EducationOrganization resource.
        /// </summary>
        public EducationOrganizationReference educationOrganizationReference { get; set; }

        /// <summary>
        /// A reference to the related Program resource.
        /// </summary>
        public ProgramReference programReference { get; set; }

        /// <summary>
        /// A reference to the related Student resource.
        /// </summary>
        public StudentReference studentReference { get; set; }

        /// <summary>
        /// The month, day, and year on which the Student first received services.
        /// </summary>
        public DateTime? beginDate { get; set; }

        /// <summary>
        /// The primary location where the student attends the reported program or receives early childhood services.
        /// </summary>
        public string deliveryMethodDescriptor { get; set; }

        /// <summary>
        /// The primary schedule followed by the student for this program or service.
        /// </summary>
        public string deliveryScheduleDescriptor { get; set; }

        /// <summary>
        /// A field to provide users with the option of adding a comment to distinguish a child for their own records. This field is optional and is not required to be completed.
        /// </summary>
        public string ecComment { get; set; }

        /// <summary>
        /// The month, day, and year on which the Student exited the Program or stopped receiving services.
        /// </summary>
        public DateTime? endDate { get; set; }

        /// <summary>
        /// Added for ELI enrollment status.
        /// </summary>
        public string enrollmentStatus { get; set; }

        /// <summary>
        /// The Federal Poverty Level (FPL) quintile that a child belongs to.
        /// </summary>
        public string federalPovertyLevelDescriptor { get; set; }

        /// <summary>
        /// The license number assigned to the location a child receives early childhood services. Licensed providers are assigned an 11-digit code (2 letters, 9 numbers) by the Michigan Department of Licensing and Regulatory Affairs. Tribal programs not licensed by the State are assigned a 14-digit code (7 letters, 7 numbers) by the Early Childhood Investment Corporation.
        /// </summary>
        public string providerLicenseNumber { get; set; }

        /// <summary>
        /// The reason the child left the Program within a school or district.
        /// </summary>
        public string reasonExitedDescriptor { get; set; }

        /// <summary>
        /// Indicates whether the Student received services during the summer session or between sessions.
        /// </summary>
        public bool? servedOutsideOfRegularSession { get; set; }

        /// <summary>
        /// 1 = parent(s) are working and unavailable to care for the child during program hours. This could include a parent attending school or working a night shift requiring sleep during the day.  0 = the parents are not working and available to provide care during program hours.
        /// </summary>
        public bool? workingParents { get; set; }

        /// <summary>
        /// The Individualized Family Service Plan (IPSP) for special services for young students with developmental delays.
        /// </summary>
        public StudentEarlyLearningProgramAssociationIndividualizedFamilyServicePlan_Writable individualizedFamilyServicePlan { get; set; }

        /// <summary>
        /// An unordered collection of studentEarlyLearningProgramAssociationAdditionalEligibilityFactors.  Additional factors used to determine a child s eligibility for a Great Start Readiness Program, along with income information collected in the Federal Poverty Level Quintile (FPLQ) characteristic. For more information, reference the MDE GSRP Implementation Manual.
        /// </summary>
        public List<StudentEarlyLearningProgramAssociationAdditionalEligibilityFactor_Writable> additionalEligibilityFactors { get; set; }

        /// <summary>
        /// An unordered collection of studentEarlyLearningProgramAssociationEarlyLearningProgramServices.  The student's program service information for early learning program.
        /// </summary>
        public List<StudentEarlyLearningProgramAssociationEarlyLearningProgramService_Writable> earlyLearningProgramServices { get; set; }

        /// <summary>
        /// An unordered collection of studentEarlyLearningProgramAssociationEligibilityReasons.  Describes the reason why a child was found eligible for Early On. Providers may report at least one code and may report up to 22 codes per record.
        /// </summary>
        public List<StudentEarlyLearningProgramAssociationEligibilityReason_Writable> eligibilityReasons { get; set; }

        /// <summary>
        /// An unordered collection of studentEarlyLearningProgramAssociationQualifyingFactors.  Factors considered in determining the child s eligibility for a Great Start Readiness Program. Qualifying factors are those which automatically prioritize a child for enrollment. For more information, reference the MDE GSRP Implementation Manual.
        /// </summary>
        public List<StudentEarlyLearningProgramAssociationQualifyingFactor_Writable> qualifyingFactors { get; set; }

        /// <summary>
        /// An unordered collection of studentProgramAssociationServices.  Indicates the Service(s) being provided to the Student by the Program.
        /// </summary>
        public List<StudentProgramAssociationService_StudentEarlyLearningProgramAssociation_Writable> services { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        public string _etag { get; set; }

        }
}

