﻿using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

namespace Michigan.Ods.Sdk.Profiles.TransportationProfile.Models 
{
    public class StudentSchoolAssociation_Readable 
    {
        /// <summary>
        /// The unique identifier of the resource.
        /// </summary>
        public string id { get; set; }

        /// <summary>
        /// A reference to the related School resource.
        /// </summary>
        public SchoolReference schoolReference { get; set; }

        /// <summary>
        /// A reference to the related SchoolYearType resource.
        /// </summary>
        public SchoolYearTypeReference schoolYearTypeReference { get; set; }

        /// <summary>
        /// A reference to the related Student resource.
        /// </summary>
        public StudentReference studentReference { get; set; }

        /// <summary>
        /// (MI EXT) The date the student was first enrolled in the District.
        /// </summary>
        public DateTime? districtEntryDate { get; set; }

        /// <summary>
        /// (MI EXT) To indicate the date of the last change to district residency status.
        /// </summary>
        public DateTime? districtResidencyStatusAsOfDate { get; set; }

        /// <summary>
        /// (MI EXT) To indicate the student s residency related to the geographic boundaries of the district and/or residency requirements outlined in the Pupil Residency section of the MDE Pupil Accounting Manual.
        /// </summary>
        public string districtResidencyStatusDescriptor { get; set; }

        /// <summary>
        /// (Mi EXT) The pro rata share for which the student receives educational programs and/or services from the operating district.
        /// </summary>
        public double? enrollmentPercent { get; set; }

        /// <summary>
        /// The month, day, and year on which an individual enters and begins to receive instructional services in a school.
        /// </summary>
        public DateTime? entryDate { get; set; }

        /// <summary>
        /// The grade level or primary instructional level at which a student enters and receives services in a school or an educational institution during a given academic session.
        /// </summary>
        public string entryGradeLevelDescriptor { get; set; }

        /// <summary>
        /// The month, day, and year of the first day after the date of an individual's last attendance at a school (if known), the day on which an individual graduated, or the date on which it becomes known officially that an individual left school.
        /// </summary>
        public DateTime? exitWithdrawDate { get; set; }

        /// <summary>
        /// (MI EXT) To support state reporting: captures when ExitWithdrawDate changed.
        /// </summary>
        public DateTime? exitWithdrawDateAsOfDate { get; set; }

        /// <summary>
        /// Indicates if a given enrollment record should be considered the primary record for a student. If omitted, the default is true.
        /// </summary>
        public bool? primarySchool { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        public string _etag { get; set; }

        }
}

