﻿using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

namespace Michigan.Ods.Sdk.Profiles.TransportationProfile.Models 
{
    public class Student_Writable 
    {
        /// <summary>
        /// The unique identifier of the resource.
        /// </summary>
        public string id { get; set; }

        /// <summary>
        /// (MI-EXT) The date of the most recent changes to Address.
        /// </summary>
        public DateTime? addressAsOfDate { get; set; }

        /// <summary>
        /// (MI EXT) A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table.
        /// </summary>
        public string foodServiceStatusDescriptor { get; set; }

        /// <summary>
        /// A unique identifier used as Primary Key, not derived from business logic, when acting as Foreign Key, references the parent table.
        /// </summary>
        public string homelessDescriptor { get; set; }

        /// <summary>
        /// (MI EXT) Medical notes such as "Allergic to Bee Stings" - For Transportation records.
        /// </summary>
        public string medicalNotes { get; set; }

        /// <summary>
        /// (MI-EXT) The date of the most recent change in Race.
        /// </summary>
        public DateTime? raceAsOfDate { get; set; }

        /// <summary>
        /// (MI-EXT) The date of the most recent change in SchoolFoodServicesEligibility.
        /// </summary>
        public DateTime? schoolFoodServicesEligibilityAsOfDate { get; set; }

        /// <summary>
        /// (MI EXT) Indicates status (OneRoster users.status) of the Student.
        /// </summary>
        public string statusDescriptor { get; set; }

        /// <summary>
        /// A unique alphanumeric code assigned to a student.
        /// </summary>
        public string studentUniqueId { get; set; }

        /// <summary>
        /// (MI EXT) Indicates the Student username (OneRoster users.username)
        /// </summary>
        public string userName { get; set; }

        /// <summary>
        /// (MI-EXT) The four-digit calendar year in which the student first attended school in the United States.
        /// </summary>
        public int? yearOfEntry { get; set; }

        /// <summary>
        /// An unordered collection of studentAddresses.  The set of elements that describes an address, including the street address, city, state, and ZIP code.
        /// </summary>
        public List<StudentAddress_Writable> addresses { get; set; }

        /// <summary>
        /// A unique system-generated value that identifies the version of the resource.
        /// </summary>
        public string _etag { get; set; }

        }
}

